SystemOrganization addCategory: #'Pythia-AST-Tests'!
SystemOrganization addCategory: #'Pythia-Tests'!
SystemOrganization addCategory: #'Pythia-Analysis-Tests'!
SystemOrganization addCategory: #'Pythia-Analysis'!
SystemOrganization addCategory: #'Pythia-Parser'!
SystemOrganization addCategory: #'Pythia-AST'!

!SequenceableCollection methodsFor: '*Pythia-Parser-converting' stamp: 'fbs 12/9/2008 21:48'!
asSexpr
	| s |
	s := WriteStream on: ''.
	s nextPut: $(.
	
	self do:
		[:each |
		(each isCollection)
			ifTrue: [s nextPutAll: each asSexpr]
			ifFalse: [each printOn: s].
			
		(each == self last) ifFalse: [s nextPut: Character space]].
	s nextPut: $).
	
	^ s contents asString.! !

OMeta subclass: #PyParser
	instanceVariableNames: 'factory'
	classVariableNames: 'Keywords FunctionDirectives CallingConventions HintingDirectives'
	poolDictionaries: ''
	category: 'Pythia-Parser'!

!PyParser commentStamp: 'fbs 11/24/2008 21:47' prior: 0!
A PyParser is a compiler of Delphi source code. As it parses it input stream, it builds an Abstract Syntax Tree (AST) using a PyNodeFactory (accessed via #factory).!

!PyParser class methodsFor: 'class initialization' stamp: 'fbs 5/18/2009 14:45'!
initialize
	super initialize.

	CallingConventions := Set withAll: #(cdecl export far local near pascal register safecall stdcall).
	
	FunctionDirectives := Set withAll: #(overload inline resident unsafe).

	HintingDirectives := Set withAll: #(deprecated experimental platform library).

	Keywords := Set withAll: #(+ - * / abstract array as asm at begin case class const constructor destructor dispinterface div do downto else end except exports #false file final finalization finally for function goto if implementation inherited initialization inline interface in label library mod #nil not object out of on operator packed procedure program property raise record repeat sealed set shl shr then threadvar to #true try type unit unsafe until uses var while with).! !

!PyParser class methodsFor: 'instance creation' stamp: 'fbs 4/21/2009 10:25'!
parseFileNamed: aString
	^ self parseFileNamed: aString using: PySexprNodeFactory new.! !

!PyParser class methodsFor: 'instance creation' stamp: 'fbs 4/21/2009 10:24'!
parseFileNamed: aString using: aNodeFactory
	| f ast p |
	[f := FileStream readOnlyFileNamed: aString.
	p := self on: f.
	p factory: aNodeFactory.
	ast := p apply: #file.
	] ensure: [f ifNotNil: [f close]].
	^ ast! !

!PyParser methodsFor: 'directives' stamp: 'fbs 12/11/2008 15:16'!
absoluteDirective ::=
	<symbol 'absolute'>! !

!PyParser methodsFor: 'directives' stamp: 'fbs 1/7/2009 09:51'!
abstractDirective ::=
	<directive 'abstract'>
	| <finalKeyword>! !

!PyParser methodsFor: 'directives' stamp: 'fbs 1/3/2009 14:51'!
addDirective ::=
	<token 'add'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 11/30/2008 20:59'!
addOp ::=
	<symbol '+'>
	| <symbol '-'>
	| <symbol 'or'>
	| <symbol 'xor'>! !

!PyParser methodsFor: 'expressions' stamp: 'fbs 1/9/2009 22:01'!
addressOf ::=
	<at> <factor>:f => [self factory buildAddressOf: f]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/11/2008 15:15'!
almostReservedWord ::=
	<atKeyword>
	| <onKeyword>! !

!PyParser methodsFor: 'assembly' stamp: 'fbs 11/10/2008 22:19'!
amdEnhancedOpcode ::=
	<ident>! !

!PyParser methodsFor: 'assembly' stamp: 'fbs 11/10/2008 22:19'!
amdOpcode ::=
	<ident>! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/7/2009 15:06'!
arrayIndex ::=
	<expression>:start <doubleDot> <expression>:end => [self factory buildRangeFrom: start to: end]
	| <typeID>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 11/30/2008 21:10'!
arrayKeyword ::=
	<symbol 'array'>! !

!PyParser methodsFor: 'types' stamp: 'fbs 12/28/2008 23:26'!
arraySubType ::=
	<constKeyword> | <type>! !

!PyParser methodsFor: 'types' stamp: 'fbs 12/28/2008 23:28'!
arrayType ::=
	<arrayKeyword> ((<openSqBracket> <commaListOf #arrayIndex>:i <closeSqBracket> => [i]) | <empty> !![nil]):indices <ofKeyword> <arraySubType>:elementType => [self factory buildArrayType: elementType withIndices: indices]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/12/2008 13:30'!
asKeyword ::=
	<symbol 'as'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 11/30/2008 21:15'!
asmKeyword ::=
	<symbol 'asm'>! !

!PyParser methodsFor: 'directives' stamp: 'fbs 12/11/2008 15:17'!
assemblerDirective ::=
	<token 'assembler'>! !

!PyParser methodsFor: 'to be processed' stamp: 'fbs 1/9/2009 17:31'!
assemblerStatement ::=
	<asmKeyword> <spaces> <delimitedListOf #lineEnd #assemblyStatement>*:s <endKeyword> => [self factory buildAsmBlock: s]! !

!PyParser methodsFor: 'to be processed' stamp: 'fbs 12/12/2008 13:29'!
assemblyAttribute ::=
	<openSqBracket> <assemblyDirective> <colon> <customAttributeDecl> <closeSqBracket>! !

!PyParser methodsFor: 'directives' stamp: 'fbs 11/30/2008 21:04'!
assemblyDirective ::=
	<token 'db'>
	| <token 'dw'>
	| <token 'dd'>
	| <token 'dq'>! !

!PyParser methodsFor: 'to be processed' stamp: 'fbs 11/10/2008 22:20'!
assemblyLabel ::=
	<label>
	| <localLabelPart>+! !

!PyParser methodsFor: 'to be processed' stamp: 'fbs 1/9/2009 06:51'!
assemblyLabelMarker ::=
	<assemblyLabel>:l <colon> => [l]! !

!PyParser methodsFor: 'to be processed' stamp: 'fbs 11/11/2008 15:51'!
assemblyOpcode ::=
	<pentium1Opcode>
	| <pentium2Opcode>
	| <pentium3Opcode>
	| <pentium4Opcode>
	| <amdOpcode>
	| <amdEnhancedOpcode>
	| <optional #ident> <assemblyDirective>! !

!PyParser methodsFor: 'to be processed' stamp: 'fbs 1/9/2009 16:51'!
assemblyOperand ::=
	<expression>
	| <localLabelPart>! !

!PyParser methodsFor: 'to be processed' stamp: 'fbs 12/10/2008 10:49'!
assemblyOperandList ::=
	<assemblyOperand> (<comma> <assemblyOperand>)*! !

!PyParser methodsFor: 'to be processed' stamp: 'fbs 11/10/2008 22:23'!
assemblyPrefix ::=
	<ident>! !

!PyParser methodsFor: 'to be processed' stamp: 'fbs 1/9/2009 16:57'!
assemblyStatement ::=
	<optional #assemblyLabelMarker> <assemblyOpcode> <optional #assemblyOperandList>
	| <assemblyLabelMarker>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/14/2008 22:41'!
assign ::=
	<token ':='>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 5/7/2009 13:59'!
at ::=
	<atSign> ~<atSign>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/14/2008 22:41'!
atAt ::=
	<token '@@'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 11/30/2008 21:17'!
atKeyword ::=
	<symbol 'at'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/30/2008 12:16'!
atSign ::=
	<token '@'>! !

!PyParser methodsFor: 'directives' stamp: 'fbs 12/11/2008 15:18'!
automatedDirective ::=
	<symbol 'automated'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 11/30/2008 21:15'!
beginKeyword ::=
	<symbol 'begin'>! !

!PyParser methodsFor: 'directives' stamp: 'fbs 1/7/2009 09:40'!
bindingDirective ::=
	<token 'message'>:m <expression>:e => [self factory buildDirective: m withValue: e]
	| (<staticDirective> | <dynamicDirective> | <overrideDirective> | <virtualDirective>)! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/30/2008 20:06'!
block ::=
	<declSection>*:decls <optional #blockBody>:body => [self factory buildBlock: body withDeclarations: decls]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 11/10/2008 20:39'!
blockBody ::=
	<compoundStatement>
	| <assemblerStatement>! !

!PyParser methodsFor: 'directives' stamp: 'fbs 5/8/2009 15:35'!
callConvention ::=
	<identifier>:d ?[self callingConvention: d] => [self factory buildDirective: d]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 11/30/2008 21:17'!
callerAddress ::=
	<atKeyword> <designator>! !

!PyParser methodsFor: 'testing' stamp: 'fbs 5/8/2009 08:55'!
callingConvention: aString
	^ CallingConventions includes: aString.! !

!PyParser methodsFor: 'statements' stamp: 'fbs 12/30/2008 13:39'!
caseItem ::=
	<commaListOf #caseLabel>:set <colon> <optional #statement>:s => [self factory buildCaseSwitchOn: set do: (s ifNil: [#()])]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 11/30/2008 21:16'!
caseKeyword ::=
	<symbol 'case'>! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/28/2008 23:26'!
caseLabel ::=
	<simpleTypeSubRange>! !

!PyParser methodsFor: 'statements' stamp: 'fbs 5/16/2009 11:08'!
caseStatement ::=
	<caseKeyword> <expression>:switcher <ofKeyword>
	<semicolonListOf #caseItem>:items
	<optional #semicolon>
	<optional #elseBlock>:elseClause <endKeyword> => [self factory buildCaseStatement: switcher withSwitches: items orElse: (elseClause ifNil: [#()])]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/2/2009 13:43'!
classBody ::=
	<visibility>:vis <classBodyChunk>*:decls => [self factory buildVisibilityClause: vis withDeclarations: decls]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/3/2009 21:14'!
classBodyChunk ::=
	<classMethod>
	| <classMethodResolution>
	| <classField>
	| <classClassProperty>
	| <classProperty>
	| <constSection>
	| <typeSection>
	| <optional #classKeyword>:class <varSection>:vars =>
		[class
			ifNil: [self factory buildInstVarClause: vars]
			ifNotNil: [self factory buildClassVarClause: vars]]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/2/2009 13:43'!
classBodyWithDefaultVisibility ::=
	<classBodyChunk>*:decls => [self factory buildVisibilityClause: (self factory defaultVisibility) withDeclarations: decls]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/3/2009 21:10'!
classClassProperty ::=
	<classKeyword> <classProperty>:prop => [self factory buildClassProperty: prop]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/2/2009 13:35'!
classField ::=
	<optional #customAttribute> <identList>:names <colon> <type>:type <optional #hintingDirective>:directives <semicolon> => [self factory buildClassField: names ofType: type hinting: directives]! !

!PyParser methodsFor: 'types' stamp: 'fbs 1/1/2009 23:59'!
classHelperDecl ::=
	<classKeyword> <helperKeyword> <optional #classParent>:parent <forKeyword> <qualifiedIdent>:class <classBodyChunk>+:decls <endKeyword> => [self factory buildClassHelperFor: class withParent: parent having: decls]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 11/30/2008 21:18'!
classKeyword ::=
	<symbol 'class'>! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/7/2009 09:48'!
classMethod ::=
	<classMethodHeading>:h <semicolon> <methodDirective>*:d => [self factory buildMethod: h withDirectives: d]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/31/2008 23:28'!
classMethodHeading ::=
	<methodKey>:type <ident>:name <formalParameterSection>:params => [self factory buildMethodHeading: type named: name withParams: params]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/9/2009 12:52'!
classMethodResolution ::=
	<procKey> <ident>:delegateClass <dot> <ident>:delegateMethod <equals> <ident>:delegated <semicolon> => [self factory buildMethodDelegationOn: delegated over: delegateMethod in: delegateClass]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/1/2009 23:49'!
classParent ::=
	<openParen> <commaListOf #typeID>:parents <closeParen> => [parents]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/9/2009 06:05'!
classProperty ::=
	<optional #customAttribute> <propertyKeyword> <ident>:name
		<optional #classPropertyArray>:array
		<optional #colonTypeID>:type
		<optional #classPropertyIndex>:index
		<classPropertySpecifier>*:specs <semicolon>
		(<optional #defaultDirective>:d !![d ifNil: [{}] ifNotNil: [{d.}]]):def =>
			[self factory buildProperty: name withArray: array withType: type withIndex: index withSpecifiers: (specs, def)]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/3/2009 15:21'!
classPropertyArray ::=
	<openSqBracket> <formalParameterList>:params <closeSqBracket> => [self factory buildParams: params]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 5/13/2009 19:51'!
classPropertyDispInterface ::=
	<readOnlyDirective>
	| <writeOnlyDirective>
	| <dispIDDirective>! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/3/2009 14:43'!
classPropertyIndex ::=
	<indexDirective>:token <expression>:value => [self factory buildDirective: token withValue: value]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/3/2009 13:58'!
classPropertyReadWrite ::=
	(<readDirective> | <writeDirective> | <addDirective> | <removeDirective>):d <qualifiedIdent>:e =>
		[self factory buildDirective: d withValue: e]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 1/9/2009 06:11'!
classPropertySpecifier ::=
	<classPropertyDispInterface>
	| <classPropertyReadWrite>
	| <storedDirective>
	| <defaultDirectiveWithValue>
	| <implementsDirective>
	| <nodefaultDirective>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/11/2008 15:18'!
classReservedWord ::=
	<privateDirective>
	| <protectedDirective>
	| <publicDirective>
	| <automatedDirective>! !

!PyParser methodsFor: 'types' stamp: 'fbs 1/5/2009 16:05'!
classType ::=
	<classTypeRef>
	| <classHelperDecl>
	| <classTypeDecl>
	| <interfaceTypeDecl>
	| <objectTypeDecl>! !

!PyParser methodsFor: 'types' stamp: 'fbs 1/2/2009 22:06'!
classTypeDecl ::=
	<classKeyword> <optional #sealedOrAbstract>:option <optional #classParent>:superclasses <optional #classBodyWithDefaultVisibility>:head <classBody>*:tail <endKeyword>
		=> [self factory
				buildClassDecl: superclasses
				withClauses: (head
					ifNil: [tail]
					ifNotNil: [(tail asOrderedCollection) addFirst: head; yourself])
				withOption: option]
	| <classKeyword> <optional #sealedOrAbstract>:option <classParent>:superclasses
		=> [self factory
				buildClassDecl: superclasses
				withClauses: #()
				withOption: option]
	| <classKeyword>
		=> [self factory buildForwardClassDecl]! !

!PyParser methodsFor: 'types' stamp: 'fbs 12/31/2008 14:04'!
classTypeRef ::=
	<classKeyword> <ofKeyword> <typeID>:type => [self factory buildClassTypeRef: type]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/14/2008 22:41'!
closeAngleBracket ::=
	<token '>'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/14/2008 22:41'!
closeParen ::=
	<token ')'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/14/2008 22:40'!
closeSqBracket ::=
	<token ']'>
	| <token '.)'>! !

!PyParser methodsFor: 'comments' stamp: 'fbs 1/6/2009 16:11'!
closeStarComment ::=
	<seq '*)'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 5/7/2009 13:59'!
colon ::=
	<token ':'> ~<equals>! !

!PyParser methodsFor: 'to be processed' stamp: 'fbs 1/7/2009 12:32'!
colonConstruct ::=
	<colon> <expression>:e (<colon> <expression> | <empty>) => [e]! !

!PyParser methodsFor: 'to be processed' stamp: 'fbs 12/9/2008 23:23'!
colonTypeID ::=
	<colon> <typeID>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/9/2008 22:54'!
comma ::=
	<token ','>! !

!PyParser methodsFor: 'higher-order-productions' stamp: 'fbs 12/11/2008 09:03'!
commaListOf :prod ::=
	<delimitedListOf #comma prod>! !

!PyParser methodsFor: 'comments' stamp: 'fbs 1/6/2009 21:15'!
comment ::=
	(<oneLineComment>
	| <curlyComment>
	| <starComment>):c => [self factory buildComment: c]! !

!PyParser methodsFor: 'statements' stamp: 'fbs 12/14/2008 21:24'!
compoundStatement ::=
	<beginKeyword> <optionalList #statementList>:statements <endKeyword> => [self factory buildCompoundStatement: statements]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/4/2009 20:34'!
constDeclaration ::=
	<optional #customAttribute> <ident>:name ((<colon> <type>) | <empty> !![nil]):type <equals> <constExpression>:value <hintingDirective>*:directives <semicolon> => [self factory buildConst: name ofType: type withValue: value hinting: directives]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/4/2009 14:32'!
constExpression ::=
	<expression>
	| <openParen> <commaListOf #constExpression>:list <closeParen> => [list]
	| <openParen> <semicolonListOf #recordConstExpression>:list <closeParen> => [list]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 11/30/2008 23:42'!
constKey ::=
	<symbol 'const'>
	| <symbol 'resourcestring'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 11/30/2008 21:11'!
constKeyword ::=
	<symbol 'const'>! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/8/2009 13:44'!
constSection ::=
	<constKey>:key <constDeclaration>+:decls => [self factory buildConstSection: decls named: key]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/12/2008 00:11'!
constructorKeyword ::=
	<symbol 'constructor'>! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/4/2009 23:39'!
containsClause ::=
	<containsDirective> <namespaceFileNameList>:files <semicolon> => [self factory buildContainsClause: files]! !

!PyParser methodsFor: 'directives' stamp: 'fbs 12/11/2008 15:22'!
containsDirective ::=
	<symbol 'contains'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/12/2008 23:44'!
controlString ::=
	$# (<intNum> | <hexNum>):n => [self factory buildControlString: n]
	! !

!PyParser methodsFor: 'comments' stamp: 'fbs 1/6/2009 16:01'!
curlyComment ::=
	${ <nonCurlyChar>*:comment $} => [String withAll: comment]! !

!PyParser methodsFor: 'to be processed' stamp: 'fbs 12/19/2008 20:51'!
customAttribute ::=
	<openSqBracket> <customAttributeDecl> <closeSqBracket>
	| <assemblyAttribute>! !

!PyParser methodsFor: 'to be processed' stamp: 'fbs 12/19/2008 20:50'!
customAttributeDecl ::=
	<namespaceQualifiedIdent> <openParen> <expressionList> <closeParen>! !

!PyParser methodsFor: 'to be processed' stamp: 'fbs 1/4/2009 23:01'!
declSection ::=
	<labelDeclSection>
	| <constSection>
	| <typeSection>
	| <varSection>
	| <methodImpl>
	| <methodDecl>
	| <procImpl>
	| <procDecl>
	| <exportsSection>
	| <assemblyAttribute>! !

!PyParser methodsFor: 'directives' stamp: 'fbs 1/9/2009 06:22'!
defaultDirective ::=
	<directive 'default'>:d <semicolon> => [d]! !

!PyParser methodsFor: 'directives' stamp: 'fbs 1/3/2009 14:30'!
defaultDirectiveWithValue ::=
	<token 'default'>:d <expression>:e => [self factory buildDirective: d withValue: e]! !

!PyParser methodsFor: 'higher-order-productions' stamp: 'fbs 5/9/2009 21:04'!
delimitedListOf :delim :prod ::=
	<apply prod>:head (<apply delim> <apply prod>)*:tail => [tail addFirst: head; yourself]! !

!PyParser methodsFor: 'directives' stamp: 'fbs 12/20/2008 20:16'!
deprecatedDirective ::=
	<directive 'deprecated'>! !

!PyParser methodsFor: 'expressions' stamp: 'fbs 5/8/2009 08:40'!
designator ::=
	<inheritedKeyword> (<designator1> | <empty> !![nil]):ident
		=> [ident
			ifNil: [self factory buildInherited]
			ifNotNil: [self factory buildInherited: ident]]
	| <designator1>! !

!PyParser methodsFor: 'expressions' stamp: 'fbs 5/9/2009 20:57'!
designator1 ::=
	<designator1>:root <dot> <designator2>:last => [self factory buildQualifiedIdent: root qualifying: last]
	| <designator2>! !

!PyParser methodsFor: 'expressions' stamp: 'fbs 1/9/2009 22:04'!
designator2 ::=
	(<ident> | <addressOf> | <openParen> <expression>:e <closeParen> !![e]):root
	<designatorItem>*:parts =>
		[self factory buildDesignator: root withParts: parts]! !

!PyParser methodsFor: 'expressions' stamp: 'fbs 1/9/2009 21:50'!
designatorItem ::=
	<upArrow> => [self factory buildDereference]
	| <openSqBracket> <expressionList>:eList <closeSqBracket>
		=> [self factory buildArrayIndex: eList]
	| <openParen> (<commaListOf #functionCallArgument> | <empty> !![#()]):arguments <closeParen>
		=> [self factory buildFunctionCallArguments: arguments]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/12/2008 00:11'!
destructorKeyword ::=
	<symbol 'destructor'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 11/11/2008 20:22'!
digit ::=
	<char>:c ?[c isDigit] => [c]! !

!PyParser methodsFor: 'higher-order-productions' stamp: 'fbs 12/20/2008 19:46'!
directive :d ::=
	<token d> => [self factory buildDirective: d]! !

!PyParser methodsFor: 'directives' stamp: 'fbs 1/3/2009 15:07'!
dispIDDirective ::=
	<token 'dispid'>:d  <expression>:e => [self factory buildDirective: d withValue: e]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/12/2008 00:11'!
dispInterfaceKeyword ::=
	<symbol 'dispinterface'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/3/2008 21:58'!
doKeyword ::=
	<symbol 'do'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 5/7/2009 13:59'!
dot ::=
	<dotChar> ~<dotChar>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/30/2008 12:17'!
dotChar ::=
	<token '.'>! !

!PyParser methodsFor: 'higher-order-productions' stamp: 'fbs 1/7/2009 22:50'!
dotListOf :prod ::=
	<delimitedListOf #dot prod>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/9/2008 23:15'!
doubleDot ::=
	<token '..'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/29/2008 20:48'!
downtoKeyword ::=
	<symbol 'downto'>! !

!PyParser methodsFor: 'directives' stamp: 'fbs 12/20/2008 20:16'!
dynamicDirective ::=
	<directive 'dynamic'>! !

!PyParser methodsFor: 'statements' stamp: 'fbs 12/18/2008 12:02'!
elseBlock ::=
	<elseKeyword> <optionalList #statementList>:s => [self factory buildElseBlock: s]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 11/30/2008 20:59'!
elseKeyword ::=
	<symbol 'else'>! !

!PyParser methodsFor: 'statements' stamp: 'fbs 11/30/2008 23:43'!
elseStatement ::=
	<elseKeyword> <statement>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 11/30/2008 21:15'!
endKeyword ::=
	<symbol 'end'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/9/2008 23:18'!
equals ::=
	<token '='>! !

!PyParser methodsFor: 'higher-order-productions' stamp: 'fbs 1/3/2009 14:00'!
exactlyNoCase :want ::=
	<anything>:got
		?[(want class canUnderstand: #asLowercase)
			ifTrue: [want asLowercase = got asLowercase]
			ifFalse: [want = got]]
		=> [got]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/8/2008 10:59'!
exceptKeyword ::=
	<symbol 'except'>! !

!PyParser methodsFor: 'directives' stamp: 'fbs 12/20/2008 20:16'!
experimentalDirective ::=
	<directive 'experimental'>! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/5/2009 10:16'!
exportItem ::=
	<ident>:name
	(<openParen> <optional #formalParameterList>:p <closeParen> !![p] | <empty> !![nil]):params
	(<indexDirective>:d <expression>:e !![self factory buildDirective:d withValue: e] | <empty> !![nil]):index
	(<nameDirective>:d <expression>:e !![self factory buildDirective:d withValue: e] | <empty> !![nil]):foreignName
	(<residentDirective> | <empty> !![nil]):resident =>
		[self factory
			buildExportItem: name
			withParams: params
			withDirectives: ({index. foreignName. resident.} select: [:each| each isNil not])]! !

!PyParser methodsFor: 'to be processed' stamp: 'fbs 12/9/2008 23:28'!
exportedProcHeading ::=
	<procKey> <ident> <formalParameterSection> <semicolon> <functionDirective>* <semicolon>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/12/2008 00:11'!
exportsKeyword ::=
	<symbol 'exports'>! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/5/2009 08:54'!
exportsSection ::=
	<exportsSemikeyword> <commaListOf #exportItem>:exports <semicolon> => [self factory buildExportsClause: exports]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/12/2008 13:29'!
exportsSemikeyword ::=
	<symbol 'exports'>! !

!PyParser methodsFor: 'expressions' stamp: 'fbs 5/11/2009 10:24'!
expression ::=
	<expression>:root <relOp>:op <simpleExpression>:crown
		=> [self factory buildExpression: root nextOperator: op nextOperand: crown]
	| <simpleExpression>! !

!PyParser methodsFor: 'expressions' stamp: 'fbs 12/23/2008 23:24'!
expressionList ::=
	<commaListOf #expression>! !

!PyParser methodsFor: 'to be processed' stamp: 'fbs 1/4/2009 22:35'!
extExpression ::=
	<expression>
	| <procDecl>! !

!PyParser methodsFor: 'directives' stamp: 'fbs 5/18/2009 14:35'!
externalDirective ::=
	<token 'external'>:d <externalSpecifier>*:extra <semicolon> => [self factory buildExternalDirective: d andSpecifiers: extra]
	| <token 'external'>:d <constExpression>:e <externalSpecifier>*:extra <semicolon> => [self factory buildExternalDirective: d withValue: e andSpecifiers: extra]
	| <directive 'external'>:d <semicolon> => [d]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 5/5/2009 09:21'!
externalSpecifier ::=
	(<nameDirective> | <indexDirective>):d <constExpression>:e => [self factory buildExternalSpecifier: d withValue: e]! !

!PyParser methodsFor: 'expressions' stamp: 'fbs 1/9/2009 21:56'!
factor ::=
	<designator>
	| <atAt> <factor>:f => [self factory buildAddressOfProc: f]
	| <addressOf>
	| <realNum>
	| <intNum>
	| <hexNum>
	| <true>
	| <false>
	| <nil>
	| <upArrow> <ident>:f => [self factory buildPointerTo: f]
	| <string>
	| <openParen> <expression>:e <closeParen> => [e]
	| <not> <factor>:f => [self factory buildNegated: f]
	| <plus> <factor>
	| <minus> <factor>:f => [self factory buildNegative: f]
	| <setSection>! !

!PyParser methodsFor: 'accessing' stamp: 'fbs 11/24/2008 21:46'!
factory
	^ factory.! !

!PyParser methodsFor: 'accessing' stamp: 'fbs 11/24/2008 21:46'!
factory: aNodeFactory
	factory := aNodeFactory.! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/9/2008 23:31'!
false ::=
	<symbol 'false'>! !

!PyParser methodsFor: 'temp' stamp: 'fbs 11/10/2008 16:52'!
file ::=
	<program> | <library> | <unit> | <package>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/12/2008 00:11'!
fileKeyword ::=
	<symbol 'file'>! !

!PyParser methodsFor: 'types' stamp: 'fbs 12/31/2008 13:44'!
fileType ::=
	<fileKeyword> ((<ofKeyword> <type>) | <empty> !![nil]):type => [self factory buildFileType: type]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/12/2008 13:31'!
finalKeyword ::=
	<symbol 'final'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/12/2008 00:12'!
finalizationKeyword ::=
	<symbol 'finalization'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/12/2008 00:12'!
finallyKeyword ::=
	<symbol 'finally'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/29/2008 20:48'!
forDirection ::=
	<toKeyword>
	| <downtoKeyword>! !

!PyParser methodsFor: 'helper' stamp: 'fbs 12/9/2008 23:13'!
forDomain ::=
	<assign> <expression>:start <forDirection> <expression>:end => [self factory buildForRangeFrom: start to: end]
	| <inKeyword> <expression>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/3/2008 21:58'!
forKeyword ::=
	<symbol 'for'>! !

!PyParser methodsFor: 'statements' stamp: 'fbs 12/9/2008 23:01'!
forStatement ::=
	<forKeyword> <designator>:counter <forDomain>:range <doKeyword> <statement>:s => [self factory buildFor: counter over: range do: s]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/19/2008 22:18'!
formalParameter ::=
	<optional #customAttribute>
	<optional #parmType>:paramType
	<identList>:names
	(<colon> <type> | <empty> !![nil]):type
	(<equals> <expression> | <empty> !![nil]):value
		=> [self factory buildParam: names ofType: type kind: paramType defaultValue: value]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/19/2008 22:22'!
formalParameterList ::=
	<semicolonListOf #formalParameter>! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/23/2008 22:57'!
formalParameterSection ::=
	(<openParen> <optionalList #formalParameterList>:p <closeParen> => [p] | <empty> => [#()]):params <optional #funcResult>: result => [self factory buildParams: params withResult: result]! !

!PyParser methodsFor: 'directives' stamp: 'fbs 12/21/2008 17:22'!
forwardDirective ::=
	<directive 'forward'>:d <semicolon> => [d]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 11/28/2008 21:44'!
fractionalNum ::=
	<intNum>:ip $. <intNum>:fp => [self factory buildDecimalFraction: ip with: fp]
	| <intNum>! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/23/2008 22:55'!
funcResult ::=
	<colon> <optional #customAttribute> <type>:resultType => [self factory buildFunctionResult: resultType]! !

!PyParser methodsFor: 'expressions' stamp: 'fbs 1/7/2009 12:32'!
functionCallArgument ::=
	<extExpression>:e <optional #colonConstruct>:c => [self factory buildFunctionCallArgument: e]! !

!PyParser methodsFor: 'directives' stamp: 'fbs 5/18/2009 14:34'!
functionDirective ::=
	<hintingDirective>
	| <callConvention>:d <semicolon> => [d]
	| <identifier>:i ?[self functionDirective: i] => [self factory buildDirective: i]
	| <inlineDirective>
	| <varargsDirective>
	| <semicolon>! !

!PyParser methodsFor: 'testing' stamp: 'fbs 5/8/2009 09:51'!
functionDirective: aString
	^ FunctionDirectives includes: aString.! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/12/2008 00:12'!
functionKeyword ::=
	<symbol 'function'>! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/29/2008 21:51'!
genericConstruct ::=
	<openAngleBracket> <commaListOf #typeIdent>:typeSpecifiers <closeAngleBracket>
		=> [self factory buildGenericConstruct: typeSpecifiers]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/9/2008 22:44'!
gotoKeyword ::=
	<symbol 'goto'>! !

!PyParser methodsFor: 'statements' stamp: 'fbs 1/12/2009 21:02'!
gotoStatement ::=
	<gotoKeyword> <label>:l => [self factory buildGoto: l]! !

!PyParser methodsFor: 'statements' stamp: 'fbs 1/7/2009 20:04'!
handler ::=
	<onKeyword> (<ident>:i <colon> !![i] | <empty> !![nil]):name <typeID>:type <doKeyword> <optional #statement>:s <semicolon> => [self factory buildHandler: name for: type doing: s]! !

!PyParser methodsFor: 'statements' stamp: 'fbs 12/18/2008 11:59'!
handlerList ::=
	<handler>+:h <optional #elseBlock>:e => [e ifNil: [h] ifNotNil: [h add: e; yourself]]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 11/30/2008 21:18'!
helperKeyword ::=
	<symbol 'helper'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 11/11/2008 20:57'!
hexDigit ::=
	<digit>:n => [n]
	| <char>:x ?[#($a $b $c $d $e $f $A $B $C $D $E $F) contains: [:each | each = x]] => [x]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 5/18/2009 15:38'!
hexNum ::=
	<spacesOrComments> $$ <hexDigit>+:digits =>
		[self factory buildHexNum: (String withAll: digits)]! !

!PyParser methodsFor: 'directives' stamp: 'fbs 5/8/2009 15:35'!
hintingDirective ::=
	<identifier>:d ?[self hintingDirective: d] => [self factory buildDirective: d]! !

!PyParser methodsFor: 'testing' stamp: 'fbs 5/8/2009 15:15'!
hintingDirective: aString
	^ HintingDirectives includes: aString.! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 5/6/2009 15:39'!
ident ::=
	<identifier>:i ?[(self reservedWord: i) not] => [self factory buildIdent: i]
	| $& <identifier>:i ?[self reservedWord: i] => [self factory buildIdent: i]! !

!PyParser methodsFor: 'helper' stamp: 'fbs 1/9/2009 22:14'!
identChar ::=
	<letter> | <digit> | $_! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/18/2008 15:03'!
identList  ::=
	<commaListOf #ident>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 1/9/2009 22:14'!
identifier ::=
	<spacesOrComments> (<letter> | $_):head <identChar>*:tail => [head asString, (String withAll: tail)]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 11/30/2008 21:00'!
ifKeyword ::=
	<symbol 'if'>! !

!PyParser methodsFor: 'statements' stamp: 'fbs 11/26/2008 15:02'!
ifStatement ::=
	<ifKeyword> <expression>:condition <thenKeyword> <statement>:mainBlock <optional #elseStatement>:elseBlock
		=> [elseBlock
			ifNil: [self factory buildIf: condition then: mainBlock]
			ifNotNil: [self factory buildIf: condition then: mainBlock else: elseBlock]]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/12/2008 00:12'!
implementationKeyword ::=
	<symbol 'implementation'>! !

!PyParser methodsFor: 'directives' stamp: 'fbs 1/3/2009 14:30'!
implementsDirective ::=
	<token 'implements'>:d <typeID>:e => [self factory buildDirective: d withValue: e]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 5/7/2009 13:59'!
inKeyword ::=
	<symbol 'in'>:in ~<identChar> => [in]! !

!PyParser methodsFor: 'directives' stamp: 'fbs 12/22/2008 11:21'!
indexDirective ::=
	<token 'index'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/12/2008 00:12'!
inheritedKeyword ::=
	<symbol 'inherited'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/8/2008 11:00'!
initializationKeyword ::=
	<symbol 'initialization'>! !

!PyParser methodsFor: 'initialize-release' stamp: 'fbs 1/10/2009 22:43'!
initialize
	super initialize.
	self factory: PySexprNodeFactory new.
	^ self.! !

!PyParser methodsFor: 'directives' stamp: 'fbs 1/3/2009 14:03'!
inlineDirective ::=
	(<directive 'inline'>	| <directive 'assembler'>):d <semicolon> => [d]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/11/2008 15:08'!
inlineKeyword ::=
	<symbol 'inline'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 5/18/2009 15:38'!
intNum ::=
	<spacesOrComments> <digit>+:digits => [self factory buildIntNum: (String withAll: digits)]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/5/2009 15:58'!
interfaceBody ::=
	<classMethod>
	| <classClassProperty>
	| <classProperty>! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/28/2008 20:35'!
interfaceDecl ::=
	<constSection>
	| <typeSection>
	| <varSection>
	| <exportedProcHeading>
	| <exportsSection>
	| <methodDecl>
	| <procDecl>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 1/5/2009 15:50'!
interfaceGuid ::=
	<openSqBracket> <quotedString>:guid <closeSqBracket> => [self factory buildGuid: guid]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/18/2008 15:31'!
interfaceKey ::=
	<interfaceKeyword>
	| <dispInterfaceKeyword>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/11/2008 15:08'!
interfaceKeyword ::=
	<symbol 'interface'>! !

!PyParser methodsFor: 'types' stamp: 'fbs 1/5/2009 15:56'!
interfaceTypeDecl ::=
	<interfaceKey> <optional #classParent>:parents <optional #interfaceGuid>:guid <interfaceBody>*:decls <endKeyword>
		=> [self factory buildInterfaceExtending: parents withGuid: guid withDecls: decls]
	| <interfaceKey> <optional #classParent>:parents
		=> [self factory buildInterfaceExtending: parents]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 1/8/2009 08:25'!
isKeyword ::=
	<symbol 'is'>! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/7/2008 15:18'!
label ::=
	(<ident>
	| <intNum>
	| <hexNum>):l => [self factory buildLabel: l]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/20/2008 22:30'!
labelDeclSection ::=
	<labelKeyword> <commaListOf #label>:list <semicolon> => [self factory buildLabelDecl: list]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/11/2008 15:08'!
labelKeyword ::=
	<symbol 'label'>! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/8/2009 16:59'!
labelMarker ::=
	<label>:l <colon> => [l]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/30/2008 15:15'!
library ::=
	<libraryHead>:head <optional #usesFileClause>:uses <block>:block <dot> =>
		[self factory
			buildLibrary: head
			withUses: uses
			with: block]! !

!PyParser methodsFor: 'directives' stamp: 'fbs 12/20/2008 20:17'!
libraryDirective ::=
	<directive 'library'>! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/30/2008 14:28'!
libraryHead ::=
	<libraryKeyword> <namespaceName>:name <hintingDirective>*:directives <semicolon> => [self factory buildLibraryHead: name withDirectives: directives]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 11/30/2008 21:12'!
libraryKeyword ::=
	<symbol 'library'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 1/9/2009 15:15'!
lineEnd ::=
	<char>:c1 <char>:c2 ?[(c1 asciiValue = 13) and: [c2 asciiValue = 10]] => [{c1. c2.}]
	| <char>:c ?[{Character cr. Character lf.} includes: c] => [c]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 11/28/2008 22:00'!
literalCharacter ::=
	$# (<intNum> | <hexNum>):n => [self factory buildLiteralCharacter: n]
	! !

!PyParser methodsFor: 'to be processed' stamp: 'fbs 5/6/2009 15:42'!
localLabelPart ::=
	<at> (<label> | <identifier>:i ?[self reservedWord: i] => [self factory buildLabel: i])! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/14/2008 22:42'!
methodBody ::=
	<block>:b <semicolon> => [b]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/4/2009 22:33'!
methodDecl ::=
	<methodDeclHeading>:heading <semicolon> <optional #functionDirective>:directive => [self factory buildMethodDecl: heading withBinding: directive]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/4/2009 23:50'!
methodDeclHeading ::=
	<optional #customAttribute>
	<methodKey>:key
	(<ident>:class <dot> <qualifiedIdent>:proc
		!![self factory buildQualifiedIdent: class qualifying: proc]):name
	<optional #formalParameterSection>:params
		=> [self factory buildMethodHeading: key named: name withParams: params]! !

!PyParser methodsFor: 'directives' stamp: 'fbs 1/7/2009 09:40'!
methodDirective ::=
	(<reintroduceDirective>
	| <overloadDirective>
	| <bindingDirective>
	| <callConvention>
	| <abstractDirective>
	| <hintingDirective>
	| <dispIDDirective>):d <semicolon> => [d]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/4/2009 22:44'!
methodImpl ::=
	<methodDecl>:declaration <methodBody>: body => [self factory buildMethodImpl: declaration withBody: body]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/4/2009 21:39'!
methodKey ::=
	<optional #classKeyword>:class <procKey>:funcOrProc
		=> [((class ifNil: [funcOrProc] ifNotNil: [class, (funcOrProc capitalized)]), 'Method') asSymbol]
	| <optional #classKeyword>:class <constructorKeyword>:cons
		=> [class ifNil: [cons] ifNotNil: [class, (cons capitalized)]]
	| <destructorKeyword>
	| <classKeyword> <operatorKeyword>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/9/2008 23:30'!
minus ::=
	<token '-'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 11/30/2008 21:01'!
mulOp ::=
	<symbol '*'>
	| <symbol '/'>
	| <symbol 'div'>
	| <symbol 'mod'>
	| <symbol 'and'>
	| <symbol 'shl'>
	| <symbol 'shr'>! !

!PyParser methodsFor: 'directives' stamp: 'fbs 12/22/2008 11:20'!
nameDirective ::=
	<token 'name'>! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/14/2008 22:04'!
namespaceFileName ::=
	<namespaceName>:name <inKeyword> <string>:path => [self factory buildNamespaceFileName: name in: path]
	| <namespaceName>:name => [self factory buildNamespaceFileName: name]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/4/2009 23:46'!
namespaceFileNameList ::=
	<commaListOf #namespaceFileName>:list => [list]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 5/9/2009 21:02'!
namespaceName ::=
	<namespaceName>:trunk <dot> <ident>:crown => [self factory buildQualifiedIdent: trunk qualifying: crown]
	| <ident>! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/14/2008 21:02'!
namespaceNameList ::=
	<commaListOf #namespaceName>! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/14/2008 20:32'!
namespaceQualifiedIdent ::=
	((<namespaceName>:name <dot> => [name]) | <empty> => [nil]):root <qualifiedIdent>:tail =>
		[root
			ifNil: [tail]
			ifNotNil: [self factory buildQualifiedIdent: root qualifying: tail]]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/11/2008 15:09'!
nil ::=
	<symbol 'nil'>! !

!PyParser methodsFor: 'directives' stamp: 'fbs 1/3/2009 14:50'!
nodefaultDirective ::=
	<directive 'nodefault'>! !

!PyParser methodsFor: 'comments' stamp: 'fbs 1/6/2009 16:02'!
nonCurlyChar ::=
	<char>:c ?[c ~= $}] => [c]! !

!PyParser methodsFor: 'helper' stamp: 'fbs 4/24/2009 11:56'!
nonIdentChar ::=
	~($_ | <digit> | <letter>)! !

!PyParser methodsFor: 'comments' stamp: 'fbs 1/9/2009 17:28'!
nonLineEnd ::=
	<char>:c ?[({Character cr. Character lf.} includes: c) not] => [c]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/11/2008 15:09'!
not ::=
	<symbol 'not'>! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/5/2009 16:23'!
objectBody ::=
	<visibility>:vis <objectBodyChunk>*:decls => [self factory buildVisibilityClause: vis withDeclarations: decls]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 5/5/2009 09:22'!
objectBodyChunk ::=
	<classMethod>
	| <classField>! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/5/2009 21:15'!
objectBodyWithDefaultVisibility ::=
	<objectBodyChunk>+:decls => [self factory buildVisibilityClause: (self factory defaultVisibility) withDeclarations: decls]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/12/2008 00:13'!
objectKeyword ::=
	<symbol 'object'>! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/5/2009 16:29'!
objectTypeDecl ::=
	<objectKeyword> <optional #classParent>:superclasses <optional #objectBodyWithDefaultVisibility>: head <objectBody>*:tail <endKeyword> =>
		[self factory
				buildObjectDecl: superclasses
				withClauses: (head
					ifNil: [tail]
					ifNotNil: [(tail asOrderedCollection) addFirst: head; yourself])]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 11/30/2008 21:10'!
ofKeyword ::=
	<symbol 'of'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/9/2008 23:33'!
onKeyword ::=
	<symbol 'on'>! !

!PyParser methodsFor: 'comments' stamp: 'fbs 5/7/2009 14:41'!
oneLineComment ::=
	<seq '//'> <nonLineEnd>*:c => [c ifNil: [''] ifNotNil: [String withAll: c]]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/9/2008 23:33'!
openAngleBracket ::=
	<token '<'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/9/2008 22:54'!
openParen ::=
	<token '('>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/10/2008 22:39'!
openSqBracket ::=
	<token '['>
	| <token '(.'>! !

!PyParser methodsFor: 'comments' stamp: 'fbs 1/6/2009 16:11'!
openStarComment ::=
	<seq '(*'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/12/2008 00:13'!
operatorKeyword ::=
	<symbol 'operator'>! !

!PyParser methodsFor: 'higher-order-productions' stamp: 'fbs 12/7/2008 15:20'!
optional :prod ::=
	<apply prod>:p => [p]
	| <empty> => [nil]! !

!PyParser methodsFor: 'higher-order-productions' stamp: 'fbs 12/14/2008 21:20'!
optionalList :prod ::=
	<apply prod>:p => [p]
	| <empty> => [#()]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/11/2008 15:10'!
outKeyword ::=
	<symbol 'out'>! !

!PyParser methodsFor: 'directives' stamp: 'fbs 12/20/2008 20:18'!
overloadDirective ::=
	<directive 'overload'>! !

!PyParser methodsFor: 'directives' stamp: 'fbs 12/20/2008 20:18'!
overrideDirective ::=
	<directive 'override'>! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/4/2009 23:31'!
package ::=
	<packageHead>:head <optional #requiresClause>:requires <optional #containsClause>:contains <endKeyword> <dot> => [self factory buildPackage: head requiring: requires containing: contains]! !

!PyParser methodsFor: 'directives' stamp: 'fbs 12/20/2008 20:19'!
packageDirective ::=
	<directive 'package'>! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/4/2009 22:57'!
packageHead ::=
	<packageKeyword> <namespaceName>:name <semicolon> => [self factory buildPackageHead: name]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/12/2008 00:13'!
packageKeyword ::=
	<symbol 'package'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/12/2008 00:13'!
packedKeyword ::=
	<symbol 'packed'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/19/2008 21:16'!
parmType ::=
	(<symbol 'const'>
	| <symbol 'var'>
	| <symbol 'out'>):kind => [self factory buildParmType: kind]! !

!PyParser methodsFor: 'assembly' stamp: 'fbs 11/10/2008 22:25'!
pentium1Opcode ::=
	<ident>! !

!PyParser methodsFor: 'assembly' stamp: 'fbs 11/10/2008 22:25'!
pentium2Opcode ::=
	<ident>! !

!PyParser methodsFor: 'assembly' stamp: 'fbs 11/10/2008 22:25'!
pentium3Opcode ::=
	<ident>! !

!PyParser methodsFor: 'assembly' stamp: 'fbs 11/10/2008 22:25'!
pentium4Opcode ::=
	<ident>! !

!PyParser methodsFor: 'directives' stamp: 'fbs 12/20/2008 20:19'!
platformDirective ::=
	<directive 'platform'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/9/2008 23:30'!
plus ::=
	<token '+'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/12/2008 00:14'!
pointerKeyword ::=
	<symbol 'pointer'>! !

!PyParser methodsFor: 'types' stamp: 'fbs 12/29/2008 20:58'!
pointerType ::=
	<upArrow> <type>:name => [self factory buildPointerType: name]
	| <pointerKeyword> => [self factory buildPointerType]! !

!PyParser methodsFor: 'directives' stamp: 'fbs 12/11/2008 15:19'!
privateDirective ::=
	<symbol 'private'>! !

!PyParser methodsFor: 'temp' stamp: 'fbs 5/18/2009 14:49'!
procBody ::=
	(<forwardDirective> | <externalDirective>):head <functionDirective>*:tail
		=> [OrderedCollection with: (self factory buildDirectives: (tail addFirst: head; yourself))]
	| <methodBody>! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/20/2008 22:45'!
procDecl ::=
	<procDeclHeading>:heading <semicolon> <functionDirective>*:directives => [self factory buildProcDecl: heading directives: directives]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/4/2009 23:03'!
procDeclHeading ::=
	<optional #customAttribute> <procKey>:funcType <ident>:name <optional #formalParameterSection>:params => [self factory buildProcHeading: funcType named: name withParams: params]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/20/2008 22:41'!
procImpl ::=
	<procDecl>:declaration <procBody>: body => [self factory buildProcImpl: declaration withBody: body]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 11/30/2008 21:01'!
procKey ::=
	<symbol 'function'>
	| <symbol 'procedure'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/12/2008 00:14'!
procedureKeyword ::=
	<symbol 'procedure'>! !

!PyParser methodsFor: 'types' stamp: 'fbs 12/29/2008 16:58'!
procedureType ::=
	<procedureTypeHeading>:heading
	(<ofKeyword> <objectKeyword> !![true] | <empty> !![false]):isMethod
	(<semicolon> <callConvention>:cc !![OrderedCollection with: cc] | <empty> !![#()]):directive
		=> [self factory
				buildProcedureType: heading
				isMethod: isMethod
				withDirectives: directive]! !

!PyParser methodsFor: 'types' stamp: 'fbs 12/29/2008 20:45'!
procedureTypeHeading ::=
	(<referenceSemikeyword> <toKeyword> !![true] | <empty> !![false]):reference
	<procedureTypeHeadingSimple>:heading
		=> [self factory buildProcedureTypeHeading: heading isReference: reference]
! !

!PyParser methodsFor: 'types' stamp: 'fbs 12/29/2008 16:15'!
procedureTypeHeadingSimple ::=
	(<functionKeyword> | <procedureKeyword>):funcOrProc <formalParameterSection>:params => [self factory buildProcedureTypeHeadingSimple: funcOrProc withParams: params]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/14/2008 21:19'!
program ::=
	<optional #programHead>:name <optionalList #usesFileClause>:uses <block>:block <dot> => [self factory buildProgram: name using: uses block: block]! !

!PyParser methodsFor: 'statements' stamp: 'fbs 12/14/2008 21:28'!
programHead ::=
	<programKeyword> <namespaceName>:name <optionalList #programParmSpec>:spec <semicolon> => [self factory buildProgramHead: name with: spec]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 11/30/2008 21:12'!
programKeyword ::=
	<symbol 'program'>! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/14/2008 20:31'!
programParmSpec ::=
	<openParen> (<commaListOf #ident> | <empty> => [#()]):params <closeParen> => [params]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 11/30/2008 22:16'!
propertyKeyword ::=
	<symbol 'property'>! !

!PyParser methodsFor: 'directives' stamp: 'fbs 12/11/2008 15:19'!
protectedDirective ::=
	<symbol 'protected'>! !

!PyParser methodsFor: 'directives' stamp: 'fbs 12/11/2008 15:19'!
publicDirective ::=
	<symbol 'public'>! !

!PyParser methodsFor: 'directives' stamp: 'fbs 12/11/2008 15:19'!
publishedDirective ::=
	<symbol 'published'>! !

!PyParser methodsFor: 'expressions' stamp: 'fbs 5/9/2009 20:48'!
qualifiedIdent ::=
	<qualifiedIdent>:root <dot> (<ident> | <identifier>:i !![self factory buildIdent: i]):last => [self factory buildQualifiedIdent: root qualifying: last]
	| <ident>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/13/2008 21:38'!
quotedString ::=
	$' ($' $' | ~$' <char>)*:xs $'	=> [self factory buildQuotedString: xs]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/3/2008 21:58'!
raiseKeyword ::=
	<symbol 'raise'>! !

!PyParser methodsFor: 'statements' stamp: 'fbs 12/13/2008 22:08'!
raiseStatement ::=
	<raiseKeyword> <designator>:varname <optional #callerAddress>:address => [self factory buildRaise: varname at: address]
	| <raiseKeyword> <optional #designator>:varname => [self factory buildRaise: varname]! !

!PyParser methodsFor: 'directives' stamp: 'fbs 1/3/2009 14:51'!
readDirective ::=
	<token 'read'>! !

!PyParser methodsFor: 'directives' stamp: 'fbs 1/3/2009 15:11'!
readOnlyDirective ::=
	<directive 'readonly'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 11/28/2008 21:25'!
realNum ::=
	<spaces>
	(<fractionalNum>:significand ($e | $E) ($+ | $- | <empty>):sign <fractionalNum>:exponent)
		=> [self factory buildReal: significand raisedTo: exponent withSign: sign]
	| <fractionalNum>:significand! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/4/2009 14:30'!
recordConstExpression ::=
	<ident>:name <colon> <constExpression>:value => [self factory buildConstRecord: name value: value]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/4/2009 13:52'!
recordField ::=
	<identList>:names <colon> <type>:type <hintingDirective>*:hints => [self factory buildRecordField: names ofType: type hinting: hints]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 5/25/2009 15:43'!
recordFieldExt ::=
	<recordField>:f <optional #semicolon> => [f]
	| <classMethod>
	| <classClassProperty>
	| <classProperty>
	| <constSection>
	| <typeSection>
	| <optional #classKeyword>:class <varSection>:vars =>
		[class
			ifNil: [self factory buildInstVarClause: vars]
			ifNotNil: [self factory buildClassVarClause: vars]]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/12/2008 00:15'!
recordKeyword ::=
	<symbol 'record'>! !

!PyParser methodsFor: 'types' stamp: 'fbs 1/4/2009 13:51'!
recordType ::=
	<recordKeyword> <recordFieldExt>*:fields <optional #recordVariantSection>:varSection <endKeyword> => [self factory buildRecord: fields withVariantSection: varSection]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/4/2009 14:14'!
recordVariant ::=
	<commaListOf #expression>:e <colon> <openParen> <semicolonListOf #recordField>:fields <optional #semicolon> <closeParen> =>
		[self factory buildRecordVariant: fields withIndices: e]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/4/2009 14:26'!
recordVariantSection ::=
	<caseKeyword> ((<ident>:i <colon>=> [i]) | <empty> !![nil]):name <type>:type <ofKeyword> <semicolonListOf #recordVariant>:unionDecls <optional #semicolon> => [self factory buildRecordVariantSection: name ofType: type containingUnions: unionDecls]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/12/2008 00:16'!
referenceSemikeyword ::=
	<symbol 'reference'>! !

!PyParser methodsFor: 'directives' stamp: 'fbs 1/7/2009 09:52'!
reintroduceDirective ::=
	<directive 'reintroduce'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 1/9/2009 22:13'!
relOp ::=
	<symbol '<>'>
	| <symbol '<='>
	| <symbol '<'>	
	| <symbol '>='>
	| <symbol '>'>
	| <symbol '='>
	| <inKeyword>
	| <isKeyword>
	| <asKeyword>! !

!PyParser methodsFor: 'directives' stamp: 'fbs 1/3/2009 14:51'!
removeDirective ::=
	<token 'remove'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/8/2008 11:01'!
repeatKeyword ::=
	<symbol 'repeat'>! !

!PyParser methodsFor: 'statements' stamp: 'fbs 12/14/2008 21:28'!
repeatStatement ::=
	<repeatKeyword> <optionalList #statementList>:statements <untilKeyword> <expression>:termCondition => [self factory buildRepeat: statements until: termCondition]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/4/2009 23:39'!
requiresClause ::=
	<requiresDirective> <namespaceNameList>:list <semicolon> => [self factory buildRequiresClause: list]! !

!PyParser methodsFor: 'directives' stamp: 'fbs 12/11/2008 15:31'!
requiresDirective ::=
	<symbol 'requires'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 5/7/2009 08:19'!
reservedWord ::=
	<identifier>:i ?[self reservedWord: i] => [self factory buildIdent: i]! !

!PyParser methodsFor: 'testing' stamp: 'fbs 5/13/2009 12:33'!
reservedWord: aString
	^ Keywords includes: aString asLowercase.! !

!PyParser methodsFor: 'directives' stamp: 'fbs 12/20/2008 19:45'!
residentDirective ::=
	<directive 'resident'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/11/2008 15:11'!
sealedKeyword ::=
	<symbol 'sealed'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/11/2008 15:11'!
sealedOrAbstract ::=
	<sealedKeyword>
	| <symbol 'abstract'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/9/2008 22:54'!
semicolon ::=
	<token ';'>! !

!PyParser methodsFor: 'higher-order-productions' stamp: 'fbs 12/11/2008 09:03'!
semicolonListOf :prod ::=
	<delimitedListOf #semicolon prod>! !

!PyParser methodsFor: 'higher-order-productions' stamp: 'fbs 12/19/2008 15:22'!
seqNoCase
	| xs next result |
	result := OrderedCollection new.
	^ self oAnd: {
		[xs := self apply: #anything.
		xs.].
		[xs do: [:x |
			next := self apply: #exactlyNoCase withArguments: {x}.
			(next = Fail)
				ifTrue: [^ Fail].
			result addLast: next]].
		[xs class withAll: result]
	}! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/12/2008 00:16'!
setKeyword ::=
	<symbol 'set'>! !

!PyParser methodsFor: 'expressions' stamp: 'fbs 12/10/2008 21:23'!
setSection ::=
	<openSqBracket> (<commaListOf #caseLabel> | <empty> => [#()]):set <closeSqBracket> => [self factory buildSet: set]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/29/2008 21:38'!
setType ::=
	<setKeyword> <ofKeyword> <type>:type => [self factory buildSetType: type]! !

!PyParser methodsFor: 'expressions' stamp: 'fbs 5/12/2009 15:55'!
simpleExpression ::=
	<simpleExpression>:root <addOp>:op <term>:crown => [self factory buildExpression: root nextOperator: op nextOperand: crown]
	| <term>! !

!PyParser methodsFor: 'statements' stamp: 'fbs 12/9/2008 23:04'!
simpleStatement ::=
	<gotoStatement>
	| <designator>:lvalue <assign> <expression>:rvalue => [self factory buildAssign: rvalue to: lvalue]
	| <designator>! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/19/2008 15:37'!
simpleType ::=
	<simpleTypeFloat>
	| <simpleTypeOrdinal>
	| <simpleTypeEnum>
	| <simpleTypeSubRange>! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/29/2008 23:27'!
simpleTypeEnum ::=
	<openParen> <commaListOf #valuedIdent>:values <closeParen> => [self factory buildEnum: values]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/10/2008 10:35'!
simpleTypeFloat ::=
	<token 'real48'>
	| <token 'real'>
	| <token 'single'>
	| <token 'double'>
	| <token 'extended'>
	| <token 'currency'>
	| <token 'comp'>! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/19/2008 15:33'!
simpleTypeOrdinal ::=
	<token 'shortint'>
	| <token 'smallint'>
	| <token 'integer'>
	| <token 'cardinal'>
	| <token 'byte'>
	| <token 'longint'>
	| <token 'int64'>
	| <token 'word'>
	| <token 'boolean'>
	| <token 'bytebool'>
	| <token 'wordbool'>
	| <token 'ansichar'>
	| <token 'pansichar'>
	| <token 'longbool'>
	| <token 'char'>
	| <token 'widechar'>
	| <token 'pwidechar'>
	| <token 'longword'>
	| <token 'pchar'>
! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/28/2008 23:25'!
simpleTypeSubRange ::=
	<expression>:start ((<doubleDot> <expression>) | <empty> => [nil]):end =>
		[end
			ifNil: [start]
			ifNotNil: [self factory buildRangeFrom: start to: end]]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 5/7/2009 14:50'!
spacesOrComments ::=
	(<space> | <comment>)*! !

!PyParser methodsFor: 'comments' stamp: 'fbs 1/6/2009 16:12'!
starComment ::=
	<openStarComment> <starCommentChar>*:comment <closeStarComment> => [String withAll: comment]! !

!PyParser methodsFor: 'comments' stamp: 'fbs 5/7/2009 13:59'!
starCommentChar ::=
	~<closeStarComment> <char>! !

!PyParser methodsFor: 'statements' stamp: 'fbs 12/7/2008 15:21'!
statement ::=
	<optional #labelMarker>:label <statementPart>:stmt =>
		[label
			ifNil: [self factory buildStatement: stmt]
			ifNotNil: [self factory buildStatement: stmt labelled: label]]! !

!PyParser methodsFor: 'statements' stamp: 'fbs 5/8/2009 08:30'!
statementList ::=
	<statement>:head <semicolon>* <statementList>:tail
		=> [tail addFirst: head; yourself]
	| <semicolon>* => [OrderedCollection new]! !

!PyParser methodsFor: 'statements' stamp: 'fbs 12/17/2008 23:18'!
statementOrHandlerList ::=
	<handlerList> | <statementList>! !

!PyParser methodsFor: 'statements' stamp: 'fbs 12/9/2008 20:32'!
statementPart ::=
	<simpleStatement>
	| <ifStatement>
	| <caseStatement>
	| <repeatStatement>
	| <whileStatement>
	| <forStatement>
	| <withStatement>
	| <tryStatement>
	| <raiseStatement>
	| <assemblerStatement>
	| <compoundStatement>! !

!PyParser methodsFor: 'directives' stamp: 'fbs 1/6/2009 21:24'!
staticDirective ::=
	<directive 'static'>! !

!PyParser methodsFor: 'directives' stamp: 'fbs 1/3/2009 14:47'!
storedDirective ::=
	<token 'stored'>:d <expression>:e => [self factory buildDirective: d withValue: e]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/12/2008 00:16'!
strictSemikeyword ::=
	<symbol 'strict'>! !

!PyParser methodsFor: 'expressions' stamp: 'fbs 5/18/2009 15:50'!
string ::=
	<spacesOrComments> (<quotedString> | <controlString>)+:s
		=> [s inject: '' into: [:result :each | result := result, each]]! !

!PyParser methodsFor: 'types' stamp: 'fbs 5/15/2009 13:06'!
stringType ::=
	(<token 'string'>:s <openSqBracket> <expression>:len <closeSqBracket>) => [self factory buildStringType: len]
	| (<token 'string'> | <token 'shortstring'> | <token 'ansistring'> | <token 'widestring'>):type => [self factory buildIdent: type]! !

!PyParser methodsFor: 'types' stamp: 'fbs 12/29/2008 21:30'!
strucType ::=
	<optional #packedKeyword>:packed <strucTypePart>:struct => [self factory buildStructType: struct isPacked: (packed isNil not)]! !

!PyParser methodsFor: 'types' stamp: 'fbs 11/10/2008 21:28'!
strucTypePart ::=
	<arrayType>
	| <setType>
	| <fileType>
	| <recordType>
	| <classType>! !

!PyParser methodsFor: 'higher-order-productions' stamp: 'fbs 12/19/2008 15:08'!
symbol :k ::=
	<token k> => [k asLowercase asSymbol]! !

!PyParser methodsFor: 'expressions' stamp: 'fbs 5/12/2009 15:56'!
term ::=
	<term>:root <mulOp>:op <factor>:crown => [self factory buildExpression: root nextOperator: op nextOperand: crown]
	| <factor>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 11/30/2008 20:58'!
thenKeyword ::=
	<symbol 'then'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/12/2008 00:16'!
threadvarKeyword ::=
	<symbol 'threadvar'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/29/2008 20:48'!
toKeyword ::=
	<symbol 'to'>! !

!PyParser methodsFor: 'higher-order-productions' stamp: 'fbs 5/7/2009 13:59'!
token :xs ::=
	<spacesOrComments> <seqNoCase xs>:tok (~<nonIdentChar> | <empty>) => [tok]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/9/2008 23:31'!
true ::=
	<symbol 'true'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/8/2008 10:59'!
tryKeyword ::=
	<symbol 'try'>! !

!PyParser methodsFor: 'statements' stamp: 'fbs 12/17/2008 23:00'!
tryStatement ::=
	<tryKeyword> <optionalList #statementList>:list <exceptKeyword> <optionalList #statementOrHandlerList>:handlers <endKeyword> => [self factory buildTry: list except: handlers]
	| <tryKeyword> <optionalList #statementList>:list <finallyKeyword> <optionalList #statementList>:cleanup <endKeyword> => [self factory buildTry: list finally: cleanup]! !

!PyParser methodsFor: 'types' stamp: 'fbs 12/29/2008 22:26'!
type ::=
	<strucType>
	| <pointerType>
	| <stringType>
	| <procedureType>
	| <variantType>
	| <optional #typeKeyword>:type <typeIdent>:ident <optional #genericConstruct>:gen
		=> [self factory
				buildType: ident
				isDistinct: (type isNil not)
				withGenerics: gen]
	| <simpleType>:name => [self factory buildIdent: name]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/31/2008 10:16'!
typeDeclaration ::=
  <optional #customAttribute> <ident>:name <optional #genericConstruct>:typeArguments <equals> <type>:type <hintingDirective>*:hints <semicolon> => [self factory buildType: name withTypeArguments: typeArguments definedBy: type withHints: hints]! !

!PyParser methodsFor: 'types' stamp: 'fbs 12/1/2008 20:52'!
typeID ::=
	<namespaceQualifiedIdent>! !

!PyParser methodsFor: 'types' stamp: 'fbs 12/19/2008 15:30'!
typeIdent ::=
	<typeID>
	| <simpleTypeFloat>
	| <stringType>
	| <variantType>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/12/2008 00:16'!
typeKeyword ::=
	<symbol 'type'>! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/31/2008 13:33'!
typeSection ::=
	<typeKeyword> <typeDeclaration>+:decls => [self factory buildTypeClause: decls]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/27/2008 16:52'!
unit ::=
	<unitHead>:head <unitInterface>:iface <unitImplementation>:impl <unitBlock>:block <dot> => [self factory buildUnit: head withInterface: iface withImplementation: impl withBlock: block]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/27/2008 17:05'!
unitBlock ::=
	<unitInitialization>:init <endKeyword> => [init]
	| <compoundStatement>
	| <endKeyword> => [#()]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/28/2008 22:02'!
unitFinalization ::=
	<finalizationKeyword> <optionalList #statementList>:stmts => [self factory buildUnitFinalization: stmts]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/28/2008 20:04'!
unitHead ::=
	<unitKeyword> <namespaceName>:name <hintingDirective>*:directives <semicolon> => [self factory buildUnitHead: name withDirectives: directives]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/27/2008 16:53'!
unitImplementation ::=
	<implementationKeyword> <optional #usesClause>:uses <declSection>*:decls => [self factory buildUnitImplementation: decls using: uses]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/28/2008 21:44'!
unitInitialization ::=
	<initializationKeyword> <optionalList #statementList>:stmts <optional #unitFinalization>:final => [self factory buildUnitInitialization: stmts withFinalization: final]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/27/2008 17:00'!
unitInterface ::=
	<interfaceKeyword> <optional #usesClause>:uses <interfaceDecl>*:decls => [self factory buildUnitInterface: decls using: uses]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/12/2008 00:17'!
unitKeyword ::=
	<symbol 'unit'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/20/2008 20:21'!
unsafeKeyword ::=
	<directive 'unsafe'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/8/2008 11:01'!
untilKeyword ::=
	<symbol 'until'>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/9/2008 23:26'!
upArrow ::=
	<token '^'>! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/6/2009 21:10'!
usesClause ::=
	<usesKeyword> <namespaceNameList>:list <semicolon> => [self factory buildUsesClause: list]! !

!PyParser methodsFor: 'statements' stamp: 'fbs 1/4/2009 23:46'!
usesFileClause ::=
	<usesKeyword> <namespaceFileNameList>:list <semicolon> => [self factory buildUsesClause: list]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/12/2008 00:17'!
usesKeyword ::=
	<symbol 'uses'>! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/29/2008 23:37'!
valuedIdent ::= 
	<ident>:name ((<equals> <expression>) | <empty> !![nil]):e => [self factory buildEnumValue: name withOrdinality: e]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/18/2008 15:02'!
varDeclaration ::=
	<optional #customAttribute> <identList>:names <colon> <type>:type <optional #varValueSpec>:value <hintingDirective>*:directives <semicolon> => [self factory buildVarDeclaration: names ofType: type havingValue: value hinting: directives]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/10/2008 10:44'!
varKey ::=
	<varKeyword>
	| <threadvarKeyword>! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/12/2008 00:17'!
varKeyword ::=
	<symbol 'var'>! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/18/2008 15:07'!
varSection
	<varKey>:varType <varDeclaration>+:decls => [self factory buildVarSection: varType for: decls]! !

!PyParser methodsFor: 'temp' stamp: 'fbs 12/19/2008 15:39'!
varValueSpec ::=
	<absoluteDirective> <constExpression>:ident  => [self factory buildAbsoluteDirective: ident]
	| <equals> <constExpression>! !

!PyParser methodsFor: 'directives' stamp: 'fbs 5/18/2009 14:35'!
varargsDirective ::=
	<directive 'varargs'>:d <semicolon> => [d]! !

!PyParser methodsFor: 'types' stamp: 'fbs 12/29/2008 21:41'!
variantType ::=
	(<token 'variant'>
	| <token 'olevariant'>):v => [self factory buildIdent: v]! !

!PyParser methodsFor: 'directives' stamp: 'fbs 12/20/2008 20:20'!
virtualDirective ::=
	<directive 'virtual'>! !

!PyParser methodsFor: 'temp' stamp: 'fbs 1/2/2009 00:22'!
visibility ::=
	<optional #strictSemikeyword>:s (<privateDirective> | <protectedDirective>):v
		=> [self factory buildVisibility: (s ifNil: [v] ifNotNil: [s, v capitalized])]
	| (<publicDirective> | <publishedDirective> | <automatedDirective>):v
		=> [self factory buildVisibility: v]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/9/2008 22:02'!
whileKeyword ::=
	<symbol 'while'>! !

!PyParser methodsFor: 'statements' stamp: 'fbs 12/14/2008 21:30'!
whileStatement ::=
	<whileKeyword> <expression>:condition <doKeyword> <optionalList #statement>:block => [self factory buildWhile: condition do: block]! !

!PyParser methodsFor: 'tokens' stamp: 'fbs 12/8/2008 16:05'!
withKeyword ::=
	<symbol 'with'>! !

!PyParser methodsFor: 'statements' stamp: 'fbs 5/13/2009 19:40'!
withStatement ::=
	<withKeyword> <commaListOf #designator>:vars <doKeyword> <statement>:s =>
		[self factory buildWith: vars doing: s]
! !

!PyParser methodsFor: 'directives' stamp: 'fbs 1/3/2009 14:51'!
writeDirective ::=
	<token 'write'>! !

!PyParser methodsFor: 'directives' stamp: 'fbs 1/3/2009 15:11'!
writeOnlyDirective ::=
	<directive 'writeonly'>! !

!String methodsFor: '*Pythia-Parser-converting' stamp: 'fbs 5/21/2009 10:00'!
asSexpr
	^ '"', self, '"'.! !

TestCase subclass: #TestPyAddressOfNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST-Tests'!

!TestPyAddressOfNode methodsFor: 'test-converting' stamp: 'fbs 5/21/2009 10:46'!
testAsSexpr
	| a ref sfac |
	ref := PyIdentNode named: 'Foo'.
	a := PyAddressOfNode new
		reference: ref;
		yourself.
	sfac := PySexprNodeFactory new.
	
	self assert: a asSexpr = (sfac buildAddressOf: a reference asOrderedCollection) asSexpr.! !

!TestPyAddressOfNode methodsFor: 'test-accessing' stamp: 'fbs 5/21/2009 14:49'!
testChildren
	| a ref |
	ref := PyIdentNode named: 'Foo'.
	a := PyAddressOfNode new
		reference: ref;
		yourself.
		
	self assert: a children size = 1.
	self assert: a children first = ref.! !

!TestPyAddressOfNode methodsFor: 'test-instance creation' stamp: 'fbs 5/21/2009 10:58'!
testReferencing
	| a i |
	i := PyIdentNode named: 'Foo'.
	a := PyAddressOfNode referencing: i.
	
	self assert: a reference == i.! !

TestCase subclass: #TestPyAddressOfProcNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST-Tests'!

!TestPyAddressOfProcNode methodsFor: 'test-converting' stamp: 'fbs 5/21/2009 11:06'!
testAsSexpr
	| a ref sfac |
	ref := PyIdentNode named: 'Foo'.
	a := PyAddressOfProcNode new
		reference: ref;
		yourself.
	sfac := PySexprNodeFactory new.
	
	self assert: a asSexpr = (sfac buildAddressOf: a reference asOrderedCollection) asSexpr.! !

!TestPyAddressOfProcNode methodsFor: 'test-accessing' stamp: 'fbs 5/21/2009 14:49'!
testChildren
	| a ref |
	ref := PyIdentNode named: 'Foo'.
	a := PyAddressOfProcNode new
		reference: ref;
		yourself.
		
	self assert: a children size = 1.
	self assert: a children first = ref.! !

!TestPyAddressOfProcNode methodsFor: 'test-instance creation' stamp: 'fbs 5/21/2009 11:06'!
testReferencing
	| a i |
	i := PyIdentNode named: 'Foo'.
	a := PyAddressOfNode referencing: i.
	
	self assert: a reference == i.! !

TestCase subclass: #TestPyArrayTypeNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST-Tests'!

!TestPyArrayTypeNode methodsFor: 'test-converting' stamp: 'fbs 5/21/2009 12:27'!
testAsSexpr
	| at indices sfac type |
	type := PyDistinctTypeNode distinctFrom: (PyIdentNode named: 'String').
	indices := #(1 2 3).
	at := PyArrayTypeNode containing: type withIndices: indices.
	sfac := PySexprNodeFactory new.
	
	self assert: at asSexpr = (sfac buildArrayType: (type asOrderedCollection) withIndices: indices) asSexpr.! !

!TestPyArrayTypeNode methodsFor: 'test-accessing' stamp: 'fbs 5/21/2009 14:50'!
testChildren
	| at indices type |
	type := PyDistinctTypeNode distinctFrom: (PyIdentNode named: 'String').
	indices := #(1 2 3).
	at := PyArrayTypeNode containing: type withIndices: indices.
	
	self assert: at children size = 2.
	self assert: at children first = type.
	self assert: at children second = indices.! !

!TestPyArrayTypeNode methodsFor: 'test-instance creation' stamp: 'fbs 5/21/2009 11:58'!
testContainingWithIndices
	| at e t |
	e := #(1 2 3) asOrderedCollection.
	t := PyNode new.
	at := PyArrayTypeNode containing: t withIndices: e.
	
	self assert: at elementType == t.
	self assert: at indices == e.! !

TestCase subclass: #TestPyAsmBlockNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST-Tests'!

!TestPyAsmBlockNode methodsFor: 'test-converting' stamp: 'fbs 5/21/2009 14:52'!
testAsSexpr
	| asm sfac stmts |
	stmts := {PyNode new. PyNode new. PyNode new.}.
	asm := PyAsmBlockNode with: stmts.
	sfac := PySexprNodeFactory new.
	
	self assert: asm asSexpr = (sfac buildAsmBlock: (stmts asOrderedCollection)) asSexpr.! !

!TestPyAsmBlockNode methodsFor: 'test-accessing' stamp: 'fbs 5/21/2009 13:13'!
testChildren
	| asm stmts |
	stmts := {PyNode new. PyNode new. PyNode new.}.
	asm := PyAsmBlockNode with: stmts.
	
	self assert: asm children size = stmts size.
	self assert: asm children first = stmts first.
	self assert: asm children second = stmts second.
	self assert: asm children third = stmts third.! !

!TestPyAsmBlockNode methodsFor: 'test-instance creation' stamp: 'fbs 5/21/2009 13:11'!
testWith
	| asm stmts |
	stmts := #().
	asm := PyAsmBlockNode with: stmts.
	
	self assert: asm statements == stmts.! !

TestCase subclass: #TestPyAssignStatementNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST-Tests'!

!TestPyAssignStatementNode methodsFor: 'private' stamp: 'fbs 5/25/2009 19:45'!
buildAssignment
	^ PyAssignmentNode new
		lvalue: (PyIdentNode named: 'a');
		rvalue: (PyIntegerNode value: '2');
		yourself.! !

!TestPyAssignStatementNode methodsFor: 'test-converting' stamp: 'fbs 5/25/2009 19:56'!
testAsSexpr
	| as sfac |
	sfac := PySexprNodeFactory new.
	as := self buildAssignment.
	
	self assert: as asSexpr = (sfac buildAssign: as rvalue to: as lvalue) asSexpr.! !

!TestPyAssignStatementNode methodsFor: 'test-accessing' stamp: 'fbs 5/18/2009 20:55'!
testChildren
	| as |
	as := self buildAssignment.
	
	self assert: as children size = 2.
	self assert: as children first == as lvalue.
	self assert: as children second == as rvalue.! !

!TestPyAssignStatementNode methodsFor: 'test-instance creation' stamp: 'fbs 5/25/2009 19:46'!
testSetTo
	| ass lvalue rvalue |
	lvalue := PyIdentNode named: 'I'.
	rvalue := PyIntegerNode value: '0'.
	
	ass := PyAssignmentNode set: lvalue to: rvalue.
	
	self assert: ass lvalue = lvalue.
	self assert: ass rvalue = rvalue.! !

TestCase subclass: #TestPyBinaryOperatorNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST-Tests'!

!TestPyBinaryOperatorNode methodsFor: 'private' stamp: 'fbs 5/21/2009 15:03'!
buildBinaryOp: aSymbol
	^ PyBinaryOperatorNode new
		operator: aSymbol;
		leftExpression: (PyIdentNode named: 'Foo');
		rightExpression: (PyIdentNode named: 'Bar');
		yourself.! !

!TestPyBinaryOperatorNode methodsFor: 'test-converting' stamp: 'fbs 5/21/2009 14:58'!
testAsSexpr
	| b sfac |
	b := self buildBinaryOp: #+.
	sfac := PySexprNodeFactory new.
	
	self assert: b asSexpr = (sfac buildExpression: (b leftExpression asOrderedCollection) nextOperator: (b operator asOrderedCollection) nextOperand: (b rightExpression asOrderedCollection)) asSexpr.! !

!TestPyBinaryOperatorNode methodsFor: 'test-accessing' stamp: 'fbs 5/19/2009 13:30'!
testChildren
	| b |
	b := self buildBinaryOp: #+.
	
	self assert: b children size = 3.
	self assert: b children first == b leftExpression.
	self assert: b children second == b operator.
	self assert: b children third == b rightExpression.! !

TestCase subclass: #TestPyBlockNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST-Tests'!

!TestPyBlockNode methodsFor: 'test-converting' stamp: 'fbs 5/21/2009 15:06'!
testAsSexpr
	| b sfac stmts |
	stmts := {PyNode new. PyNode new. PyNode new.}.
	b := PyBlockNode with: stmts.
	sfac := PySexprNodeFactory new.
	
	self assert: b asSexpr = (sfac buildBlock: (stmts asOrderedCollection)) asSexpr.! !

!TestPyBlockNode methodsFor: 'test-accessing' stamp: 'fbs 5/21/2009 13:18'!
testChildren
	| asm stmts |
	stmts := {PyNode new. PyNode new. PyNode new.}.
	asm := PyBlockNode with: stmts.
	
	self assert: asm children size = stmts size.
	self assert: asm children first = stmts first.
	self assert: asm children second = stmts second.
	self assert: asm children third = stmts third.! !

!TestPyBlockNode methodsFor: 'test-accessing' stamp: 'fbs 5/21/2009 13:18'!
testWith
	| asm stmts |
	stmts := #().
	asm := PyBlockNode with: stmts.
	
	self assert: asm statements == stmts.! !

TestCase subclass: #TestPyCaseStatementNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST-Tests'!

!TestPyCaseStatementNode methodsFor: 'test-converting' stamp: 'fbs 5/21/2009 14:47'!
testAsSexpr
	| case condition defaultSwitch sfac switches |
	sfac := PySexprNodeFactory new.
	condition := PyIdentNode named: 'Foo'.
	defaultSwitch := PyBlockNode new.
	switches := {PyNode new. PyNode new.}.
	case := PyCaseStatementNode new
		condition: condition;
		defaultSwitch: defaultSwitch;
		switches: switches;
		yourself.
		
	self assert: case asSexpr = (sfac buildCaseStatement: condition asOrderedCollection withSwitches: switches asOrderedCollection orElse: defaultSwitch asOrderedCollection) asSexpr.! !

!TestPyCaseStatementNode methodsFor: 'test-accessing' stamp: 'fbs 5/21/2009 14:47'!
testChildren
	| case condition defaultSwitch switches |
	condition := PyIdentNode named: 'Foo'.
	defaultSwitch := PyBlockNode new.
	switches := {PyNode new. PyNode new.}.
	case := PyCaseStatementNode new
		condition: condition;
		defaultSwitch: defaultSwitch;
		switches: switches;
		yourself.
		
	self assert: case children size = 3.
	self assert: case children first = condition.
	self assert: case children second = switches.
	self assert: case children third = defaultSwitch.
		! !

TestCase subclass: #TestPyCaseSwitchNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST-Tests'!

!TestPyCaseSwitchNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 14:47'!
testAsSexpr
	| sfac statement switch values |
	sfac := PySexprNodeFactory new.
	statement := PyBlockNode new.
	values := #(1 2 3).
	switch := PyCaseSwitchNode new
		statement: statement;
		values: values;
		yourself.
		
	self assert: switch asSexpr = (sfac buildCaseSwitchOn: (values asOrderedCollection) do: statement asOrderedCollection) asSexpr.! !

!TestPyCaseSwitchNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 14:39'!
testChildren
	| statement switch values |
	statement := PyNode new.
	values := #(1 2 3).
	switch := PyCaseSwitchNode new
		statement: statement;
		values: values;
		yourself.
		
	self assert: switch children size = 2.
	self assert: switch children first = values.
	self assert: switch children second = statement.! !

TestCase subclass: #TestPyClassHelperNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST-Tests'!

!TestPyClassHelperNode methodsFor: 'test-accessing' stamp: 'fbs 5/23/2009 21:02'!
testAsSexpr
	| ch class decls parent sfac |
	class := PyIdentNode named: 'TFoo'.
	parent := PyIdentNode named: 'TBar'.
	decls := PyCollectionNode with: (PyIdentNode named: 'Foo') with: (PyIdentNode named: 'Bar').	
	ch := PyClassHelperNode
		helping: class
		parent: parent
		declarations: decls.
	sfac := PySexprNodeFactory new.
	
	self assert: ch asSexpr = (sfac
		buildClassHelperFor: class asOrderedCollection
		withParent: parent asOrderedCollection
		having: decls asOrderedCollection) asSexpr.! !

!TestPyClassHelperNode methodsFor: 'test-accessing' stamp: 'fbs 5/23/2009 21:00'!
testChildren
	| ch class decls parent |
	class := PyIdentNode named: 'TFoo'.
	parent := PyIdentNode named: 'TBar'.
	decls := PyCollectionNode with: (PyIdentNode named: 'Foo') with: (PyIdentNode named: 'Bar').	
	ch := PyClassHelperNode
		helping: class
		parent: parent
		declarations: decls.
		
	self assert: ch children size = 3.
	self assert: ch children first = class.
	self assert: ch children second = parent.
	self assert: ch children third = decls.! !

!TestPyClassHelperNode methodsFor: 'test-instance creation' stamp: 'fbs 5/23/2009 20:58'!
testHelpingParentDeclarations
	| ch class decls parent |
	class := PyIdentNode named: 'TFoo'.
	parent := PyIdentNode named: 'TBar'.
	decls := PyCollectionNode with: (PyIdentNode named: 'Foo') with: (PyIdentNode named: 'Bar').
	
	ch := PyClassHelperNode
		helping: class
		parent: parent
		declarations: decls.
		
	self assert: ch helpedClass = class.
	self assert: ch parent = parent.
	self assert: ch declarations = decls.! !

TestCase subclass: #TestPyClassNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST-Tests'!

!TestPyClassNode methodsFor: 'test-accessing' stamp: 'fbs 5/22/2009 15:24'!
testAbstract
	| class |
	class := PyClassNode new.
	
	class option: nil.
	self deny: class abstract.
	
	class option: #abstract.
	self assert: class abstract.
	
	class option: #unknownOption.
	self deny: class abstract.
	
	class option: #sealed.
	self deny: class abstract.! !

!TestPyClassNode methodsFor: 'test-converting' stamp: 'fbs 5/22/2009 13:38'!
testAsSexprBlankClass
	| class sfac |
	class := PyClassNode new.
	class superclasses add: (PyIdentNode named: 'TObject').
	sfac := PySexprNodeFactory new.
	
	self assert: class asSexpr = (sfac buildClassDecl: nil withClauses: #() withOption: nil) asSexpr.! !

!TestPyClassNode methodsFor: 'test-converting' stamp: 'fbs 5/22/2009 15:20'!
testAsSexprBlankClassWithOption
	| class sfac |
	class := PyClassNode new.
	class superclasses add: (PyIdentNode named: 'TObject').
	class beSealed.
	sfac := PySexprNodeFactory new.
	
	self assert: class asSexpr = (sfac buildClassDecl: nil withClauses: #() withOption: #sealed) asSexpr.
	
	class beAbstract.
	self assert: class asSexpr = (sfac buildClassDecl: nil withClauses: #() withOption: #abstract) asSexpr.! !

!TestPyClassNode methodsFor: 'test-converting' stamp: 'fbs 5/22/2009 15:48'!
testAsSexprClass
	| class sfac |
	class := PyClassNode new.
	class superclasses add: (PyIdentNode named: 'TFoo'); add: (PyIdentNode named: 'IListener')..
	class clauses add: (PyIdentNode named: 'fake').
	sfac := PySexprNodeFactory new.
	
	self assert:
		class asSexpr = (sfac
			buildClassDecl: class superclasses asOrderedCollection
			withClauses: {class clauses asOrderedCollection.}
			withOption: nil) asSexpr.! !

!TestPyClassNode methodsFor: 'test-converting' stamp: 'fbs 5/22/2009 15:42'!
testAsSexprClassNoClauses
	| class sfac |
	class := PyClassNode new.
	class superclasses add: (PyIdentNode named: 'TFoo'); add: (PyIdentNode named: 'IListener')..
	sfac := PySexprNodeFactory new.
	
	self assert: class asSexpr = (sfac buildClassDecl: {sfac buildIdent: 'TFoo'. sfac buildIdent: 'IListener'.} withClauses: #() withOption: nil) asSexpr.! !

!TestPyClassNode methodsFor: 'test-converting' stamp: 'fbs 5/22/2009 15:57'!
testAsSexprClassWithOption
	| class sfac |
	class := PyClassNode new.
	class superclasses add: (PyIdentNode named: 'TFoo'); add: (PyIdentNode named: 'IListener')..
	class clauses add: (PyIdentNode named: 'fake').
	class beAbstract.
	sfac := PySexprNodeFactory new.
	
	self assert:
		class asSexpr = (sfac
			buildClassDecl: class superclasses asOrderedCollection
			withClauses: {class clauses asOrderedCollection.}
			withOption: #abstract) asSexpr.! !

!TestPyClassNode methodsFor: 'test-accessing' stamp: 'fbs 5/22/2009 15:38'!
testChildren
	| class |
	class := PyClassNode new.
	
	self assert: class children size = 2.
	self assert: class children first = class superclasses.
	self assert: class children second = class clauses.
	
	class beSealed.
	self assert: class children size = 2.! !

!TestPyClassNode methodsFor: 'test-accessing' stamp: 'fbs 5/22/2009 15:25'!
testSealed
	| class |
	class := PyClassNode new.
	
	class option: nil.
	self deny: class sealed.
	
	class beAbstract.
	self deny: class sealed.
	
	class option: #unknownOption.
	self deny: class sealed.
	
	class beSealed.
	self assert: class sealed.! !

TestCase subclass: #TestPyClassTypeNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST-Tests'!

!TestPyClassTypeNode methodsFor: 'test-accessing' stamp: 'fbs 5/25/2009 14:38'!
testAsSexpr
	| class classType sfac |
	class := PyClassNode new.
	classType := PyClassTypeNode of: class.
	sfac := PySexprNodeFactory new.
	
	self assert: classType asSexpr = (sfac buildClassTypeRef: class asOrderedCollection) asSexpr.! !

!TestPyClassTypeNode methodsFor: 'test-accessing' stamp: 'fbs 5/25/2009 14:36'!
testChildren
	| class classType |
	class := PyClassNode new.
	classType := PyClassTypeNode of: class.
	
	self assert: classType children size = 1.
	self assert: classType children first = class.! !

!TestPyClassTypeNode methodsFor: 'test-instance creation' stamp: 'fbs 5/25/2009 14:34'!
testOf
	| class classType |
	class := PyClassNode new.
	classType := PyClassTypeNode of: class.
	
	self assert: classType classType = class.! !

TestCase subclass: #TestPyCodeWalker
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-Analysis-Tests'!

!TestPyCodeWalker methodsFor: 'test-enumerating' stamp: 'fbs 5/19/2009 14:07'!
testWalkAndTerminate
	| ast w |
	w := PyNullWalker new.
	ast := PyAssignmentNode new
		lvalue: PyNode new;
		rvalue: PyNode new;
		yourself.
	
	w walk: ast.
	self assert: w output isNil.
! !

!TestPyCodeWalker methodsFor: 'test-enumerating' stamp: 'fbs 5/19/2009 13:34'!
testWalkNestedNode
	| n w |
	w := PyObjectNamer new.
	n := PyAssignmentNode new
		lvalue: PyNode new;
		rvalue: PyNode new;
		yourself..
	
	w walk: n.
	self assert: w output = (n class asString, ' ', n lvalue class asString, ' ', n rvalue class asString).! !

!TestPyCodeWalker methodsFor: 'test-enumerating' stamp: 'fbs 5/19/2009 13:55'!
testWalkNestedNodes
	| e w |
	w := PyObjectNamer new.
	"e: (a * b) + (c / d)"
	e := PyBinaryOperatorNode new
		leftExpression: (PyBinaryOperatorNode new operator: (PyOperatorNode new operator: #*; yourself); leftExpression: PyNode new; rightExpression: PyNode new);
		operator: (PyOperatorNode new operator: #+; yourself);		
		rightExpression: (PyBinaryOperatorNode new operator: (PyOperatorNode new operator: #/; yourself); leftExpression: PyNode new; rightExpression: PyNode new);
		yourself.
	
	w walk: e.
	self assert: w output = (e class asString, ' ', e leftExpression class asString, ' ', e leftExpression leftExpression class asString, ' ', e leftExpression operator class asString, ' ', e leftExpression rightExpression class asString, ' ', e operator class asString, ' ', e rightExpression class asString, ' ', e rightExpression leftExpression class asString, ' ', e rightExpression operator class asString, ' ', e rightExpression rightExpression class asString)! !

!TestPyCodeWalker methodsFor: 'test-enumerating' stamp: 'fbs 5/19/2009 13:02'!
testWalkSimpleNode
	| n w |
	w := PyObjectNamer new.
	n := PyNode new.
	
	w walk: n.
	self assert: w output = n class asString.! !

TestCase subclass: #TestPyCollectionNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST-Tests'!

!TestPyCollectionNode methodsFor: 'test-converting' stamp: 'fbs 5/22/2009 13:40'!
testAsOrderedCollection
	| c |
	c := PyCollectionNode new.
	
	self assert: c asOrderedCollection = OrderedCollection new.

	c
		add: (PyIdentNode named: 'foo');
		add: (PyIdentNode named: 'bar');
		add: (PyIdentNode named: 'baz').
	
	self assert: c asOrderedCollection = (OrderedCollection with: (c first asOrderedCollection) with: (c second asOrderedCollection) with: (c third asOrderedCollection)).! !

!TestPyCollectionNode methodsFor: 'test-converting' stamp: 'fbs 5/22/2009 14:51'!
testAsSexpr
	| c |
	c := PyCollectionNode new
		add: (PyIdentNode named: 'Foo');
		add: (PyIdentNode named: 'Bar');
		yourself.
		
	self assert:
		c asSexpr = (OrderedCollection
			with: (PyIdentNode named: 'Foo') asOrderedCollection
			with: (PyIdentNode named: 'Bar') asOrderedCollection) asSexpr.
	self assert: PyCollectionNode new asSexpr = #() asSexpr.! !

!TestPyCollectionNode methodsFor: 'test-accessing' stamp: 'fbs 5/19/2009 15:46'!
testChildren
	| c |
	c := PyCollectionNode new.
	
	self assert: c children isEmpty.
	
	c items add: 1.
	c items add: 2.
	c items add: 3.
	self assert: c children = c items.! !

TestCase subclass: #TestPyCommentNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST-Tests'!

!TestPyCommentNode methodsFor: 'test-converting' stamp: 'fbs 5/25/2009 15:20'!
testAsSexpr
	| comment sfac |
	comment := PyCommentNode saying: 'I am a comment'.
	sfac := PySexprNodeFactory new.
	
	self assert: comment asSexpr = (sfac buildComment: comment comment) asSexpr.! !

!TestPyCommentNode methodsFor: 'test-instance creation' stamp: 'fbs 5/25/2009 15:20'!
testComment
	| comment text |
	text := 'I am not a comment'.
	comment := PyCommentNode saying: text.
	
	self assert: comment comment = text.! !

TestCase subclass: #TestPyCompoundStatementNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST-Tests'!

!TestPyCompoundStatementNode methodsFor: 'test-accessing' stamp: 'fbs 5/25/2009 19:52'!
testChildren
	| block coll i zero |
	i := PyIdentNode named: 'i'.
	zero := PyIntegerNode value: '0'.
	coll := PyCollectionNode
		with: (PyAssignmentNode set: i to: zero)
		with: (PyAssignmentNode set: i to: zero).
		
	block := PyCompoundStatementNode newFrom: coll.
	
	self assert: block children size = coll size.
	self assert: block children first = coll first.
	self assert: block children second = coll second.! !

!TestPyCompoundStatementNode methodsFor: 'test-instance creation' stamp: 'fbs 5/25/2009 19:49'!
testNewFrom
	| block coll i zero |
	i := PyIdentNode named: 'i'.
	zero := PyIntegerNode value: '0'.
	coll := PyCollectionNode
		with: (PyAssignmentNode set: i to: zero)
		with: (PyAssignmentNode set: i to: zero).
		
	block := PyCompoundStatementNode newFrom: coll.

	self assert: block size = 2.
	self assert: block first lvalue = i.
	self assert: block second lvalue = i.! !

TestCase subclass: #TestPyDirectiveNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST-Tests'!

!TestPyDirectiveNode methodsFor: 'test-accessing' stamp: 'fbs 5/22/2009 21:32'!
testAsSexpr
	| d sfac |
	d := PyDirectiveNode experimental.
	sfac := PySexprNodeFactory new.

	self assert: d asSexpr = (sfac buildDirective: d name) asSexpr.
	
	d value: (PyIntegerNode value: '123').
	
	self assert: d asSexpr = (sfac buildDirective: d name withValue: d value literalValue) asSexpr.! !

!TestPyDirectiveNode methodsFor: 'test-accessing' stamp: 'fbs 5/21/2009 15:13'!
testChildren
	| d value |
	d := PyDirectiveNode new
		name: 'absolute';
		yourself.
	value := PyIntegerNode value: '1'.		
	
	self assert: d children isEmpty.
	
	d value: value.
	
	self assert: d children size = 1.
	self assert: d children first = value.! !

!TestPyDirectiveNode methodsFor: 'test-instance creation' stamp: 'fbs 5/22/2009 20:03'!
testDeprecated
	self assert: (PyDirectiveNode deprecated) deprecated.! !

!TestPyDirectiveNode methodsFor: 'test-instance creation' stamp: 'fbs 5/22/2009 20:03'!
testExperimental
	self assert: (PyDirectiveNode experimental) experimental.! !

!TestPyDirectiveNode methodsFor: 'test-instance creation' stamp: 'fbs 5/22/2009 20:03'!
testLibrary
	self assert: (PyDirectiveNode library) library.! !

!TestPyDirectiveNode methodsFor: 'test-instance creation' stamp: 'fbs 5/22/2009 20:00'!
testNamed
	| d name |
	name := 'foo'.
	d := PyDirectiveNode named: name.
	
	self assert: d name = name.! !

!TestPyDirectiveNode methodsFor: 'test-instance creation' stamp: 'fbs 5/22/2009 20:04'!
testPlatform
	self assert: (PyDirectiveNode platform) platform.! !

TestCase subclass: #TestPyDistinctTypeNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST-Tests'!

!TestPyDistinctTypeNode methodsFor: 'test-converting' stamp: 'fbs 5/21/2009 15:14'!
testAsSexpr
	| referringType sfac type |
	referringType := PyIdentNode named: 'String'.
	type := PyDistinctTypeNode distinctFrom: referringType.
	sfac := PySexprNodeFactory new.
	
	self assert: type asSexpr = (sfac buildType: (referringType asOrderedCollection) isDistinct: true) asSexpr.! !

!TestPyDistinctTypeNode methodsFor: 'test-accessing' stamp: 'fbs 5/21/2009 12:37'!
testChildren
	| referringType type |
	referringType := PyIdentNode named: 'String'.
	type := PyDistinctTypeNode distinctFrom: referringType.
	
	self assert: type children size = 1.
	self assert: type children first == referringType.! !

!TestPyDistinctTypeNode methodsFor: 'test-instance creation' stamp: 'fbs 5/21/2009 12:37'!
testDistinctFrom
	| referringType type |
	referringType := PyIdentNode named: 'String'.
	type := PyDistinctTypeNode distinctFrom: referringType.
	
	self assert: type referringType == referringType.! !

TestCase subclass: #TestPyGenericTypeNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST-Tests'!

TestCase subclass: #TestPyIdentNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST-Tests'!

!TestPyIdentNode methodsFor: 'test-converting' stamp: 'fbs 5/21/2009 10:45'!
testAsSexpr
	| i sfac |
	i := PyIdentNode new name: 'Foo'; yourself.
	sfac := PySexprNodeFactory new.
	
	self assert: i asSexpr = (sfac buildIdent: i name) asSexpr.! !

!TestPyIdentNode methodsFor: 'test-accessing' stamp: 'fbs 5/21/2009 10:53'!
testChildren
	| i |
	i := PyIdentNode named: 'Foo'.
	
	self assert: i children isEmpty.! !

!TestPyIdentNode methodsFor: 'test-instance creation' stamp: 'fbs 5/21/2009 10:48'!
testNamed
	| name |
	name := 'Foo'.
	
	self assert: name = (PyIdentNode named: name) name.! !

TestCase subclass: #TestPyIfStatementNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST-Tests'!

!TestPyIfStatementNode methodsFor: 'private' stamp: 'fbs 5/18/2009 20:49'!
buildIf
	^ PyIfStatementNode new
		condition: 'a';
		ifBlock: 'b';
		elseBlock: 'c';
		yourself.! !

!TestPyIfStatementNode methodsFor: 'test-converting' stamp: 'fbs 5/18/2009 20:41'!
testAsSexpr
	| if sfac |
	sfac := PySexprNodeFactory new.
	if := self buildIf.
	
	self assert: if asSexpr = (sfac buildIf: if condition then: if ifBlock else: if elseBlock).! !

!TestPyIfStatementNode methodsFor: 'test-accessing' stamp: 'fbs 5/18/2009 20:39'!
testChildren
	| if |
	if := self buildIf.
	
	self assert: if children size = 3.
	self assert: if children first == if condition.
	self assert: if children second == if ifBlock.
	self assert: if children third == if elseBlock..! !

TestCase subclass: #TestPyIntegerNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST-Tests'!

!TestPyIntegerNode methodsFor: 'test-converting' stamp: 'fbs 5/25/2009 13:29'!
testAsSexpr
	| i sfac |
	i := PyIntegerNode value: '123'.
	sfac := PySexprNodeFactory new.
	
	self assert: i asSexpr = (sfac buildIntNum: i literalValue) asSexpr.! !

!TestPyIntegerNode methodsFor: 'test-accessing' stamp: 'fbs 5/21/2009 09:40'!
testChildren
	| i |
	i := PyIntegerNode value: '123'.
	
	self assert: i children isEmpty.! !

TestCase subclass: #TestPyNamedCollectionNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST-Tests'!

!TestPyNamedCollectionNode methodsFor: 'test-converting' stamp: 'fbs 5/21/2009 11:23'!
testAsSexpr
	| c e sfac |
	e := {1. 2. 3.}.
	c := PyNamedCollectionNode named: #arrayIndex withElements: e.
	sfac := PySexprNodeFactory new.
	
	self assert: c asSexpr = (sfac namedList: c name withElements: c items) asSexpr.! !

!TestPyNamedCollectionNode methodsFor: 'test-accessing' stamp: 'fbs 5/21/2009 15:29'!
testChildren
	| c e |
	e := {1. 2. 3.}.
	c := PyNamedCollectionNode named: #arrayIndex withElements: e.
	
	self assert: c children size = (e size + 1).
	self assert: c children first = c name.
	self assert: c children second = e first.
	self assert: c children third = e second.
	self assert: c children fourth = e third.! !

!TestPyNamedCollectionNode methodsFor: 'test-copying' stamp: 'fbs 5/25/2009 13:29'!
testCopy
	| c e new |
	e := {1. 2. 3.}.
	c := PyNamedCollectionNode named: #arrayIndex withElements: e.
	
	new := c copy.

	self assert: new size = c size.
	self assert: new first = c first.
	self assert: new second = c second.
	self assert: new third = c third.! !

!TestPyNamedCollectionNode methodsFor: 'test-instance creation' stamp: 'fbs 5/21/2009 11:20'!
testNamedWithElements
	| c e name |
	e := OrderedCollection with: PyNode new.
	name := #arrayIndex.
	c := PyNamedCollectionNode named: name withElements: e.
	
	self assert: name = c name.
	self assert: e = c items.! !

TestCase subclass: #TestPyNodeFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-Tests'!

!TestPyNodeFactory methodsFor: 'test-testing' stamp: 'fbs 1/5/2009 21:37'!
testIsList
	| f |
	f := PyNodeFactory new.
	
	self assert: (f isList: #((1) (2))).
	self assert: (f isList: #()).
	self deny: (f isList: 1).
	self deny: (f isList: #(1 2)).
	self deny: (f isList: #(procedureMethodDecl (ident 'Foo') (params) (directives)))! !

TestCase subclass: #TestPyObjectNodeFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-Tests'!

!TestPyObjectNodeFactory methodsFor: 'test-building' stamp: 'fbs 5/21/2009 10:59'!
testBuildAddressOf
	| a fac i |
	fac := PyObjectNodeFactory new.
	i := PyIdentNode named: 'Foo'.
	
	a := fac buildAddressOf: i.
	self assert: a reference == i.! !

!TestPyObjectNodeFactory methodsFor: 'test-building' stamp: 'fbs 5/21/2009 11:05'!
testBuildAddressOfProc
	| a fac i |
	fac := PyObjectNodeFactory new.
	i := PyIdentNode named: 'Foo'.
	
	a := fac buildAddressOfProc: i.
	self assert: a reference == i.! !

!TestPyObjectNodeFactory methodsFor: 'test-building' stamp: 'fbs 5/21/2009 11:30'!
testBuildArrayIndex
	| a e fac |
	e := #(1 2 3) asOrderedCollection.
	fac := PyObjectNodeFactory new.
	a := fac buildArrayIndex: e.
	
	self assert: a name = #arrayIndex.
	self assert: a items = e.! !

!TestPyObjectNodeFactory methodsFor: 'test-building' stamp: 'fbs 5/21/2009 13:32'!
testBuildArrayTypeWithIndices
	| arrType indices fac type |
	indices := #(1 2 3) asOrderedCollection.
	type := PyNode new.
	fac := PyObjectNodeFactory new.
	arrType := fac buildArrayType: type withIndices: indices.
	
	self assert: arrType elementType == type.
	self assert: arrType indices == indices.! !

!TestPyObjectNodeFactory methodsFor: 'test-building' stamp: 'fbs 5/21/2009 13:33'!
testBuildAsmBlock
	| asm fac stmts |
	fac := PyObjectNodeFactory new.
	stmts := #(1 2 3).
	asm := fac buildAsmBlock: stmts.
	
	self assert: asm statements = stmts.! !

!TestPyObjectNodeFactory methodsFor: 'test-building' stamp: 'fbs 5/21/2009 10:56'!
testBuildAssignTo
	| a fac lvalue rvalue |
	fac := PyObjectNodeFactory new.
	lvalue := PyIdentNode named: 'Foo'.
	rvalue := PyIntegerNode value: '1'..
	a := fac buildAssign: rvalue to: lvalue.
	
	self assert: a rvalue == rvalue.
	self assert: a lvalue == lvalue.! !

!TestPyObjectNodeFactory methodsFor: 'test-building' stamp: 'fbs 5/21/2009 13:39'!
testBuildBlock
	| block fac stmts |
	fac := PyObjectNodeFactory new.
	stmts := #(1 2 3).
	block := fac buildBlock: stmts.
	
	self assert: block statements = stmts.! !

!TestPyObjectNodeFactory methodsFor: 'test-building' stamp: 'fbs 5/21/2009 13:42'!
testBuildBlockWithDeclarations
	| block decls fac stmts |
	fac := PyObjectNodeFactory new.
	decls := #(1 2 3).
	stmts := #(1 2 3).
	block := fac buildBlock: stmts withDeclarations: decls.
	
	self assert: block first = decls.
	self assert: block second = stmts.! !

!TestPyObjectNodeFactory methodsFor: 'test-building' stamp: 'fbs 5/21/2009 14:35'!
testBuildCaseStatementWithSwitchesOrElse
	| case condition defaultSwitch fac switches |
	fac := PyObjectNodeFactory new.
	condition := PyNode new.
	defaultSwitch := PyNode new.
	switches := {PyNode new. PyNode new.}.
	
	case := fac buildCaseStatement: condition withSwitches: switches orElse: defaultSwitch.
	
	self assert: case condition = condition.
	self assert: case switches = switches.
	self assert: case defaultSwitch = defaultSwitch.! !

!TestPyObjectNodeFactory methodsFor: 'test-building' stamp: 'fbs 5/21/2009 15:31'!
testBuildCaseSwitchOnDo
	| fac statement switch values |
	fac := PyObjectNodeFactory new.
	statement := PyBlockNode new.
	values := #(1 2 3).
	switch := fac buildCaseSwitchOn: values do: statement.
	
	self assert: switch statement = statement.
	self assert: switch values = values.
		
	! !

!TestPyObjectNodeFactory methodsFor: 'test-building' stamp: 'fbs 5/22/2009 15:16'!
testBuildClassDeclWithClausesWithNoOptionBlankSuperclassNoDecls
	| class fac |
	fac := PyObjectNodeFactory new.
	
	class := fac buildClassDecl: nil withClauses: #() withOption: nil.
	
	self assert: class superclasses size = 1.
	self assert: class superclasses first name = 'TObject'.
	self assert: class clauses isEmpty.
	self assert: class option isNil.! !

!TestPyObjectNodeFactory methodsFor: 'test-building' stamp: 'fbs 5/22/2009 15:25'!
testBuildClassDeclWithClausesWithOptionBlankSuperclassNoDecls
	| class fac |
	fac := PyObjectNodeFactory new.
	
	class := fac buildClassDecl: nil withClauses: #() withOption: #sealed.
	
	self assert: class superclasses size = 1.
	self assert: class superclasses first name = 'TObject'.
	self assert: class clauses isEmpty.
	self assert: class sealed.! !

!TestPyObjectNodeFactory methodsFor: 'test-building' stamp: 'fbs 5/22/2009 20:58'!
testBuildClassFieldOfTypeHinting
	| coll fac hint idents someType |
	fac := PyObjectNodeFactory new.
	hint := PyDirectiveNode experimental.
	idents := PyCollectionNode with: (PyIdentNode named: 'Foo') with: (PyIdentNode named: 'Bar').
	someType := PyIdentNode named: 'Integer'.
	coll := fac buildClassField: idents ofType: someType hinting: hint.
	
	self assert: coll size = 2.
	self assert: coll first name = idents first name.
	self assert: coll second name = idents second name.
	
	coll do:
		[:each|
		self assert: each type = someType.
		self assert: each hintingDirective = hint.]! !

!TestPyObjectNodeFactory methodsFor: 'test-building' stamp: 'fbs 5/25/2009 12:25'!
testBuildClassHelperForWithParentHaving
	| declarations fac helpedClass helper parent |
	declarations := PyCollectionNode new.
	fac := PyObjectNodeFactory new.
	helpedClass := PyClassNode new.
	parent := PyClassNode new.
	
	helper := fac buildClassHelperFor: helpedClass withParent: parent having: declarations.
	
	self assert: helper declarations = declarations.
	self assert: helper helpedClass = helpedClass.
	self assert: helper parent = parent.! !

!TestPyObjectNodeFactory methodsFor: 'test-building' stamp: 'fbs 5/25/2009 14:23'!
testBuildClassProperty
	| fac prop property |
	fac := PyObjectNodeFactory new.
	prop := PyPropertyNode new
		name: (PyIdentNode named: 'Foo');
		type: (PyIdentNode named: 'TFoo');
		specifiers: (PyCollectionNode new);
		yourself.
	property := fac buildClassProperty: prop.
	
	self assert: property name = prop name.
	self assert: property type = prop type.
	self assert: property specifiers size = prop specifiers size.! !

!TestPyObjectNodeFactory methodsFor: 'test-building' stamp: 'fbs 5/25/2009 14:32'!
testBuildClassTypeRef
	| class classType fac |
	fac := PyObjectNodeFactory new.
	class := PyClassNode new.
	classType := fac buildClassTypeRef: class.
	
	self assert: classType classType = class.
	
	! !

!TestPyObjectNodeFactory methodsFor: 'test-building' stamp: 'fbs 5/25/2009 19:30'!
testBuildClassVarClause
	| classvars clause fac |
	clause := PyVariableClauseNode new.
	clause add: (PyVariableNode name: (PyIdentNode named: 'Foo') type: (PyIdentNode named: 'TFoo')).
	fac := PyObjectNodeFactory new.	
	classvars := fac buildClassVarClause: clause.

	self assert: classvars size = 1.
	self assert: classvars first name = clause first name.
	self assert: classvars first type = clause first type.
	self assert: classvars first hintingDirective = clause first hintingDirective.! !

!TestPyObjectNodeFactory methodsFor: 'test-building' stamp: 'fbs 5/25/2009 15:54'!
testBuildClassVarClauseEmptyClause
	| classvars clause fac |
	clause := PyVariableClauseNode new.
	fac := PyObjectNodeFactory new.
	
	classvars := fac buildClassVarClause: clause.
	
	self assert: classvars isEmpty.! !

!TestPyObjectNodeFactory methodsFor: 'test-building' stamp: 'fbs 5/25/2009 15:21'!
testBuildComment
	| fac comment text |
	text := 'I am not a comment'.
	fac := PyObjectNodeFactory new.
	comment := fac buildComment: text.
	
	self assert: comment comment = text.! !

!TestPyObjectNodeFactory methodsFor: 'test-building' stamp: 'fbs 5/25/2009 19:42'!
testBuildCompoundStatement
	| block e1 e2 f fac i sfac |
	f := PyIdentNode named: 'Foo'.
	i := PyIntegerNode value: '1'.
	e1 := PyAssignmentNode set: f to: i.
	e2 := PyAssignmentNode set: f to: i.
	fac := PyObjectNodeFactory new.
	sfac := PySexprNodeFactory new.
	
	block := fac buildCompoundStatement: (PyCollectionNode with: e1 with: e2).
	
	self assert: block asSexpr = (sfac buildCompoundStatement: {block first. block second.}) asSexpr.! !

!TestPyObjectNodeFactory methodsFor: 'test-building' stamp: 'fbs 5/22/2009 14:16'!
testBuildIdent
	| fac ident name |
	fac := PyObjectNodeFactory new.
	name := 'Foo'.
	
	ident := fac buildIdent: name.
	
	self assert: ident name = name.
	! !

!TestPyObjectNodeFactory methodsFor: 'test-building' stamp: 'fbs 5/19/2009 10:58'!
testBuildIfThen
	| block condition if fac |
	fac := PyObjectNodeFactory new.
	block := Object new.
	condition := Object new.
	if := fac buildIf: condition then: block.
	
	self assert: if condition == condition.
	self assert: if ifBlock == block.
	self assert: if elseBlock isNil.! !

!TestPyObjectNodeFactory methodsFor: 'test-building' stamp: 'fbs 5/19/2009 10:58'!
testBuildIfThenElse
	| block condition if fac otherBlock |
	fac := PyObjectNodeFactory new.
	block := Object new.
	condition := Object new.
	otherBlock := Object new.
	if := fac buildIf: condition then: block else: otherBlock.
	
	self assert: if condition == condition.
	self assert: if ifBlock == block.
	self assert: if elseBlock == otherBlock.! !

!TestPyObjectNodeFactory methodsFor: 'test-building' stamp: 'fbs 5/25/2009 19:32'!
testBuildInstVarClause
	| instvars clause fac |
	clause := PyVariableClauseNode new.
	clause add: (PyVariableNode name: (PyIdentNode named: 'Foo') type: (PyIdentNode named: 'TFoo')).
	fac := PyObjectNodeFactory new.	
	instvars := fac buildInstVarClause: clause.

	self assert: instvars size = 1.
	self assert: instvars first name = clause first name.
	self assert: instvars first type = clause first type.
	self assert: instvars first hintingDirective = clause first hintingDirective.! !

!TestPyObjectNodeFactory methodsFor: 'test-building' stamp: 'fbs 5/19/2009 10:59'!
testBuildStatement
	| fac stmt |
	fac := PyObjectNodeFactory new.
	stmt := Object new.

	self assert: (fac buildStatement: stmt) == stmt.! !

!TestPyObjectNodeFactory methodsFor: 'test-building' stamp: 'fbs 5/19/2009 12:00'!
testBuildStatementLabelled
	| fac label s stmt |
	fac := PyObjectNodeFactory new.
	stmt := PyStatementNode new.
	label := Object new.
	
	s := fac buildStatement: stmt labelled: label.

	self assert: s label == label.! !

!TestPyObjectNodeFactory methodsFor: 'test-building' stamp: 'fbs 5/19/2009 15:14'!
testBuildUnitWithInterfaceWithImplementationWithBlock
	| code fac implementation interface u unitHead |
	fac := PyObjectNodeFactory new.
	unitHead := PyNode new.
	interface := PyNode new.
	implementation := PyNode new.
	code := PyNode new.	
	u := fac buildUnit: unitHead withInterface: interface withImplementation: implementation withBlock: code.
	
	self assert: u head == unitHead.
	self assert: u interface == interface.
	self assert: u implementation == implementation.
	self assert: u code == code.! !

!TestPyObjectNodeFactory methodsFor: 'test-building' stamp: 'fbs 5/25/2009 19:37'!
testIntNum
	| fac i value |
	fac := PyObjectNodeFactory new.
	value := '123'.
	i := fac buildIntNum: value.
	
	self assert: i literalValue = value.! !

TestCase subclass: #TestPyOperatorNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST-Tests'!

!TestPyOperatorNode methodsFor: 'private' stamp: 'fbs 5/19/2009 13:45'!
buildOp
	^ PyOperatorNode new operator: #+; yourself.! !

!TestPyOperatorNode methodsFor: 'test-converting' stamp: 'fbs 5/19/2009 13:46'!
testAsSexpr
	| op |
	op := self buildOp.
	
	self assert: op asSexpr = op operator asSexpr.! !

!TestPyOperatorNode methodsFor: 'test-accessing' stamp: 'fbs 5/19/2009 13:44'!
testChildren
	| op |
	op := self buildOp.
	
	self assert: op children isEmpty.! !

TestCase subclass: #TestPyParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-Tests'!

!TestPyParser methodsFor: 'private' stamp: 'fbs 11/28/2008 21:32'!
assertEquals: seqA collection: seqB
	self assertEquals: seqA collection: seqB description: ''.! !

!TestPyParser methodsFor: 'private' stamp: 'fbs 12/4/2008 09:49'!
assertEquals: expected collection: received description: aString
	"Assert that SequenceableCollections expected, receive each contain equal (not necessarily identical) data, recursively."
	
	self assert: expected isCollection = received isCollection description: aString.
	
	expected isCollection ifFalse:
		[^ self assert: expected = received description: aString].

	self assert: (expected size = received size) description: aString.
	
	1 to: expected size do:
		[:i | self
			assertEquals: (expected at: i)
			collection: (received at: i)
			description: aString].! !

!TestPyParser methodsFor: 'private' stamp: 'fbs 12/12/2008 22:54'!
assertIdentity: aString using: aProduction
	self assertParsesAs: aString from: aString using: aProduction.
! !

!TestPyParser methodsFor: 'private' stamp: 'fbs 12/10/2008 14:05'!
assertParseFailFrom: aString using: aProduction
	self assert: ((self parse: aString using: aProduction) isKindOf: OMetaFail)
		description: 'Parsing ''', aString, ''''.! !

!TestPyParser methodsFor: 'private' stamp: 'fbs 1/9/2009 16:30'!
assertParses: aString using: aProduction.
	self deny: ((self parse: aString using: aProduction) isKindOf: OMetaFail) description: 'Parsing ''', aString, ''''.
	! !

!TestPyParser methodsFor: 'private' stamp: 'fbs 12/19/2008 10:06'!
assertParsesAs: aCollectionOrOther from: aString using: aProduction
	self
		assertParsesAs: aCollectionOrOther
		from: aString
		using: aProduction
		description: 'Parsing ''', aString, ''''.! !

!TestPyParser methodsFor: 'private' stamp: 'fbs 11/28/2008 21:31'!
assertParsesAs: aCollectionOrOther from: aString using: aProduction description: aStringDesc
	self
		assertEquals: aCollectionOrOther
		collection: (self parse: aString using: aProduction)
		description: aStringDesc.! !

!TestPyParser methodsFor: 'private' stamp: 'fbs 12/19/2008 10:06'!
assertParsesAs: aCollectionOrOther from: aString using: aProduction withArguments: someArgs
	self
		assertParsesAs: aCollectionOrOther
		from: aString
		using: aProduction
		withArguments: someArgs
		description: 'Parsing ''', aString, ''''.! !

!TestPyParser methodsFor: 'private' stamp: 'fbs 12/19/2008 10:06'!
assertParsesAs: aCollectionOrOther from: aString using: aProduction withArguments: someArgs description: aStringDesc
	self
		assertEquals: aCollectionOrOther
		collection: (self parse: aString using: aProduction withArguments: someArgs)
		description: aStringDesc.! !

!TestPyParser methodsFor: 'private' stamp: 'fbs 11/14/2008 22:23'!
parse: aString using: aProduction
	^ (PyParser onString: aString) apply: aProduction.! !

!TestPyParser methodsFor: 'private' stamp: 'fbs 12/19/2008 10:07'!
parse: aString using: aProduction withArguments: someArgs
	^ (PyParser onString: aString) apply: aProduction withArguments: someArgs.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/3/2009 15:25'!
testArrayProperty
	self assertParsesAs: #(property (ident 'Foo') (params (param (ident 'Index') (ident 'Integer'))) (ident 'Integer') (propertySpecifiers)) from: 'property Foo[Index: Integer]: Integer;' using: #classProperty.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/7/2009 15:07'!
testArrayType
	self assertParsesAs: #(arrayType (ident 'Foo') ((range (intNum '0') (intNum '1')))) from: 'array[0..1] of Foo' using: #arrayType.
	self assertParsesAs: #(arrayType const ((range (intNum '0') (intNum '1')) (range (intNum '1') (intNum '2')))) from: 'array[0..1, 1..2] of const' using: #arrayType.
	self assertParsesAs: #(arrayType (ident 'Foo') ((ident 'Bar'))) from: 'array[Bar] of Foo' using: #arrayType.
	
	self assertParsesAs: #(arrayType (ident 'Foo') ((ident 'Bar') (range (intNum '0') (intNum '1')))) from: 'array[Bar, 0..1] of Foo' using: #arrayType.
	
	self assertParsesAs: #(arrayType (ident 'Foo') ((range (funcall (ident 'Low') (args (ident 'Integer'))) (funcall (ident 'High') (args (ident 'Integer')))))) from: 'array[Low(Integer)..High(Integer)] of Foo' using: #arrayType.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/9/2009 17:23'!
testAssemblerStatement
	self assertParses: 'asm mul edx end;' using: #assemblerStatement.
	self assertParses: 'asm @end: end;' using: #assemblerStatement.
	self assertParses: 'asm mov esi, edx end;' using: #assemblerStatement.
	self assertParses:
		'asm', String crlf,
		'  mul edx', String crlf,
		'  jno @end', String crlf,
		'  call System.@IntOver', String crlf,
		' @end:', String crlf,
		'end;', String crlf
		using: #assemblerStatement.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/9/2009 16:30'!
testAssemblyStatement
	self assertParses: 'mul edx' using: #assemblyStatement.
	self assertParses: 'jno @end' using: #assemblyStatement.
	self assertParses: 'call System.@IntOver' using: #assemblyStatement.
	self assertParses: '@end:' using: #assemblyStatement.
	self assertParses: 'mov esi, edx' using: #assemblyStatement.
	self assertParses: 'mov [eax], ebx' using: #assemblyStatement.
	self assertParses: 'mov WORD PTR [eax], bx' using: #assemblyStatement.! !

!TestPyParser methodsFor: 'test-asserts' stamp: 'fbs 11/28/2008 16:14'!
testAssertEqualsCollection
	self assertEquals: nil collection: nil.
	self assertEquals: 1 collection: 1.
	self assertEquals: #() collection: #().
	self assertEquals: #() collection: #() asOrderedCollection.
	self assertEquals: #(#()) collection: #(#()).
	self assertEquals: #(1 #(2 '3') #4) collection: #(1 #(2 '3') #4).
	
	self should: [self assertEquals: #() collection: #(#())] raise: TestResult failure.
	self should: [self assertEquals: #(1 #(2 3) 4) collection: #(1 #(2 '3') 4)] raise: TestResult failure.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/7/2009 16:34'!
testBlock
	self assertParsesAs: #(() (block)) from: 'begin end' using: #block.
	self assertParsesAs: #(((procedureImpl (procedureDecl (ident 'Foo') (params) (directives)) () (block))) (block (ident 'foo') (ident 'bar'))) from: 'procedure Foo; begin end; begin foo; bar end' using: #block.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/30/2008 20:20'!
testBlockBody
	self assertParsesAs: #(block) from: 'begin end' using: #blockBody.
	self assertParsesAs: #(block #(ident 'foo') #(ident 'bar')) from: 'begin foo; bar end' using: #blockBody.
	self assertParsesAs: #(block #(ident 'foo') #(ident 'bar')) from: 'begin foo; bar; end' using: #blockBody.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/19/2008 23:20'!
testCallConvention
	#('cdecl' 'far' 'local' 'near' 'pascal' 'register' 'safecall' 'stdcall' 'export') do:
		[:each |
		self assertParsesAs: {#directive. each.} from: each, ';' using: #callConvention.]! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/30/2008 13:11'!
testCaseItem
	self assertParsesAs: #(switch ((range (intNum '1') (intNum '2')) (intNum '3')) (if #true (ident 'Foo') ())) from: '1..2, 3: if true then Foo' using: #caseItem.
	self assertParsesAs: #(switch ((range (intNum '1') (intNum '2')) (intNum '3')) (if #true (ident 'Foo') ())) from: '1..2, 3: if true then Foo;' using: #caseItem.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 5/16/2009 11:13'!
testCaseStatement
	self assertParsesAs: #(case (ident 'Foo') ((switch ((intNum '1')) ())) ()) from: 'case Foo of 1:end' using: #caseStatement.
	
	self assertParsesAs: #(case (ident 'Quux') ((switch ((intNum '1')) (ident 'Foo')) (switch ((intNum '2')) (ident 'Bar'))) ()) from: 'case Quux of 1: Foo; 2: Bar; end;' using: #caseStatement.
	self assertParsesAs: #(case (ident 'Quux') ((switch ((intNum '1')) (ident 'Foo')) (switch ((intNum '2')) (ident 'Bar'))) ()) from: 'case Quux of 1: Foo; 2: Bar end;' using: #caseStatement.

	self assertParsesAs: #(case (ident 'Quux') ((switch ((intNum '1')) (ident 'Foo')) (switch ((intNum '2')) (ident 'Bar'))) (elseBlock (ident 'Baz'))) from: 'case Quux of 1: Foo; 2: Bar; else Baz end;' using: #caseStatement.
	
	self assertParsesAs: #(case (ident 'Foo') ((switch ((intNum '1')) ())) (elseBlock)) from: 'case Foo of 1: else end' using: #caseStatement description: 'case with empty else clause'..
	self assertParsesAs: #(case (ident 'Foo') ((switch ((intNum '1')) ())) (elseBlock)) from: 'case Foo of 1: else end;' using: #caseStatement description: 'case with semicolon else clause'.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/2/2009 21:33'!
testClassBody
	self assertParsesAs: #((instvar (ident 'Foo') (ident 'Integer') (directives) (visibility private))) from: 'private Foo: Integer;' using: #classBody.
	self assertParsesAs: #((instvar (ident 'Foo') (ident 'Integer') (directives) (visibility private)) (instvar (ident 'Bar') (ident 'Integer') (directives) (visibility private))) from: 'private Foo: Integer; Bar: Integer;' using: #classBody.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/2/2009 13:14'!
testClassBodyChunk
	self assertParsesAs: #((classvar (ident 'Foo') (ident 'Integer') () (directives))) from: 'class var Foo: Integer;' using: #classBodyChunk.
	self assertParsesAs: #((instvar (ident 'Foo') (ident 'Integer') () (directives))) from: 'var Foo: Integer;' using: #classBodyChunk.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/2/2009 21:33'!
testClassBodyWithDefaultVisibility
	self assertParsesAs: #((instvar (ident 'Foo') (ident 'Integer') (directives) (visibility published))) from: 'Foo: Integer;' using: #classBodyWithDefaultVisibility.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/3/2009 21:13'!
testClassClassProperty
	self assertParsesAs: #(classProperty (ident 'Foo') () (ident 'Integer') (propertySpecifiers (directive 'read' (ident 'fFoo')) (directive 'write' (ident 'fFoo')))) from: 'class property Foo: Integer read fFoo write fFoo;' using: #classClassProperty.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/31/2008 23:07'!
testClassField
	self assertParsesAs: #((instvar (ident 'Foo') (ident 'Integer') (directives))) from: 'Foo: Integer;' using: #classField.
	self assertParsesAs: #((instvar (ident 'Foo') (ident 'Integer') (directives)) (instvar (ident 'Bar') (ident 'Integer') (directives))) from: 'Foo, Bar: Integer;' using: #classField.
	self assertParsesAs: #((instvar (ident 'Foo') (ident 'Integer') (directives (directive 'library')))) from: 'Foo: Integer library;' using: #classField.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/1/2009 23:58'!
testClassHelperDecl
	self assertParsesAs: #(classHelper (ident 'TFoo') () ((procedureMethodDecl (ident 'Foo') (params) (directives)))) from: 'class helper for TFoo procedure Foo; end;' using: #classHelperDecl.
	
	self assertParsesAs: #(classHelper (ident 'TFoo') ((ident 'TBar') (ident 'TBaz')) ((procedureMethodDecl (ident 'Foo') (params) (directives)))) from: 'class helper (TBar, TBaz) for TFoo procedure Foo; end;' using: #classHelperDecl.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/9/2009 11:24'!
testClassMethod
	self assertParsesAs: #(procedureMethodDecl (ident 'Foo') (params) (directives)) from: 'procedure Foo;' using: #classMethod.
	self assertParsesAs: #(procedureMethodDecl (ident 'Package') (params) (directives)) from: 'procedure Package;' using: #classMethod.
	self assertParsesAs: #(procedureMethodDecl (ident 'Foo') (params) (directives (directive 'virtual'))) from: 'procedure Foo; virtual;' using: #classMethod.
	self assertParsesAs: #(procedureMethodDecl (ident 'Foo') (params) (directives (directive 'cdecl'))) from: 'procedure Foo; cdecl;' using: #classMethod.
	self assertParsesAs: #(procedureMethodDecl (ident 'Foo') (params) (directives (directive 'deprecated'))) from: 'procedure Foo; deprecated;' using: #classMethod.
	self assertParsesAs: #(procedureMethodDecl (ident 'Foo') (params) (directives (directive 'reintroduce'))) from: 'procedure Foo; reintroduce;' using: #classMethod.
	self assertParsesAs: #(procedureMethodDecl (ident 'Foo') (params) (directives (directive 'virtual') (directive 'abstract'))) from: 'procedure Foo; virtual; abstract;' using: #classMethod.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/4/2009 21:33'!
testClassMethodHeading
	self assertParsesAs: #(procedureMethodDecl (ident 'Foo') (params)) from: 'procedure Foo;' using: #classMethodHeading.
	self assertParsesAs: #(functionMethodDecl (ident 'Foo') (params (result (ident 'Integer')))) from: 'function Foo: Integer;' using: #classMethodHeading.
	
	self assertParsesAs: #(classProcedureMethodDecl (ident 'Foo') (params)) from: 'class procedure Foo;' using: #classMethodHeading.
	self assertParsesAs: #(classFunctionMethodDecl (ident 'Foo') (params (result (ident 'Integer')))) from: 'class function Foo: Integer;' using: #classMethodHeading.
	
	self assertParsesAs: #(constructorDecl (ident 'Foo') (params)) from: 'constructor Foo;' using: #classMethodHeading.
	self assertParsesAs: #(destructorDecl (ident 'Foo') (params)) from: 'destructor Foo;' using: #classMethodHeading.
	self assertParsesAs: #(classConstructorDecl (ident 'Foo') (params)) from: 'class constructor Foo;' using: #classMethodHeading.
	
	self assertParsesAs: #(operatorDecl (ident 'Foo') (params)) from: 'class operator Foo;' using: #classMethodHeading.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/9/2009 12:53'!
testClassMethodResolution
	self assertParsesAs: #(methodDelegation (ident 'MyFoo') (ident 'Foo') (ident 'IFoo')) from: 'procedure IFoo.Foo = MyFoo;' using: #classMethodResolution.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/1/2009 23:54'!
testClassParent
	self assertParsesAs: #((ident 'TBar')) from: '(TBar)' using: #classParent.
	self assertParsesAs: #((ident 'TBar') (ident 'TBaz')) from: '(TBar, TBaz)' using: #classParent.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/3/2009 15:03'!
testClassProperty
	self assertParsesAs: #(property (ident 'Foo') () () (propertySpecifiers)) from: 'property Foo;' using: #classProperty.
	self assertParsesAs: #(property (ident 'Foo') () (ident 'Integer') (propertySpecifiers (directive 'read' (ident 'fFoo')) (directive 'write' (ident 'fFoo')))) from: 'property Foo: Integer read fFoo write fFoo;' using: #classProperty.
	self assertParsesAs: #(property (ident 'Foo') () () (propertySpecifiers (directive 'index' (intNum '0')) (directive 'stored' 'Foo') (directive 'default' (intNum '0')) (directive 'nodefault'))) from: 'property Foo index 0 stored ''Foo'' default 0 nodefault;' using: #classProperty.
	self assertParsesAs: #(property (ident 'Foo') () () (propertySpecifiers (directive 'readonly') (directive 'writeonly') (directive 'dispid' 'Foo'))) from: 'property Foo readonly writeonly dispid ''Foo'';' using: #classProperty.
	self assertParsesAs: #(property (ident 'Foo') () () (propertySpecifiers (directive 'default'))) from: 'property Foo; default;' using: #classProperty.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/3/2009 15:13'!
testClassPropertySpecifier
	self assertParsesAs: #(directive 'read' (ident 'fFoo')) from: 'read fFoo' using: #classPropertySpecifier.
	self assertParsesAs: #(directive 'write' (ident 'fFoo')) from: 'write fFoo' using: #classPropertySpecifier.
	self assertParsesAs: #(directive 'add' (ident 'fFoo')) from: 'add fFoo' using: #classPropertySpecifier.
	self assertParsesAs: #(directive 'remove' (ident 'fFoo')) from: 'remove fFoo' using: #classPropertySpecifier.
	self assertParsesAs: #(directive 'stored' (ident 'Foo')) from: 'stored Foo;' using: #classPropertySpecifier.
	self assertParsesAs: #(directive 'default' (ident 'Foo')) from: 'default Foo;' using: #classPropertySpecifier.
	self assertParsesAs: #(directive 'implements' (ident 'TFoo')) from: 'implements TFoo;' using: #classPropertySpecifier.
	self assertParsesAs: #(directive 'readonly') from: 'readonly' using: #classPropertySpecifier.
	self assertParsesAs: #(directive 'writeonly') from: 'writeonly' using: #classPropertySpecifier.
	self assertParsesAs: #(directive 'dispid' 'Foo') from: 'dispid ''Foo''' using: #classPropertySpecifier.
	self assertParsesAs: #(directive 'nodefault') from: 'nodefault' using: #classPropertySpecifier.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 5/22/2009 15:10'!
testClassType
	self assertParsesAs: #(classType (ident 'TFoo')) from: 'class of TFoo' using: #classType.
	self assertParsesAs: #(class ((ident 'TFoo')) ()) from: 'class(TFoo)' using: #classType.
	self assertParsesAs: #(class ((ident 'TObject')) ()) from: 'class end' using: #classType.
	self assertParsesAs: #(classHelper (ident 'TFoo') () ((procedureMethodDecl (ident 'Foo') (params) (directives)))) from: 'class helper for TFoo procedure Foo; end' using: #classType.
	self assertParsesAs: #(interface ((ident 'IInterface')) () ()) from: 'interface' using: #classType.
	self assertParsesAs: #(objectType ((ident 'TObject')) ()) from: 'object end' using: #classType! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/9/2009 05:58'!
testClassTypeDecl
	"Specified superclass (or superclass + implemented interfaces)."
	self assertParsesAs: #(class ((ident 'TBar')) ()) from: 'class(TBar)' using: #classTypeDecl.
	self assertParsesAs: #(class ((ident 'TBar') (ident 'TBaz')) ()) from: 'class (TBar, TBaz)' using: #classTypeDecl.
	
	self assertParsesAs: #(forwardClass) from: 'class' using: #classTypeDecl.

	"Field declared."
	self assertParsesAs: #(class ((ident 'TObject')) ((instvar (ident 'Foo') (ident 'Integer') (directives) (visibility private)))) from: 'class private Foo: Integer; end;' using: #classTypeDecl.

	"Visibility-less field, with and without other visibility clauses."
	self assertParsesAs: #(class ((ident 'TObject')) ((instvar (ident 'Foo') (ident 'Integer') (directives) (visibility published)))) from: 'class Foo: Integer; end;' using: #classTypeDecl.
	self assertParsesAs: #(class ((ident 'TObject')) ((instvar (ident 'Foo') (ident 'Integer') (directives) (visibility published)) (instvar (ident 'Bar') (ident 'Integer') (directives) (visibility public)))) from: 'class Foo: Integer; public Bar: Integer; end;' using: #classTypeDecl.

	"Multiple visibility clauses."
	self assertParsesAs: #(class ((ident 'TObject')) ((instvar (ident 'Foo') (ident 'Integer') (directives) (visibility private)) (instvar (ident 'Bar') (ident 'Integer') (directives) (visibility public)))) from: 'class private Foo: Integer; public Bar: Integer; end;' using: #classTypeDecl.
	self assertParsesAs: #(class ((ident 'TBar')) ((instvar (ident 'Foo') (ident 'Integer') (directives) (visibility private)) (instvar (ident 'Bar') (ident 'Integer') (directives) (visibility public)))) from: 'class(TBar) private Foo: Integer; public Bar: Integer; end;' using: #classTypeDecl.
	
	"'Sealed/abstract classes."
	self assertParsesAs: #(class ((ident 'TBar')) ((instvar (ident 'Foo') (ident 'Integer') (directives) (visibility private))) #sealed) from: 'class sealed (TBar) private Foo: Integer; end;' using: #classTypeDecl.
	self assertParsesAs: #(class ((ident 'TBar')) () #abstract) from: 'class abstract (TBar);' using: #classTypeDecl.
	
	"Regression tests."
	self assertParsesAs: #(class ((ident 'TObject')) ((property (ident 'Foo') () () (propertySpecifiers (directive 'default')) (visibility published)))) from: 'class property Foo; default; end;' using: #classTypeDecl.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/31/2008 14:00'!
testClassTypeRef
	self assertParsesAs: #(classType (ident 'TFoo')) from: 'class of TFoo;' using: #classTypeRef.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/2/2009 21:34'!
testClassVarClause
	"This is part of the classBody production."
	self assertParsesAs: #((classvar (ident 'Foo') (ident 'Integer') () (directives) (visibility published))) from: 'published class var Foo: Integer;' using: #classBody.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/9/2008 08:33'!
testCompoundStatement
	self assertParsesAs: #(#block) from: 'begin end' using: #compoundStatement.
	self assertParsesAs: #(#block) from: 'begin ; end' using: #compoundStatement.
	self assertParsesAs: #(#block) from: 'begin ;; end' using: #compoundStatement.
	self assertParsesAs: #(#block) from: 'begin ; ; end' using: #compoundStatement.
	self assertParsesAs: #(#block #(#ident 'A')) from: 'begin A; end' using: #compoundStatement.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/7/2009 16:19'!
testConstDeclaration
	self assertParsesAs: #(const (ident 'Foo') () (intNum '0') (directives)) from: 'Foo = 0;' using: #constDeclaration.
	self assertParsesAs: #(const (ident 'Foo') (ident 'Integer') (intNum '0') (directives)) from: 'Foo: Integer = 0;' using: #constDeclaration.
	self assertParsesAs: #(const (ident 'Foo') () (intNum '0') (directives (directive 'library') (directive 'experimental'))) from: 'Foo = 0 library experimental;' using: #constDeclaration.
	self assertParsesAs: #(const (ident 'Foo') () (funcall (ident 'WideChar') (args (hexNum 'feff'))) (directives)) from: 'Foo = WideChar($feff);' using: #constDeclaration.

	"Constant arrays of a previously declared type."	
	self assertParsesAs: #(const (ident 'Foo') (ident 'TFoos') ((intNum '0') (intNum '1'))  (directives)) from: 'Foo: TFoos = (0, 1);' using: #constDeclaration.

	"Constant arrays."
	self assertParsesAs: #(const (ident 'Foo') (arrayType (ident 'Integer') ((range (intNum '0') (intNum '1')))) ((intNum '0') (intNum '1'))  (directives)) from: 'Foo: array[0..1] of Integer = (0, 1);' using: #constDeclaration.

	"Constant multidimensional arrays, and arrays of arrays."	
	self assertParsesAs: #(const (ident 'Foo') (arrayType (ident 'Integer') ((range (intNum '0') (intNum '1')) (range (intNum '0') (intNum '1')))) (((intNum '0') (intNum '1')) ((intNum '1') (intNum '0'))) (directives)) from: 'Foo: array[0..1, 0..1] of Integer = ((0, 1), (1, 0));' using: #constDeclaration.
	self assertParsesAs: #(const (ident 'Foo') (arrayType (arrayType (ident 'Integer') ((range (intNum '2') (intNum '3')))) ((range (intNum '0') (intNum '1')))) (((intNum '0') (intNum '1')) ((intNum '1') (intNum '0'))) (directives)) from: 'Foo: array[0..1] of array[2..3] of Integer = ((0, 1), (1, 0));' using: #constDeclaration.
	
	"Constant records."
	self assertParsesAs: #(const (ident 'Foo') (ident 'TFoo') (((ident 'I') (intNum '0')) ((ident 'J') (ident 'Bar'))) (directives)) from: 'Foo: TFoo = (I: 0; J: Bar);' using: #constDeclaration.
	self assertParsesAs: #(const (ident 'Foo') (record ((recordField (ident 'Bar') (ident 'Integer') (directives)))) (((ident 'Bar') (intNum '42'))) (directives)) from: 'Foo: record Bar: Integer end = (Bar: 42);' using: #constDeclaration.
	
	self assertParsesAs: #(const (ident 'Foo') (record ((recordField (ident 'Bar') (ident 'Integer') (directives)) (recordField (ident 'Baz') (ident 'Integer') (directives)))) (((ident 'Bar') (intNum '42'))) (directives)) from: 'Foo: record Bar: Integer; Baz: Integer end = (Bar: 42);' using: #constDeclaration.

"	self assert: false description: 'Constant record'.
	self assert: false description: 'Constant nested records'."! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/4/2009 20:52'!
testConstExpression
	"Constant arrays."
	self assertParsesAs: #((intNum '0') (intNum '1') 'Foo') from: '(0, 1, ''Foo'')' using: #constExpression.

	"Constant records."
	self assertParsesAs: #(((ident 'I') (intNum '0'))) from: '(I: 0)' using: #constExpression.
	self assertParsesAs: #(((ident 'I') (intNum '0')) ((ident 'J') (intNum '1'))) from: '(I: 0; J: 1)' using: #constExpression.

	"Constant nested records."
	self assertParsesAs: #(((ident 'I') (intNum '0')) ((ident 'J') (((ident 'K') (intNum '1')) ((ident 'L') (intNum '2'))))) from: '(I: 0; J: (K: 1; L: 2))' using: #constExpression
! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/8/2009 13:49'!
testConstSection
	self assertParsesAs: #(consts #(const #(ident 'Foo') #() #(intNum '0') (directives))) from: 'const Foo = 0;' using: #constSection.
	self assertParsesAs: #(consts #(const #(ident 'Foo') #() #(intNum '0') (directives)) #(const #(ident 'Bar') #() #(intNum '1') (directives))) from: 'const Foo = 0; Bar = 1;' using: #constSection.
	
	self assertParsesAs: #(resourceStrings #(const #(ident 'Foo') #() 'Foo' (directives))) from: 'resourcestring Foo = ''Foo'';' using: #constSection.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/4/2009 23:15'!
testContainsClause
	self assertParsesAs: #(contains (unitFile (ident 'Foo') (fileName 'Foo.pas'))) from: 'contains Foo in ''Foo.pas'';' using: #containsClause.

	self assertParsesAs: #(contains (unitFile (ident 'Foo') (fileName 'Foo.pas')) (unitFile (ident 'Bar') (fileName 'Bar.pas'))) from: 'contains Foo in ''Foo.pas'', Bar in ''Bar.pas'';' using: #containsClause.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/6/2009 16:03'!
testCurlyComment
	self assertParsesAs: '' from: '{}' using: #curlyComment.
	self assertParsesAs: 'foo' from: '{foo}', String cr using: #curlyComment.
	self assertParsesAs: 'foo', String cr, 'bar' from: '{foo', String cr, 'bar}' using: #curlyComment.
	self assertParsesAs: 'foo', String lf, 'bar' from: '{foo', String lf, 'bar}' using: #curlyComment.
	self assertParsesAs: 'foo', String crlf, 'bar' from: '{foo', String crlf, 'bar}' using: #curlyComment.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/5/2009 10:18'!
testDeclSection
	self assertParsesAs: #(procedureImpl (procedureDecl (ident 'Foo') (params) (directives)) () (block)) from: 'procedure Foo; begin end;' using: #declSection.
	
	self assertParsesAs: #(labels (label (ident 'Foo'))) from: 'label Foo;' using: #declSection.
	self assertParsesAs: #(consts (const (ident 'Foo') () (intNum '0') (directives))) from: 'const Foo = 0;' using: #declSection.
	self assertParsesAs: #(types (type (ident 'Foo') (typeArguments) (typeOf (ident 'Integer')) (directives))) from: 'type Foo = type Integer;' using: #declSection.
	self assertParsesAs: #(vars (var (ident 'Foo') (ident 'Integer') () (directives))) from: 'var Foo: Integer;' using: #declSection.
	self assertParsesAs: #(exports (export (ident 'Foo') (params) (directives))) from: 'exports Foo;' using: #declSection.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 5/9/2009 21:01'!
testDesignator
	self assertParsesAs: #(ident 'root') from: 'root' using: #designator.
	self assertParsesAs: #(dereference #(ident 'root')) from: 'root^' using: #designator.
	self assertParsesAs: #(dereference #(dereference #(ident 'root'))) from: 'root^^' using: #designator.
	self assertParsesAs: #(dot #(dot #(ident 'root') #(ident 'trunk')) #(ident 'crown')) from: 'root.trunk.crown' using: #designator.
	self assertParsesAs: #(inherited #(ident 'foo')) from: 'inherited foo' using: #designator.
	self assertParsesAs: #(dot (funcall (ident 'Foo') (args (ident 'Bar'))) (funcall (ident 'Baz') (args (intNum '1')))) from: 'Foo(Bar).Baz(1)' using: #designator.
	self assertParsesAs: #(dot (as (ident 'Foo') (ident 'Bar')) (ident 'Baz')) from: '(Foo as Bar).Baz' using: #designator.
	self assertParsesAs: #(dot (as (dot (ident 'Foo') (ident 'Bar')) (ident 'Baz')) (ident 'Quux')) from: '(Foo.Bar as Baz).Quux' using: #designator.
	
	self assertParsesAs: #(inherited) from: 'inherited' using: #designator.
	
	self assertParsesAs: #(dot (ident 'System') (addressOf (ident 'IntOver'))) from: 'System.@IntOver' using: #designator.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/10/2008 22:39'!
testDesignatorArrayAccess
	self assertParsesAs: #(arrayIndex #(ident 'foo') #(intNum '1') #(intNum '2')) from: 'foo[1, 2]' using: #designator.
	self assertParsesAs: #(arrayIndex #(arrayIndex #(ident 'foo') #(intNum '1')) #(intNum '2')) from: 'foo[1][2]' using: #designator.
	
	self assertParsesAs: #(arrayIndex #(ident 'foo') #(intNum '1')) from: 'foo(.1.)' using: #designator.
	self assertParsesAs: #(arrayIndex #(ident 'foo') #(intNum '1')) from: 'foo(.1]' using: #designator.
	self assertParsesAs: #(arrayIndex #(ident 'foo') #(intNum '1')) from: 'foo[1.)' using: #designator.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/7/2009 14:46'!
testDesignatorProcedureCall
	self assertParsesAs: #(funcall (ident 'Foo') (args)) from: 'Foo()' using: #designator.
	self assertParsesAs: #(funcall (ident 'Foo') (args (ident 'Bar'))) from: 'Foo(Bar)' using: #designator.
	self assertParsesAs: #(funcall (ident 'Foo') (args (intNum '0') (intNum '1'))) from: 'Foo(0, 1)' using: #designator.
	self assertParsesAs: #(funcall (funcall (ident 'Foo') (args)) (args)) from: 'Foo()()' using: #designator.
	self assertParsesAs: #(funcall (funcall (ident 'Foo') (args (intNum '1'))) (args (intNum '2'))) from: 'Foo(1)(2)' using: #designator.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/20/2008 19:44'!
testDispIDDirective
	self assertParsesAs: #(directive 'dispid' #(ident 'someID')) from: 'dispid someID;' using: #dispIDDirective.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/18/2008 12:02'!
testElseBlock
	self assertParsesAs: #(elseBlock #(ident 'Foo')) from: 'else Foo' using: #elseBlock.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/5/2009 09:38'!
testExportItem
	self assertParsesAs: #(export (ident 'Foo') (params) (directives)) from: 'Foo' using: #exportItem.
	self assertParsesAs: #(export (ident 'Foo') (params (param (ident 'Bar') (ident 'Integer'))) (directives)) from: 'Foo(Bar: Integer)' using: #exportItem.
	self assertParsesAs: #(export (ident 'Foo') (params) (directives (directive 'resident'))) from: 'Foo resident' using: #exportItem.
	self assertParsesAs: #(export (ident 'Foo') (params) (directives (directive 'index' (intNum '1')))) from: 'Foo index 1' using: #exportItem.
	self assertParsesAs: #(export (ident 'Foo') (params) (directives (directive 'name' (ident 'Foozle')))) from: 'Foo name Foozle' using: #exportItem.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/5/2009 10:19'!
testExports
	self assertParsesAs: #(exports (export (ident 'Foo') (params) (directives))) from: 'exports Foo;' using: #exportsSection.
	self assertParsesAs: #(exports (export (ident 'Foo') (params) (directives)) (export (ident 'Bar') (params) (directives))) from: 'exports Foo, Bar;' using: #exportsSection.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/21/2008 22:53'!
testExpression
	#(#< #> #<= #>= #<> #= #in #is #as) do:
		[:each |
		self assertParsesAs: {each. #(ident 'foo'). #(ident 'bar').} from: ('foo ', each asString, ' bar') using: #expression.].

	self assertParsesAs: #(#< #(#= #(intNum '1') #(intNum '2')) #(intNum '3')) from: '1=2 < 3' using: #expression.
	
	self assertParsesAs: '1234' from: '''1234''' using: #expression.
	self assertParsesAs: '1234' from: '  ''1234''' using: #expression.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/5/2008 10:14'!
testExpressionList
	self assertParsesAs: #(#(ident 'foo')) from: 'foo' using: #expressionList.
	self assertParsesAs: #(#(ident 'foo') #(ident 'bar')) from: 'foo,bar' using: #expressionList.
	self assertParsesAs: #(#(ident 'foo') #(ident 'bar')) from: 'foo, bar' using: #expressionList.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 5/18/2009 14:36'!
testExternalDirective
	self assertParsesAs: #(#directive 'external') from: 'external;' using: #externalDirective.
	self assertParsesAs: #(#directive 'external' 'foo') from: 'external ''foo'';' using: #externalDirective.
	self assertParsesAs: #(#directive 'external' #(name 'foo')) from: 'external name ''foo'';' using: #externalDirective.
	self assertParsesAs: #(directive 'external' (ident 'Bar') (name 'Foo')) from: 'external Bar name ''Foo'';' using: #externalDirective.
	
	self assertParsesAs: #(#directive 'external' #(index #(intNum '42'))) from: 'external index 42;' using: #externalDirective.
	self assertParsesAs: #(#directive 'external' #(name 'foo') #(index #(intNum '42'))) from: 'external name ''foo'' index 42;' using: #externalDirective.
! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/9/2009 22:26'!
testFactor
	self assertParsesAs: #(addressOfProc #(ident 'F')) from: '@@F' using: #factor.
	self assertParsesAs: #(addressOf #(intNum '1')) from: '@1' using: #factor.	
	self assertParsesAs: #(addressOf (arrayIndex (ident 'Foo') (intNum '42'))) from: '@(Foo[42])' using: #factor.
	self assertParsesAs: #(pointerTo #(ident 'foo')) from: '^foo' using: #factor.
	self assertParsesAs: #(intNum '123') from: '123' using: #factor.
	self assertParsesAs: #(intNum '123') from: '(123)' using: #factor.
	self assertParsesAs: #(real '1e-4') from: '1e-4' using: #factor.
	self assertParsesAs: #true from: 'true' using: #factor.
	self assertParsesAs: #false from: 'false' using: #factor.
	self assertParsesAs: #nil from: 'nil' using: #factor.
	self assertParsesAs: #(not #true) from: 'not true' using: #factor.
	self assertParsesAs: #(intNum '1') from: '+1' using: #factor.
	self assertParsesAs: #(#- #(intNum '1')) from: '-1' using: #factor.
	self assertParsesAs: #(- #(#- #(intNum '1'))) from: '--1' using: #factor.
	self assertParsesAs: #(set #(range #(intNum '1') #(intNum '3')) #(intNum '5')) from: '[1..3, 5]' using: #factor.
	self assertParsesAs: #(dot #(ident 'foo') #(ident 'bar')) from: 'foo.bar' using: #factor.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/31/2008 13:45'!
testFileType
	self assertParsesAs: #(fileType ()) from: 'file' using: #fileType.
	self assertParsesAs: #(fileType (ident 'TFoo')) from: 'file of TFoo' using: #fileType.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/9/2008 23:15'!
testForStatement
	self assertParsesAs: #(#for #(#ident 'I') #(#range #(#intNum '1') #(#intNum '10')) #(#assign #(#ident 'N') #(#+ #(#ident 'N') #(#ident 'I')))) from: 'for I := 1 to 10 do N := N + I' using: #forStatement.
	
	self assertParsesAs: #(#for #(#ident 'I') #(#range #(#intNum '10') #(#intNum '1')) #(#assign #(#ident 'N') #(#+ #(#ident 'N') #(#ident 'I')))) from: 'for I := 10 downto 1 do N := N + I' using: #forStatement.
	
	self assertParsesAs: #(#for #(#ident 'each') #(#ident 'bar') #(#assign #(#ident 'N') #(#+ #(#ident 'N') #(#ident 'each')))) from: 'for each in bar do N := N + each' using: #forStatement.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/23/2008 23:36'!
testFormalParameter
	self assertParsesAs: #(#(param #(ident 'Bar') #(ident 'String'))) from: 'Bar: String' using: #formalParameter.
	self assertParsesAs: #(#(param #(ident 'Bar') #(ident 'Integer') #(intNum '42'))) from: 'Bar: Integer = 42' using: #formalParameter.

	self assertParsesAs: #(#(param #(ident 'Bar') #(ident 'String') #(paramType 'const'))) from: 'const Bar: String' using: #formalParameter.
	self assertParsesAs: #(#(param #(ident 'Bar') #(ident 'String') #(paramType 'out'))) from: 'out Bar: String' using: #formalParameter.
	self assertParsesAs: #(#(param #(ident 'Bar') #(ident 'String') #(paramType 'var'))) from: 'var Bar: String' using: #formalParameter.
	
	self assertParsesAs: #(#(param #(ident 'Bar') #(ident 'String')) #(param #(ident 'Baz') #(ident 'String'))) from: 'Bar, Baz: String' using: #formalParameter.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/20/2008 21:54'!
testFormalParameterSection
	self assertParsesAs: #(params #(param #(ident 'Bar') #(ident 'String'))) from: '(Bar: String)' using: #formalParameterSection.

	self assertParsesAs: #(params #(param #(ident 'Bar') #(ident 'String')) #(param #(ident 'Baz') #(ident 'String'))) from: '(Bar, Baz: String)' using: #formalParameterSection.
	self assertParsesAs: #(params #(param #(ident 'Bar') #(ident 'String')) #(param #(ident 'Baz') #(ident 'String'))) from: '(Bar: String; Baz: String)' using: #formalParameterSection.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 11/28/2008 21:37'!
testFractionalNum
	self assertParsesAs: #(real '0.1') from: '  0.1 ' using: #fractionalNum.
	
	self assertParsesAs: #(real '0.0') from: '0.0' using: #fractionalNum.
	self assertParsesAs: #(real '1.101') from: '1.101' using: #fractionalNum.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/23/2008 23:32'!
testFuncDecl
	self assertParsesAs: #(functionDecl #(ident 'Foo') #(params #(result #(ident 'Integer'))) #(directives)) from: 'function Foo: Integer;' using: #procDecl.
	self assertParsesAs: #(functionDecl #(ident 'Foo') #(params #(param #(ident 'Bar') #(ident 'String')) #(result #(ident 'Integer'))) #(directives)) from: 'function Foo(Bar: String): Integer;' using: #procDecl.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 5/18/2009 14:46'!
testFunctionDirective
	#('inline' 'overload' 'resident' 'unsafe') do:
		[:each |
		self assertParsesAs: {#directive. each.} from: each, ';' using: #functionDirective.]! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/29/2008 21:52'!
testGenericConstruct
	self assertParsesAs: #(typeArguments #(ident 'Foo') #(ident 'Bar')) from: '<Foo, Bar>' using: #genericConstruct.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/7/2009 19:54'!
testHandler
	self assertParsesAs: #(exceptionHandler () (ident 'Exception') ()) from: 'on Exception do;' using: #handler.
	self assertParsesAs: #(exceptionHandler () (ident 'Exception') (ident 'Foo')) from: 'on Exception do Foo;' using: #handler.
	self assertParsesAs: #(exceptionHandler () (ident 'Exception') (funcall (ident 'Foo') (args))) from: 'on Exception do Foo();' using: #handler.
	
	self assertParsesAs: #(exceptionHandler (ident 'E') (ident 'Exception') (ident 'Foo')) from: 'on E: Exception do Foo;' using: #handler.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 11/11/2008 20:55'!
testHexDigit
	#($0 $1 $2 $3 $4 $5 $6 $7 $8 $9 $a $b $c $d $e $f $A $B $C $D $E $F) do:
		[:each |
		self assert: each = (self parse: each asString using: #hexDigit)]! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 5/18/2009 15:35'!
testHexNum
	self assertParsesAs: #(hexNum '0') from: '$0' using: #hexNum.
	self assertParsesAs: #(hexNum '0') from: '{}$0' using: #hexNum.
	self assertParsesAs: #(hexNum '1234') from: '$1234' using: #hexNum.
	self assertParsesAs: #(hexNum 'decafbad') from: '$decafbad' using: #hexNum.
	
	self assertParsesAs: #(hexNum '0') from: '  $0 ' using: #hexNum.	
	
	self assertParsesAs: #(hexNum '123456789abcdef0123456789ABCDEF') from: '$123456789abcdef0123456789ABCDEF' using: #hexNum.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/19/2008 23:31'!
testHintingDirective
	#('deprecated' 'experimental' 'library' 'platform') do:
		[:each |
		self assertParsesAs: {#directive. each.} from: each, ';' using: #hintingDirective.]! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/10/2008 14:07'!
testIdent
	self assertParsesAs: #(#ident 'foo') from: 'foo' using: #ident.
	self assertParsesAs: #(#ident 'for') from: '&for' using: #ident.
	self assertParsesAs: #(#ident 'one') from: 'one' using: #ident. "'leading' reserved word."
	
	self assertParseFailFrom: 'repeat' using: #ident. "reserved word."! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/18/2008 15:03'!
testIdentList
	self assertParsesAs: #(#(ident 'Foo')) from: 'Foo' using: #identList.
	self assertParsesAs: #(#(ident 'Foo') #(ident 'Bar')) from: 'Foo, Bar' using: #identList.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 11/11/2008 20:29'!
testIdentifier
	self assertIdentity: 'foo' using: #identifier.
	self assertIdentity: '_f' using: #identifier.
	self assertIdentity: 'b4r' using: #identifier.
	self assertIdentity: 'foo_bar' using: #identifier.

	self assert: 'foo' = (self parse: '  foo' using: #identifier).! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/14/2008 21:35'!
testIfThen
	self assertParsesAs: #(if #(ident 'A') #(ident 'B') #()) from: 'if A then B' using: #ifStatement.
	self assertParsesAs: #(if #(ident 'A') #(ident 'B') #(ident 'C')) from: 'if A then B else C' using: #ifStatement.
	self assertParsesAs: #(if #(ident 'A') #(block) #()) from: 'if A then begin end' using: #ifStatement.
	self assertParsesAs: #(if #(ident 'A') #(ident 'B') #(block)) from: 'if A then B else begin end' using: #ifStatement.
	self assertParsesAs: #(if #(ident 'A') #(block) #(block)) from: 'if A then begin end else begin end' using: #ifStatement.
	self assertParsesAs: #(if #(ident 'A') #(block #(ident 'B')) #(block #(ident 'C'))) from: 'if A then begin B end else begin C end' using: #ifStatement.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 5/18/2009 15:34'!
testIntNum
	self assertParsesAs: #(intNum '0') from: '  0 ' using: #intNum.
	self assertParsesAs: #(intNum '0') from: '{}0' using: #intNum.

	#('0' '1234' '1234567890123456789') do:
		[:each |
		self
			assertParsesAs: {#intNum. each.}
			from: each using: #intNum
			description: 'Parsing ''', each, ''' failed']! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 11/26/2008 21:53'!
testInterfaceGuid
	self assertEquals: #(#guid 'foo') collection: (self parse: '[''foo'']' using: #interfaceGuid)! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/5/2009 21:43'!
testInterfaceType
	"Minimal interface."
	self assertParsesAs: #(interface ((ident 'IInterface')) () ()) from: 'interface' using: #interfaceTypeDecl.

	"Minimal + GUID".
	self assertParsesAs: #(interface ((ident 'IInterface')) (guid '{C3255325-A52E-46FF-9C21-478880FB350A}') ()) from: 'interface [''{C3255325-A52E-46FF-9C21-478880FB350A}''] end' using: #interfaceTypeDecl.
	
	"Superclass + GUID."
	self assertParsesAs: #(interface ((ident 'IBar')) (guid 'foozle') ()) from: 'interface(IBar) [''foozle''] end' using: #interfaceTypeDecl.
	
	"Superclass, no GUID, declared procedure."
	self assertParsesAs: #(interface ((ident 'IBar')) () ((procedureMethodDecl (ident 'Foo') (params) (directives)))) from: 'interface(IBar) procedure Foo; end' using: #interfaceTypeDecl.
	
	"No superclass, no GUID, declared procedure."
	self assertParsesAs: #(interface ((ident 'IInterface')) () ((procedureMethodDecl (ident 'Foo') (params) (directives)))) from: 'interface procedure Foo; end' using: #interfaceTypeDecl.
	
	"Superclass, GUID, declared procedure."
	self assertParsesAs: #(interface ((ident 'IBar')) (guid 'foozle') ((procedureMethodDecl (ident 'Foo') (params) (directives)))) from: 'interface(IBar) [''foozle''] procedure Foo; end' using: #interfaceTypeDecl.
	
	"All other allowed declarations."
	self assertParsesAs: #(interface ((ident 'IInterface')) () ((property (ident 'Foo') () (ident 'Integer') (propertySpecifiers (directive 'read' (ident 'GetFoo')) (directive 'write' (ident 'SetFoo')))) (classProperty (ident 'Bar') () (ident 'Integer') (propertySpecifiers (directive 'read' (ident 'GetBar')) (directive 'write' (ident 'SetBar')))))) from: 'interface property Foo: Integer read GetFoo write SetFoo; class property Bar: Integer read GetBar write SetBar; end' using: #interfaceTypeDecl.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/7/2008 15:18'!
testLabel
	self assertParsesAs: #(#label #(ident 'foo')) from: 'foo:' using: #label.
	self assertParsesAs: #(#label #(intNum '1')) from: '1:' using: #label.
	self assertParsesAs: #(#label #(hexNum '1')) from: '$1:' using: #label.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/20/2008 22:34'!
testLabelDeclSection
	self assertParsesAs: #(labels #(label #(ident 'fee')) #(label #(ident 'fie')) #(label #(ident 'foe')) #(label #(ident 'fum'))) from: 'label fee, fie, foe, fum;' using: #labelDeclSection.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/7/2008 15:17'!
testLabelMarker
	self assertParsesAs: #(#label #(ident 'foo')) from: 'foo:' using: #labelMarker.
	self assertParsesAs: #(#label #(intNum '1')) from: '1:' using: #labelMarker.
	self assertParsesAs: #(#label #(hexNum '1')) from: '$1:' using: #labelMarker.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/7/2008 20:37'!
testLabelledStatement
	self assertParsesAs: #(#ident 'A' #(#label #(#ident 'B'))) from: 'B: A' using: #statement.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/30/2008 20:34'!
testLibrary
	self assertParsesAs: #(library (ident 'Foo') (directives) (uses) () (block)) from: 'library Foo; begin end.; ' using: #library.

	self assertParsesAs: #(library (ident 'Foo') (directives) (uses (unitFile (ident 'Bar') (fileName 'Bar.pas'))) () (block)) from: 'library Foo; uses Bar in ''Bar.pas''; begin end.; ' using: #library.

	self assertParsesAs: #(library (ident 'Foo') (directives (directive 'experimental') (directive 'platform')) (uses) () (block)) from: 'library Foo experimental platform; begin end.' using: #library.
	
	self assertParsesAs: #(library (ident 'Foo') (directives) (uses) ((procedureImpl (procedureDecl (ident 'Bar') (params) (directives)) () (block))) (block)) from: 'library Foo; procedure Bar; begin end; begin end.' using: #library.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/30/2008 14:47'!
testLibraryHead
	self assertParsesAs: #(library (ident 'Foo') (directives)) from: 'library Foo;' using: #libraryHead.
	self assertParsesAs: #(library (ident 'Foo') (directives (directive 'experimental') (directive 'platform'))) from: 'library Foo experimental platform;' using: #libraryHead.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/9/2009 15:02'!
testLineEnd
	self assertParsesAs: {Character cr. Character lf.} from: Character cr asString, Character lf asString using: #lineEnd.
	self assertParsesAs: Character cr from: Character cr asString using: #lineEnd.
	self assertParsesAs: Character lf from: Character lf asString using: #lineEnd.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 11/28/2008 22:03'!
testLiteralCharacter
	self assertParsesAs: #(char '32') from: '#32' using: #literalCharacter.
	self assertParsesAs: #(char '32') from: '#$20' using: #literalCharacter.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/30/2008 20:35'!
testMethodBody
	self assertParsesAs: #(() (block (ident 'Foo') (if (ident 'Bar') (ident 'Baz') (ident 'Quux')))) from: 'begin Foo; if Bar then Baz else Quux; end;' using: #methodBody.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/4/2009 23:07'!
testMethodDecl
	self assertParsesAs: #(procedureMethodDecl #(dot (ident 'TFoo') (ident 'Foo')) (params) (directives)) from: 'procedure TFoo.Foo;' using: #methodDecl.
	self assertParsesAs: #(procedureMethodDecl (dot (ident 'TFoo') (ident 'Foo')) (params) (directives (directive 'cdecl'))) from: 'procedure TFoo.Foo; cdecl;' using: #methodDecl.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/20/2008 19:42'!
testMethodDirective
	#('reintroduce' 'overload' 'abstract') do:
		[:each |
		self assertParsesAs: {#directive. each.} from: each, ';' using: #methodDirective.]! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/4/2009 22:44'!
testMethodImpl
	self assertParsesAs: #(methodImpl (procedureMethodDecl (dot (ident 'TFoo') (ident 'Foo')) (params) (directives)) () (block)) from: 'procedure TFoo.Foo; begin end;' using: #methodImpl.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/31/2008 23:46'!
testMethodKey
	self assertParsesAs: #functionMethod from: 'function' using: #methodKey.
	self assertParsesAs: #procedureMethod from: 'procedure' using: #methodKey.
	self assertParsesAs: #classFunctionMethod from: 'class function' using: #methodKey.
	self assertParsesAs: #classProcedureMethod from: 'class procedure' using: #methodKey.
	self assertParsesAs: #constructor from: 'constructor' using: #methodKey.	
	self assertParsesAs: #destructor from: 'destructor' using: #methodKey.		
	self assertParsesAs: #operator from: 'class operator' using: #methodKey.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/14/2008 22:19'!
testNamespaceFileName
	self assertParsesAs: #(ident 'Foo') from: 'Foo' using: #namespaceFileName.
	self assertParsesAs: #(unitFile #(ident 'Foo') #(fileName 'Foo.pas')) from: 'Foo in ''Foo.pas''' using: #namespaceFileName.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 5/9/2009 21:03'!
testNamespaceName
	self assertParsesAs: #(ident 'one') from: 'one' using: #namespaceName.
	self assertParsesAs: #(dot #(dot #(ident 'one') #(ident 'two')) #(ident 'three')) from: 'one.two.three' using: #namespaceName.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 5/9/2009 20:49'!
testNamespaceQualifiedIdent
	self assertParsesAs: #(ident 'one') from: 'one' using: #namespaceQualifiedIdent.
	self assertParsesAs: #(dot #(dot (ident 'one') #(ident 'two')) #(ident 'three')) from: 'one.two.three' using: #namespaceQualifiedIdent.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/5/2009 16:32'!
testObjectBody
	self assertParsesAs: #((procedureMethodDecl (ident 'Foo') (params) (directives) (visibility public))) from: 'public procedure Foo;' using: #objectBody.
	self assertParsesAs: #((procedureMethodDecl (ident 'Foo') (params) (directives) (visibility public)) (procedureMethodDecl (ident 'Bar') (params) (directives) (visibility public))) from: 'public procedure Foo; procedure Bar;' using: #objectBody.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/5/2009 21:38'!
testObjectBodyWithDefaultVisibility
	self assertParsesAs: #((procedureMethodDecl (ident 'Foo') (params) (directives) (visibility published))) from: 'procedure Foo;' using: #objectBodyWithDefaultVisibility.
	self assertParsesAs: #((procedureMethodDecl (ident 'Foo') (params) (directives) (visibility published)) (procedureMethodDecl (ident 'Bar') (params) (directives) (visibility published))) from: 'procedure Foo; procedure Bar;' using: #objectBodyWithDefaultVisibility.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/5/2009 17:04'!
testObjectTypeDecl
	self assertParsesAs: #(objectType ((ident 'TObject')) ()) from: 'object end;' using: #objectTypeDecl.
	self assertParsesAs: #(objectType ((ident 'TObject')) ()) from: 'object(TObject) end;' using: #objectTypeDecl.
	self assertParsesAs: #(objectType ((ident 'TFoo')) ()) from: 'object(TFoo) end;' using: #objectTypeDecl.
	self assertParsesAs: #(objectType ((ident 'TObject')) ((procedureMethodDecl (ident 'Foo') (params) (directives) (visibility published)))) from: 'object(TObject) procedure Foo; end;' using: #objectTypeDecl.
	self assertParsesAs: #(objectType ((ident 'TObject')) ((procedureMethodDecl (ident 'Foo') (params) (directives) (visibility published)) (procedureMethodDecl (ident 'Bar') (params) (directives) (visibility public)))) from: 'object(TObject) procedure Foo; public procedure Bar; end;' using: #objectTypeDecl.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/6/2009 15:52'!
testOneLineComment
	self assertParsesAs: '' from: '//' using: #oneLineComment.
	self assertParsesAs: '' from: '//', String cr using: #oneLineComment.
	self assertParsesAs: ' foo' from: '// foo' using: #oneLineComment.
	self assertParsesAs: 'foo' from: '//foo', String cr, 'bar' using: #oneLineComment.
	self assertParsesAs: 'foo' from: '//foo', String lf, 'bar' using: #oneLineComment.
	self assertParsesAs: 'foo' from: '//foo', String crlf, 'bar' using: #oneLineComment.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/4/2009 23:42'!
testPackage
	self assertParsesAs: #(package (ident 'Foo') (requires) (contains)) from: 'package Foo; end.' using: #package.
	self assertParsesAs: #(package (ident 'Foo') (requires (ident 'Bar')) (contains)) from: 'package Foo; requires Bar; end.' using: #package.
	self assertParsesAs: #(package (ident 'Foo') (requires) (contains (unitFile (ident 'Baz') (fileName 'Baz.pas')))) from: 'package Foo; contains Baz in ''Baz.pas''; end.' using: #package.
	self assertParsesAs: #(package (ident 'Foo') (requires (ident 'Bar')) (contains (unitFile (ident 'Baz') (fileName 'Baz.pas')))) from: 'package Foo; requires Bar; contains Baz in ''Baz.pas''; end.' using: #package.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/4/2009 22:58'!
testPackageHead
	self assertParsesAs: #(package (ident 'Foo')) from: 'package Foo;' using: #packageHead.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/19/2008 21:14'!
testParmType
	self assertParsesAs: #(#paramType #const) from: 'const' using: #parmType.
	self assertParsesAs: #(#paramType #out) from: 'out' using: #parmType.
	self assertParsesAs: #(#paramType #var) from: 'var' using: #parmType.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/29/2008 20:57'!
testPointerType
	self assertParsesAs: #(pointerType #(ident 'TFoo')) from: '^TFoo' using: #type.
	self assertParsesAs: #(pointerType #pointer) from: 'Pointer' using: #type.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 5/18/2009 14:56'!
testProcBody
	self assertParsesAs: #(() (block (ident 'Foo') (if (ident 'Bar') (ident 'Baz') (ident 'Quux')))) from: 'begin Foo; if Bar then Baz else Quux; end;' using: #procBody.
	
	self assertParsesAs: #((directives (directive 'forward'))) from: 'forward;' using: #procBody.
	self assertParsesAs: #((directives (directive 'forward') (directive 'cdecl'))) from: 'forward; cdecl;' using: #procBody.
	
	self assertParsesAs: #((directives (directive 'external'))) from: 'external;' using: #procBody.
	
"	self assertParsesAs: #((directives (directive 'external' 'strlib.dll'))) from: 'external ''strlib.dll'';' using: #procBody.
	
	self assertParsesAs: #((directives (directive 'external' 'strlib.dll')) (directive 'name' 'Foo')) from: 'external ''strlib.dll'' name ''Foo'';' using: #procBody.
	
	self assertParsesAs: #((directives (directive 'external' (ident 'strlib'))) (directive 'name' 'Foo')) from: 'external strlib name ''Foo'';' using: #procBody.
	
	self assertParsesAs: #((directives (directive 'external' 'strlib.dll') (directive 'index' (intNum 42)))) from: 'external ''strlib.dll'' index 42;' using: #procBody.
	
	self assertParsesAs: #((directives (directive 'external' 'strlib.dll') (directive 'name' 'Foo') (directive 'name' 'Foo') (directive 'index' (intNum 42)))) from: 'external ''strlib.dll'' name ''Foo'' index 42;' using: #procBody.
	
	self assert: false description: 'external with name & index directives & function directives'.
	
	self assertParsesAs: #((directives (directive 'external') (directive 'cdecl') (directive 'unsafe'))) from: 'external; cdecl; unsafe;' using: #procBody."
	

	self assertParsesAs: #(procedureImpl (procedureDecl (ident 'Foo') (params) (directives)) (directives (directive 'external' (ident 'Bar') (name 'Foo')))) from: 'procedure Foo; external Bar name ''Foo'';' using: #procImpl.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/9/2009 09:57'!
testProcDecl
	self assertParsesAs: #(procedureDecl (ident 'Foo') (params) (directives)) from: 'procedure Foo;' using: #procDecl.
	self assertParsesAs: #(procedureDecl (ident 'Foo') (params) (directives (directive 'varargs'))) from: 'procedure Foo; varargs;' using: #procDecl.
	self assertParsesAs: #(procedureDecl (ident 'Foo') (params) (directives (directive 'cdecl') (directive 'varargs'))) from: 'procedure Foo; cdecl; varargs;' using: #procDecl.
	
	self assertParsesAs: #(procedureDecl #(ident 'Foo') #(params #(param #(ident 'Bar') #(ident 'String'))) #(directives)) from: 'procedure Foo(Bar: String);' using: #procDecl.
	
	self assertParsesAs: #(procedureDecl (ident 'Foo') (params (param (ident 'Bar') (ident 'String')) (param #(ident 'Baz') (ident 'String'))) (directives)) from: 'procedure Foo(Bar: String; Baz: String);' using: #procDecl.
	self assertParsesAs: #(procedureDecl (ident 'Foo') (params (param (ident 'Bar') (ident 'String')) (param (ident 'Baz') #(ident 'String'))) (directives)) from: 'procedure Foo(Bar, Baz: String);' using: #procDecl.
	
	self assertParsesAs: #(procedureDecl (ident 'Foo') (params (param (ident 'Bar') (ident 'Integer') (intNum '42'))) (directives)) from: 'procedure Foo(Bar: Integer = 42);' using: #procDecl.
	
	self assertParsesAs: #(procedureDecl (ident 'Foo') (params (param (ident 'Bar') ())) (directives)) from: 'procedure Foo(Bar);' using: #procDecl.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 5/18/2009 14:16'!
testProcImpl
	self assertParsesAs: #(procedureImpl (procedureDecl (ident 'Foo') (params) (directives)) () (block (ident 'Foo'))) from: 'procedure Foo; begin Foo; end;' using: #procImpl.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/29/2008 20:46'!
testProcedureType
	self assertParsesAs: #(procedureType #(params) #(directives)) from: 'procedure;' using: #procedureType.
	self assertParsesAs: #(functionType #(params #(result #(ident 'Integer'))) #(directives)) from: 'function: Integer;' using: #procedureType.
	self assertParsesAs: #(procedureMethodType #(params) #(directives)) from: 'procedure of object;' using: #procedureType.
	self assertParsesAs: #(functionMethodType #(params #(result #(ident 'Integer'))) #(directives)) from: 'function: Integer of object;' using: #procedureType.
	
	self assertParsesAs: #(procedureType #(params #(param #(ident 'Foo') #(ident 'Integer'))) #(directives)) from: 'procedure(Foo: Integer);' using: #procedureType.
	self assertParsesAs: #(procedureType #(params) #(directives #(directive 'cdecl'))) from: 'procedure; cdecl;' using: #procedureType.
	
	self assertParsesAs: #(procedureType #(params) #(directives) #reference) from: 'reference to procedure' using: #procedureType.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 5/18/2009 15:51'!
testProgram
	self assertParsesAs: #(program () (block)) from: 'begin end.' using: #program.
	self assertParsesAs: #(program (ident 'Foo') () (block)) from: 'program Foo; begin end.' using: #program.
	self assertParsesAs: #(program (uses (ident 'Foo')) () (block)) from: 'uses Foo; begin end.' using: #program.
	self assertParsesAs: #(program (uses (ident 'Foo') (ident 'Bar')) () (block)) from: 'uses Foo, Bar; begin end.' using: #program.

	self assertParsesAs: #(program (uses (unitFile (ident 'Foo') (fileName 'Foo.pas'))) () (block)) from: 'uses Foo in ''Foo.pas''; begin end.' using: #program.
	
	self assertParsesAs: #(program (ident 'Foo') () (block)) from: '{}program {} Foo; {} begin {} end.' using: #program.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/14/2008 20:38'!
testProgramHead
	self assertParsesAs: #(#program #(ident 'Foo')) from: 'program Foo;' using: #programHead.
	self assertParsesAs: #(#program #(ident 'Foo') #(#(ident 'Bar'))) from: 'program Foo(Bar);' using: #programHead.
	self assertParsesAs: #(#program #(ident 'Foo') #(#(ident 'Bar') #(ident 'Baz'))) from: 'program Foo(Bar, Baz);' using: #programHead.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 5/9/2009 20:31'!
testQualifiedIdent
	self assertParsesAs: #(ident 'one') from: 'one' using: #qualifiedIdent.
	self assertParsesAs: #(ident 'one') from: 'one..' using: #qualifiedIdent.
	self assertParsesAs: #(dot #(dot #(ident 'one') #(ident 'two')) #(ident 'three')) from: 'one.two.three' using: #qualifiedIdent.

	self assertParsesAs: #(dot #(ident 'system') #(ident 'type')) from: 'system.type' using: #qualifiedIdent.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/13/2008 21:37'!
testQuotedString
	self assertParsesAs: '' from: '''''' using: #quotedString.
	self assertParsesAs: 'one' from: '''one''' using: #quotedString.
	self assertParsesAs: 'a b' from: '''a b''' using: #quotedString.
	self assertParsesAs: 'a''b' from: '''a''''b''' using: #quotedString.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/13/2008 22:05'!
testRaiseStatement
	self assertParsesAs: #(raise) from: 'raise' using: #raiseStatement.
	self assertParsesAs: #(raise #(ident 'foo')) from: 'raise foo' using: #raiseStatement.
	self assertParsesAs: #(raise #(ident 'foo') #(ident 'self')) from: 'raise foo at self' using: #raiseStatement.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 5/18/2009 15:35'!
testRealNum
	self assertParsesAs: #(real '0.0') from: '0.0' using: #realNum.
	self assertParsesAs: #(real '0.0') from: '{}0.0' using: #realNum.
	self assertParsesAs: #(real '0.00') from: '0.00' using: #realNum.
	self assertParsesAs: #(real '0.001') from: '0.001' using: #realNum.
	self assertParsesAs: #(real '0.0') from: ' 0.0' using: #realNum.
	self assertParsesAs: #(real '1.0') from: '1.0' using: #realNum.
	self assertParsesAs: #(real '3.14') from: '3.14' using: #realNum.
	
	self assertParsesAs: #(real '0e1') from: '0e1' using: #realNum.
	self assertParsesAs: #(real '0e-1') from: '0e-1' using: #realNum.
	self assertParsesAs: #(real '0.1e2.3') from: '0.1e2.3' using: #realNum.
	self assertParsesAs: #(real '0.1e-2.3') from: '0.1e-2.3' using: #realNum.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/4/2009 14:38'!
testRecordConstExpression
	self assertParsesAs: #((ident 'I') (intNum '0')) from: 'I: 0' using: #recordConstExpression.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/2/2009 22:49'!
testRecordField
	self assertParsesAs: #((recordField (ident 'Foo') (ident 'Integer') (directives))) from: 'Foo: Integer;' using: #recordField.
	
	self assertParsesAs: #((recordField (ident 'Foo') (ident 'Integer') (directives (directive 'platform') (directive 'library')))) from: 'Foo: Integer platform library;' using: #recordField.
	
	self assertParsesAs: #((recordField (ident 'Foo') (ident 'Integer') (directives)) (recordField (ident 'Bar') (ident 'Integer') (directives))) from: 'Foo, Bar: Integer;' using: #recordField.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 5/25/2009 15:42'!
testRecordFieldExt
	self assertParsesAs: #((recordField (ident 'Foo') (ident 'Integer') (directives))) from: 'Foo: Integer;' using: #recordFieldExt.
	self assertParsesAs: #(classProcedureMethodDecl (ident 'Foo') (params) (directives)) from: 'class procedure Foo;' using: #recordFieldExt.
	self assertParsesAs: #(classProperty (ident 'Foo') () (ident 'Integer') (propertySpecifiers (directive 'read' (ident 'fFoo')))) from: 'class property Foo: Integer read fFoo;' using: #recordFieldExt.	
	self assertParsesAs: #(consts (const (ident 'Foo') () (intNum '0') (directives))) from: 'const Foo = 0;' using: #recordFieldExt.
	self assertParsesAs: #(types (type (ident 'Foo') (typeArguments) (typeOf (ident 'Integer')) (directives))) from: 'type Foo = type Integer;' using: #recordFieldExt.
	self assertParsesAs: #((classvar (ident 'Foo') (ident 'Integer') () (directives))) from: 'class var Foo: Integer;' using: #recordFieldExt.
	self assertParsesAs: #((instvar (ident 'Foo') (ident 'Integer') () (directives))) from: 'var Foo: Integer;' using: #recordFieldExt.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/3/2009 21:54'!
testRecordType
	self assertParsesAs: #(record ()) from: 'record end;' using: #recordType.

	self assertParsesAs: #(record ((recordField (ident 'Foo') (ident 'Integer') (directives)))) from: 'record Foo: Integer; end;' using: #recordType.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/3/2009 23:24'!
testRecordVariant
	self assertParsesAs: #(recordVariant (#set (intNum '0')) ((#recordField (ident 'Foo') (ident 'Integer') (directives)))) from: '0: (Foo: Integer);' using: #recordVariant.
	
	self assertParsesAs: #(recordVariant (#set (intNum '0')) ((#recordField (ident 'Foo') (ident 'Integer') (directives)) (#recordField (ident 'Bar') (ident 'Integer') (directives)))) from: '0: (Foo: Integer; Bar: Integer);' using: #recordVariant.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/4/2009 14:19'!
testRecordVariantSection
	self assertParsesAs: #(recordVariants () (ident 'Integer') ((recordVariant (set (intNum '0')) ((recordField (ident 'Foo') (ident 'Integer') (directives)))))) from: 'case Integer of 0: (Foo: Integer)' using: #recordVariantSection.

	self assertParsesAs: #(recordVariants () (ident 'Integer') ((recordVariant (set (intNum '0')) ((recordField (ident 'Foo') (ident 'Integer') (directives)))) (recordVariant (set (intNum '1') (intNum '2')) ((recordField (ident 'Bar') (ident 'Integer') (directives)))))) from: 'case Integer of 0: (Foo: Integer); 1,2: (Bar: Integer)' using: #recordVariantSection.
	
	self assertParsesAs: #(recordVariants (ident 'I') (ident 'Integer') ((recordVariant (set (intNum '0')) ((recordField (ident 'Foo') (ident 'Integer') (directives)))))) from: 'case I: Integer of 0: (Foo: Integer)' using: #recordVariantSection.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/9/2008 20:31'!
testRepeatUntil
	self assertParsesAs: #(#repeat #() #true) from: 'repeat until true' using: #repeatStatement.
	self assertParsesAs: #(#repeat #(#(ident 'A')) #(#< #(intNum '1') #(intNum '2'))) from: 'repeat A; until 1 < 2;' using: #repeatStatement.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/4/2009 23:30'!
testRequiresClause
	self assertParsesAs: #(requires (ident 'Foo')) from: 'requires Foo;' using: #requiresClause.

	self assertParsesAs: #(requires (ident 'Foo') (ident 'Bar')) from: 'requires Foo, Bar;' using: #requiresClause.! !

!TestPyParser methodsFor: 'test-testing' stamp: 'fbs 5/13/2009 12:32'!
testReservedWord
	| parser |
	parser := PyParser new.
	self deny: (parser reservedWord: 'foo').
	self assert: (parser reservedWord: 'begin').
	self assert: (parser reservedWord: 'BEGIN') description: 'Case insensitivity'.
	self deny: (parser reservedWord: 'one'). "leading reserved word"! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/10/2008 21:56'!
testSetSection
	self assertParsesAs: #(#set) from: '[]' using: #setSection.
	self assertParsesAs: #(#set #(intNum '1')) from: '[1]' using: #setSection.
	
	self assertParsesAs: #(#set #(#range #(#intNum '1') #(#intNum '10'))) from: '[1..10]' using: #setSection.
	self assertParsesAs: #(#set #(#range #(#intNum '1') #(#intNum '10'))) from: '[1 .. 10]' using: #setSection.
	self assertParsesAs: #(#set #(#range #(#intNum '1') #(#intNum '10'))) from: '[1 ..10]' using: #setSection.
	self assertParsesAs: #(#set #(#range #(#intNum '1') #(#intNum '10'))) from: '[1.. 10]' using: #setSection.
	
	self assertParsesAs: #(#set #(#range #(#intNum '1') #(#intNum '10')) #(#intNum '13') #(#range #(#intNum '15') #(#intNum '17'))) from: '[1..10, 13, 15..17]' using: #setSection.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/29/2008 21:39'!
testSetType
	self assertParsesAs: #(setType #(ident 'Char')) from: 'set of Char' using: #setType.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/4/2008 22:37'!
testSimpleExpression
	#(#+ #- #or #xor) do:
		[:each |
		self assertParsesAs: {each. #(ident 'foo'). #(ident 'bar').} from: ('foo ', each asString, ' bar') using: #simpleExpression.].
	
	self assertParsesAs: #(#- #(#+ #(intNum '1') #(intNum '2')) #(intNum '3')) from: '1+2 - 3' using: #simpleExpression.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/9/2008 22:58'!
testSimpleStatement
	self assertParsesAs: #(#goto #(#label #(#ident 'foo'))) from: 'goto foo' using: #simpleStatement.

	self assertParsesAs: #(#assign #(#ident 'foo') #(#intNum '2')) from: 'foo:=2' using: #simpleStatement.
	self assertParsesAs: #(#assign #(#ident 'foo') #(#intNum '2')) from: 'foo := 2' using: #simpleStatement.
	self assertParsesAs: #(#assign #(#ident 'foo') #(#intNum '2')) from: 'foo :=2' using: #simpleStatement.
	
	self assertParsesAs: #(#ident 'foo') from: 'foo' using: #simpleStatement.
	self assertParsesAs: #(#dot #(#ident 'foo') #(#ident 'bar')) from: 'foo.bar' using: #simpleStatement.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/29/2008 23:46'!
testSimpleTypeEnum
	self assertParsesAs: #(enum #((ident 'Foo') (intNum '0'))) from: '(Foo)' using: #simpleTypeEnum.
	self assertParsesAs: #(enum ((ident 'Foo') (intNum '42'))) from: '(Foo = 42)' using: #simpleTypeEnum.
	self assertParsesAs: #(enum ((ident 'Foo') (intNum '0')) ((ident 'Bar') (intNum '1')) ((ident 'Baz') (intNum '2'))) from: '(Foo, Bar, Baz)' using: #simpleTypeEnum.
	self assertParsesAs: #(enum ((ident 'Foo') (intNum '0')) ((ident 'Bar') (+ (intNum '1') (intNum '2'))) ((ident 'Baz') (+ (ident 'Bar') (intNum '1')))) from: '(Foo = 0, Bar = 1 + 2, Baz = Bar + 1)' using: #simpleTypeEnum.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/29/2008 23:49'!
testSimpleTypeSubrange
	self assertParsesAs: #(range (intNum '1') (intNum '10')) from: '1..10' using: #simpleTypeSubRange.
	self assertParsesAs: #(range (ident 'foo') (ident 'bar')) from: 'foo..bar' using: #simpleTypeSubRange.
	self assertParsesAs: #(range 'a' 'z') from: '''a''..''z''' using: #simpleTypeSubRange.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/6/2009 16:09'!
testStarComment
	self assertParsesAs: '' from: '(**)' using: #starComment.
	self assertParsesAs: 'foo' from: '(*foo*)', String cr using: #starComment.
	self assertParsesAs: 'foo', String cr, 'bar' from: '(*foo', String cr, 'bar*)' using: #starComment.
	self assertParsesAs: 'foo', String lf, 'bar' from: '(*foo', String lf, 'bar*)' using: #starComment.
	self assertParsesAs: 'foo', String crlf, 'bar' from: '(*foo', String crlf, 'bar*)' using: #starComment.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/12/2009 21:14'!
testStatement
	self assertParsesAs: #(ident 'A') from: 'A' using: #statement.

	"Check that this isn't recognised as a label"
	self assertParsesAs: #(#assign #(#ident 'foo') #(#intNum '2')) from: 'foo:=2' using: #statement.
	
	"Check that 'Exit' isn't always special."
	self assertParsesAs: #(assign (ident 'Exit') (intNum '0')) from: 'Exit := 0' using: #statement.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 5/18/2009 15:50'!
testString
	self assertParsesAs: 'foo bar' from: '''foo''#$20''bar''' using: #string.
	self assertParsesAs: 'foo' from: '{}''foo''' using: #string.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 5/15/2009 13:05'!
testStringType
	self assertParsesAs: #(shortStringType #(intNum '42')) from: 'string[42]' using: #stringType.
	self assertParsesAs: #(ident 'string') from: 'string' using: #stringType.
	self assertParsesAs: #(ident 'shortstring') from: 'shortstring' using: #stringType.
	self assertParsesAs: #(ident 'ansistring') from: 'ansistring' using: #stringType.
	self assertParsesAs: #(ident 'widestring') from: 'widestring' using: #stringType.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/29/2008 21:37'!
testStructType
	self assertParsesAs: #(setType #(ident 'Char')) from: 'set of Char' using: #strucType.
	self assertParsesAs: #(packed #(setType #(ident 'Char'))) from: 'packed set of Char' using: #strucType.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/4/2008 22:39'!
testTerm
	#(#* #/ #div #mod #and #shr #shl) do:
		[:each |
		self assertParsesAs: {each. #(ident 'foo'). #(ident 'bar').} from: ('foo ', each asString, ' bar') using: #term.].

	self assertParsesAs: #(#/ #(#* #(intNum '1') #(intNum '2')) #(intNum '3')) from: '1*2 / 3' using: #term.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/6/2009 20:25'!
testToken
	self assertParsesAs: 'integer' from: 'integer' using: #token withArguments: {'integer'.}.
	self assertParsesAs: 'integer' from: 'integer stuff' using: #token withArguments: {'integer'.}.
	self assertParsesAs: 'integer' from: '  integer' using: #token withArguments: {'integer'.}.
	
	self assertParsesAs: ';' from: ';:;!!!!!!' using: #token withArguments: {';'.}.
	
	self assertParsesAs: 'Integer' from: 'Integer' using: #token withArguments: {'integer'.}.
	
	"Parsing tokens ignores leading whitespace and comments."
	self assertParsesAs: 'integer' from: '{}integer' using: #token withArguments: {'integer'.}.
	self assertParsesAs: 'integer' from: '{} {} {}integer' using: #token withArguments: {'integer'.}.
	self assertParsesAs: 'integer' from: '(* foo foo *)integer' using: #token withArguments: {'integer'.}.
	self assertParsesAs: 'integer' from: '// foo', String crlf, 'integer' using: #token withArguments: {'integer'.}.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/18/2008 12:03'!
testTryExceptStatement
	self assertParsesAs: #(tryExcept #() #()) from: 'try except end' using: #tryStatement.
	self assertParsesAs: #(tryExcept #(#(ident 'Foo') #(ident 'Bar')) #(#(ident 'Baz'))) from: 'try Foo; Bar; except Baz; end' using: #tryStatement.
	
	self assertParsesAs: #(tryExcept #(#(ident 'Foo')) #(#(exceptionHandler #(ident 'E') #(ident 'Exception') #(ident 'Bar')))) from: 'try Foo; except on E: Exception do Bar; end;' using: #tryStatement.
	
	self assertParsesAs: #(tryExcept #(#(ident 'Foo')) #(#(exceptionHandler #(ident 'E') #(ident 'Exception') #(ident 'Bar')) #(exceptionHandler #(ident 'Er') #(ident 'Error') #(ident 'Baz')))) from: 'try Foo; except on E: Exception do Bar; on Er: Error do Baz; end;' using: #tryStatement.
	
	self assertParsesAs: #(tryExcept #(#(ident 'Foo')) #(#(exceptionHandler #(ident 'E') #(ident 'Exception') #(ident 'Bar')) #(exceptionHandler #(ident 'Er') #(ident 'Error') #(ident 'Baz')) #(elseBlock #(ident 'Quux')))) from: 'try Foo; except on E: Exception do Bar; on Er: Error do Baz; else Quux; end;' using: #tryStatement.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/15/2008 23:27'!
testTryFinallyStatement
	self assertParsesAs: #(tryFinally #() #()) from: 'try finally end' using: #tryStatement.
	self assertParsesAs: #(tryFinally #(#(ident 'Foo') #(ident 'Bar')) #(#(ident 'Baz'))) from: 'try Foo; Bar; finally Baz; end' using: #tryStatement.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/29/2008 22:24'!
testType
	self assertParsesAs: #(ident 'Foo') from: 'Foo' using: #type.
	self assertParsesAs: #(procedureType #(params) #(directives)) from: 'procedure' using: #type.
	self assertParsesAs: #(pointerType #(ident 'TFoo')) from: '^TFoo' using: #type.
	self assertParsesAs: #(shortStringType #(intNum '42')) from: 'string[42]' using: #type.
	self assertParsesAs: #(setType #(ident 'Char')) from: 'set of Char' using: #type.	
	self assertParsesAs: #(ident 'variant') from: 'variant' using: #type.
	self assertParsesAs: #(genericType #(ident 'Foo') #(typeArguments #(ident 'Integer'))) from: 'Foo<Integer>' using: #type.

	self assertParsesAs: #(typeOf #(ident 'Foo')) from: 'type Foo' using: #type.
	self assertParsesAs: #(typeOf #(genericType #(ident 'Foo') #(typeArguments #(ident 'Integer')))) from: 'type Foo<Integer>' using: #type.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/31/2008 13:16'!
testTypeDeclaration
	self assertParsesAs: #(type (ident 'TFoo') (typeArguments) (typeOf (ident 'Integer')) (directives)) from: 'TFoo = type Integer;' using: #typeDeclaration.

	self assertParsesAs: #(type (ident 'TFoo') (typeArguments (ident 'C')) (genericType (ident 'GenericList') (typeArguments (ident 'C'))) (directives)) from: 'TFoo<C> = GenericList<C>;' using: #typeDeclaration.
	
	self assertParsesAs: #(type (ident 'TFoo') (typeArguments) (typeOf (ident 'Integer')) (directives (directive 'platform'))) from: 'TFoo = type Integer platform;' using: #typeDeclaration.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/7/2009 09:26'!
testTypeSection
	self assertParsesAs:
		#(types
			(type (ident 'TFoo') (typeArguments) (typeOf (ident 'Integer')) (directives))
			(type (ident 'TBar') (typeArguments) (typeOf (ident 'Integer')) (directives))) from: 'type TFoo = type Integer; TBar = type Integer;' using: #typeSection.
			
	self assertParsesAs:
		#(types
			(type (ident 'TFoo') (typeArguments) (forwardClass) (directives))
			(type (ident 'TFooClass') (typeArguments) (classType (ident 'TFoo')) (directives))
			(type (ident 'TFoo') (typeArguments) (class ((ident 'TObject')) ((instvar (ident 'Foo') (ident 'Integer') (directives) (visibility published)))) (directives)))
		from: 'type TFoo = class; TFooClass = class of TFoo; TFoo = class(TObject) Foo: Integer; end;' using: #typeSection.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/28/2008 21:47'!
testUnit
	self assertParsesAs: #(#unit #(ident 'Foo') #(directives) #(unitInterface #(uses) #()) #(unitImplementation #(uses) #()) #()) from: 'unit Foo; interface implementation end.' using: #unit.
	
	self assertParsesAs: #(#unit #(ident 'Foo') #(directives) #(unitInterface #(uses) #()) #(unitImplementation #(uses) #()) #(block)) from: 'unit Foo; interface implementation begin end.' using: #unit.
	
	self assertParsesAs: #(#unit #(ident 'Foo') #(directives) #(unitInterface #(uses) #()) #(unitImplementation #(uses) #()) #(unitInitialization #())) from: 'unit Foo; interface implementation initialization end.' using: #unit.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/28/2008 22:15'!
testUnitFinalization
	self assertParsesAs: #(unitFinalization #()) from: 'finalization' using: #unitFinalization.
	self assertParsesAs: #(unitFinalization #(#(ident 'Foo'))) from: 'finalization Foo' using: #unitFinalization.
	self assertParsesAs: #(unitFinalization #(#(ident 'Foo'))) from: 'finalization Foo;' using: #unitFinalization.
	self assertParsesAs: #(unitFinalization #(#(ident 'Foo') #(ident 'Bar'))) from: 'finalization Foo; Bar' using: #unitFinalization.
	self assertParsesAs: #(unitFinalization #(#(ident 'Foo') #(ident 'Bar'))) from: 'finalization Foo; Bar;' using: #unitFinalization.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/28/2008 20:06'!
testUnitHead
	self assertParsesAs: #(#unit #(ident 'Foo') #(directives)) from: 'unit Foo;' using: #unitHead.
	self assertParsesAs: #(#unit #(ident 'Foo') #(directives #(directive 'experimental') #(directive 'platform'))) from: 'unit Foo experimental platform;' using: #unitHead.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/30/2008 20:39'!
testUnitImplementation
	self assertParsesAs: #(unitImplementation (uses) ()) from: 'implementation' using: #unitImplementation.
	self assertParsesAs: #(unitImplementation (uses #(ident 'Foo')) ()) from: 'implementation uses Foo;' using: #unitImplementation.
	self assertParsesAs: #(unitImplementation (uses) ((procedureImpl (procedureDecl (ident 'Foo') (params) (directives)) () (block)))) from: 'implementation procedure Foo; begin end;' using: #unitImplementation.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/28/2008 22:15'!
testUnitInitialization
	self assertParsesAs: #(unitInitialization #()) from: 'initialization' using: #unitInitialization.
	self assertParsesAs: #(unitInitialization #(#(ident 'Foo'))) from: 'initialization Foo' using: #unitInitialization.
	self assertParsesAs: #(unitInitialization #(#(ident 'Foo'))) from: 'initialization Foo;' using: #unitInitialization.
	self assertParsesAs: #(unitInitialization #(#(ident 'Foo') #(ident 'Bar'))) from: 'initialization Foo; Bar' using: #unitInitialization.
	self assertParsesAs: #(unitInitialization #(#(ident 'Foo') #(ident 'Bar'))) from: 'initialization Foo; Bar;' using: #unitInitialization.
	
	self assertParsesAs: #(unitInitialization #() #(unitFinalization #())) from: 'initialization finalization' using: #unitInitialization.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/28/2008 20:37'!
testUnitInterface
	self assertParsesAs: #(unitInterface #(uses) #()) from: 'interface' using: #unitInterface.
	self assertParsesAs: #(unitInterface #(uses #(ident 'Foo')) #()) from: 'interface uses Foo;' using: #unitInterface.
	self assertParsesAs: #(unitInterface #(uses) #(#(procedureDecl #(ident 'Foo') #(params) #(directives)))) from: 'interface procedure Foo;' using: #unitInterface.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/14/2008 20:57'!
testUsesClause
	self assertParsesAs: #(uses #(ident 'Foo')) from: 'uses Foo;' using: #usesClause.
	self assertParsesAs: #(uses #(ident 'Foo') #(ident 'Bar')) from: 'uses Foo, Bar;' using: #usesClause.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/30/2008 22:54'!
testVarDeclaration
	self assertParsesAs: #((var (ident 'Foo') (ident 'Integer') () (directives))) from: 'Foo: Integer;' using: #varDeclaration.
	self assertParsesAs: #((var (ident 'Foo') (ident 'Integer') () (directives)) (var (ident 'Bar') (ident 'Integer') () (directives))) from: 'Foo, Bar: Integer;' using: #varDeclaration.
	self assertParsesAs: #((var (ident 'StrLen') (ident 'Byte') (absolute #(ident 'Str')) (directives))) from: 'StrLen: Byte absolute Str;' using: #varDeclaration.
	self assertParsesAs: #((var (ident 'Foo') (ident 'Integer') (intNum '0') (directives))) from: 'Foo: Integer = 0;' using: #varDeclaration.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/31/2008 10:07'!
testVarSection
	self assertParsesAs: #(vars (var (ident 'Foo') (ident 'Byte') () (directives))) from: 'var Foo: Byte;' using: #varSection.
	self assertParsesAs: #(vars (var (ident 'Foo') (ident 'Byte') () (directives)) (var (ident 'Bar') (ident 'Byte') () (directives))) from: 'var Foo: Byte; Bar: Byte;' using: #varSection.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/19/2008 19:55'!
testVarValueSpec
	self assertParsesAs: #(absolute #(ident 'I')) from: 'absolute I' using: #varValueSpec.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 5/18/2009 14:37'!
testVarargsDirective
	self assertParsesAs: #(directive 'varargs') from: 'varargs;' using: #varargsDirective.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/29/2008 21:41'!
testVariantType
	self assertParsesAs: #(ident 'variant') from: 'variant' using: #variantType.
	self assertParsesAs: #(ident 'olevariant') from: 'olevariant' using: #variantType.
! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 1/2/2009 00:21'!
testVisibility
	self assertParsesAs: #(visibility automated) from: 'automated' using: #visibility.
	self assertParsesAs: #(visibility private) from: 'private' using: #visibility.
	self assertParsesAs: #(visibility protected) from: 'protected' using: #visibility.
	self assertParsesAs: #(visibility public) from: 'public' using: #visibility.
	self assertParsesAs: #(visibility published) from: 'published' using: #visibility.
	self assertParsesAs: #(visibility strictPrivate) from: 'strict private' using: #visibility.
	self assertParsesAs: #(visibility strictProtected) from: 'strict protected' using: #visibility.! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/9/2008 21:57'!
testWhile
	self assertParsesAs: #(#while #true #()) from: 'while true do;' using: #whileStatement.
	self assertParsesAs: #(#while #(#< #(intNum '1') #(intNum '2')) #(#block)) from: 'while 1 < 2 do begin end;' using: #whileStatement.
	self assertParsesAs: #(#while #(#< #(intNum '1') #(intNum '2')) #(#block)) from: 'while (1 < 2) do begin end;' using: #whileStatement! !

!TestPyParser methodsFor: 'test-productions' stamp: 'fbs 12/9/2008 22:33'!
testWith
	self assertParsesAs: #(#with (#(#ident 'A')) #(ident 'foo')) from: 'with A do foo;' using: #withStatement.
	self assertParsesAs: #(#with (#(#ident 'A') #(#ident 'B')) #(#block #(ident 'foo'))) from: 'with A, B do begin foo; end;' using: #withStatement.
	self assertParsesAs: #(#with (#(#ident 'A') #(#ident 'B')) #(#block #(ident 'foo'))) from: 'with A , B do begin foo; end;' using: #withStatement.
	self assertParsesAs: #(#with (#(#ident 'A')) #(#block #(#ident 'foo') #(ident 'bar'))) from: 'with A do begin foo; bar; end' using: #withStatement.! !

TestCase subclass: #TestPyProgramHeadNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST-Tests'!

!TestPyProgramHeadNode methodsFor: 'private' stamp: 'fbs 5/19/2009 09:13'!
buildProgramHead
	^ PyProgramHeadNode new
		name: 'a';
		specialisers: #('b' 'c').! !

!TestPyProgramHeadNode methodsFor: 'test-converting' stamp: 'fbs 5/19/2009 10:14'!
testAsSexpr
	| p sfac |
	p := self buildProgramHead.
	sfac := PySexprNodeFactory new.
	
	self assert: p asSexpr = (sfac buildProgramHead: (sfac buildIdent: p name) with: p specialisers).! !

!TestPyProgramHeadNode methodsFor: 'test-accessing' stamp: 'fbs 5/19/2009 09:17'!
testChildren
	| ph |
	ph := self buildProgramHead.
	
	self assert: ph children size = 2.
	self assert: ph children first == ph name.
	self assert: ph children second == ph specialisers.! !

TestCase subclass: #TestPyProgramNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST-Tests'!

!TestPyProgramNode methodsFor: 'private' stamp: 'fbs 5/19/2009 09:01'!
buildProgram
	^ PyProgramNode new
		code: 'a';
		head: 'b';
		usedUnits: 'c';
		yourself.! !

!TestPyProgramNode methodsFor: 'test-converting' stamp: 'fbs 5/19/2009 10:12'!
testAsSexpr
	| p sfac |
	p := self buildProgram.
	sfac := PySexprNodeFactory new.
	
	self assert: p asSexpr = (sfac buildProgram: (p head asSexpr) using: p usedUnits block: (p code asSexpr)).! !

!TestPyProgramNode methodsFor: 'test-accessing' stamp: 'fbs 5/19/2009 09:02'!
testChildren
	| p |
	p := self buildProgram.
	
	self assert: p children size = 3.
	self assert: p children first == p head.
	self assert: p children second == p usedUnits.
	self assert: p children third == p code.! !

TestCase subclass: #TestPyPropertyNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST-Tests'!

TestPyPropertyNode subclass: #TestPyClassPropertyNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST-Tests'!

!TestPyClassPropertyNode methodsFor: 'private' stamp: 'fbs 5/25/2009 13:06'!
newNode
	^ PyClassPropertyNode new.! !

!TestPyClassPropertyNode methodsFor: 'test-accessing' stamp: 'fbs 5/25/2009 13:31'!
testAsSexpr
	| name property sfac specifiers type |
	name := PyIdentNode named: 'Foo'.
	specifiers := PyNamedCollectionNode named: #propertySpecifiers withElements: (PyCollectionNode new).
	type := PyIdentNode named: 'TFoo'.
	property := self newNode
		name: name;
		specifiers: specifiers;
		type: type;
		yourself.
	sfac := PySexprNodeFactory new.
		
	self assert: property asSexpr = (sfac buildClassProperty: (sfac
		buildProperty: name asOrderedCollection
		withArray: nil
		withType: type asOrderedCollection
		withIndex: nil
		withSpecifiers: #())) asSexpr.! !

!TestPyClassPropertyNode methodsFor: 'test-accessing' stamp: 'fbs 5/25/2009 13:32'!
testAsSexprArrayProperty
	| arrayDefn name property sfac specifiers type |
	arrayDefn := PyCollectionNode new.
	name := PyIdentNode named: 'Foo'.
	specifiers := PyNamedCollectionNode named: #propertySpecifiers withElements: (PyCollectionNode new).
	type := PyIdentNode named: 'TFoo'.
	property := self newNode
		arrayDefinition: arrayDefn;
		name: name;
		specifiers: specifiers;
		type: type;
		yourself.
	sfac := PySexprNodeFactory new.
		
	self assert: property asSexpr = ((sfac buildClassProperty: (sfac
		buildProperty: name asOrderedCollection
		withArray: arrayDefn asOrderedCollection
		withType: type asOrderedCollection
		withIndex: nil
		withSpecifiers: #())) asSexpr).! !

!TestPyClassPropertyNode methodsFor: 'test-accessing' stamp: 'fbs 5/25/2009 13:32'!
testAsSexprArrayPropertyPlusIndex
	| arrayDefn index name property sfac specifiers type |
	arrayDefn := PyCollectionNode new.
	index := PyIntegerNode value: '1'.
	name := PyIdentNode named: 'Foo'.
	specifiers := PyNamedCollectionNode named: #propertySpecifiers withElements: (PyCollectionNode new).
	type := PyIdentNode named: 'TFoo'.
	property := self newNode
		arrayDefinition: arrayDefn;
		index: index;
		name: name;
		specifiers: specifiers;
		type: type;
		yourself.
	sfac := PySexprNodeFactory new.
		
	self assert: property asSexpr = ((sfac buildClassProperty: (sfac
		buildProperty: name asOrderedCollection
		withArray: arrayDefn asOrderedCollection
		withType: type asOrderedCollection
		withIndex: index asOrderedCollection
		withSpecifiers: OrderedCollection new))) asSexpr.! !

!TestPyClassPropertyNode methodsFor: 'test-instance creation' stamp: 'fbs 5/25/2009 14:29'!
testFrom
	| baseProp classProp name specifiers type |
	name := PyIdentNode named: 'Foo'.
	type := PyIdentNode named: 'TFoo'.
	specifiers := PyCollectionNode new.
	baseProp := PyPropertyNode new
		name: name;
		type: type;
		specifiers: specifiers;
		yourself.
		
	classProp := PyClassPropertyNode from: baseProp.
	
	self assert: classProp name = baseProp name.
	self assert: classProp type = baseProp type.
	self assert: classProp specifiers = baseProp specifiers.! !

!TestPyPropertyNode methodsFor: 'private' stamp: 'fbs 5/25/2009 13:05'!
newNode
	^ PyPropertyNode new.! !

!TestPyPropertyNode methodsFor: 'test-accessing' stamp: 'fbs 5/25/2009 13:14'!
testAsSexpr
	| name property sfac specifiers type |
	name := PyIdentNode named: 'Foo'.
	specifiers := PyNamedCollectionNode named: #propertySpecifiers withElements: (PyCollectionNode new).
	type := PyIdentNode named: 'TFoo'.
	property := self newNode
		name: name;
		specifiers: specifiers;
		type: type;
		yourself.
	sfac := PySexprNodeFactory new.
		
	self assert: property asSexpr = (sfac
		buildProperty: name asOrderedCollection
		withArray: nil
		withType: type asOrderedCollection
		withIndex: nil
		withSpecifiers: #()) asSexpr.! !

!TestPyPropertyNode methodsFor: 'test-accessing' stamp: 'fbs 5/25/2009 13:20'!
testAsSexprArrayProperty
	| arrayDefn name property sfac specifiers type |
	arrayDefn := PyCollectionNode new.
	name := PyIdentNode named: 'Foo'.
	specifiers := PyNamedCollectionNode named: #propertySpecifiers withElements: (PyCollectionNode new).
	type := PyIdentNode named: 'TFoo'.
	property := self newNode
		arrayDefinition: arrayDefn;
		name: name;
		specifiers: specifiers;
		type: type;
		yourself.
	sfac := PySexprNodeFactory new.
		
	self assert: property asSexpr = (sfac
		buildProperty: name asOrderedCollection
		withArray: arrayDefn asOrderedCollection
		withType: type asOrderedCollection
		withIndex: nil
		withSpecifiers: #()) asSexpr.! !

!TestPyPropertyNode methodsFor: 'test-accessing' stamp: 'fbs 5/25/2009 13:22'!
testAsSexprArrayPropertyPlusIndex
	| arrayDefn index name property sfac specifiers type |
	arrayDefn := PyCollectionNode new.
	index := PyIntegerNode value: '1'.
	name := PyIdentNode named: 'Foo'.
	specifiers := PyNamedCollectionNode named: #propertySpecifiers withElements: (PyCollectionNode new).
	type := PyIdentNode named: 'TFoo'.
	property := self newNode
		arrayDefinition: arrayDefn;
		index: index;
		name: name;
		specifiers: specifiers;
		type: type;
		yourself.
	sfac := PySexprNodeFactory new.
		
	self assert: property asSexpr = (sfac
		buildProperty: name asOrderedCollection
		withArray: arrayDefn asOrderedCollection
		withType: type asOrderedCollection
		withIndex: index asOrderedCollection
		withSpecifiers: OrderedCollection new) asSexpr.! !

!TestPyPropertyNode methodsFor: 'test-accessing' stamp: 'fbs 5/25/2009 13:05'!
testChildren
	| name property specifiers type |
	name := PyNode new.
	specifiers := PyNode new.
	type := PyNode new.
	property := self newNode
		name: name;
		specifiers: specifiers;
		type: type;
		yourself.
		
	self assert: property children size = 3.
	self assert: property children first = name.
	self assert: property children second = type.
	self assert: property children third = specifiers.! !

!TestPyPropertyNode methodsFor: 'test-accessing' stamp: 'fbs 5/25/2009 13:05'!
testChildrenArrayProperty
	| arrayDefn index name property specifiers type |
	arrayDefn := PyNode new.
	index := PyNode new.
	name := PyNode new.
	specifiers := PyNode new.
	type := PyNode new.
	property := self newNode
		arrayDefinition: arrayDefn;
		index: index;
		name: name;
		specifiers: specifiers;
		type: type;
		yourself.
		
	self assert: property children size = 5.
	self assert: property children first = name.
	self assert: property children second = type.
	self assert: property children third = specifiers.
	self assert: property children fourth = index.
	self assert: property children fifth = arrayDefn.! !

!TestPyPropertyNode methodsFor: 'test-accessing' stamp: 'fbs 5/25/2009 13:05'!
testChildrenArrayPropertyNoIndex
	| arrayDefn name property specifiers type |
	arrayDefn := PyNode new.
	name := PyNode new.
	specifiers := PyNode new.
	type := PyNode new.
	property := self newNode
		arrayDefinition: arrayDefn;
		name: name;
		specifiers: specifiers;
		type: type;
		yourself.
		
	self assert: property children size = 4.
	self assert: property children first = name.
	self assert: property children second = type.
	self assert: property children third = specifiers.
	self assert: property children fourth = arrayDefn.! !

TestCase subclass: #TestPyUnitHeadNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST-Tests'!

!TestPyUnitHeadNode methodsFor: 'private' stamp: 'fbs 5/19/2009 09:32'!
buildUnitHead
	^ PyUnitHeadNode new
		name: 'a';
		hintingDirectives: #('b' 'c').! !

!TestPyUnitHeadNode methodsFor: 'test-accessing' stamp: 'fbs 5/19/2009 09:37'!
testAsSexpr
	| sfac uh |
	uh := self buildUnitHead.
	sfac := PySexprNodeFactory new.
	
	self assert: uh asSexpr = (sfac buildUnitHead: uh name withDirectives: uh hintingDirectives).! !

!TestPyUnitHeadNode methodsFor: 'test-accessing' stamp: 'fbs 5/19/2009 09:36'!
testChildren
	| uh |
	uh := self buildUnitHead.
	
	self assert: uh children size = 2.
	self assert: uh children first == uh name.
	self assert: uh children second == uh hintingDirectives.! !

TestCase subclass: #TestPyUnitInterfaceNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST-Tests'!

!TestPyUnitInterfaceNode methodsFor: 'test-converting' stamp: 'fbs 5/21/2009 10:09'!
testAsSexpr
	| decls sfac uintf usedUnits |
	decls := PyCollectionNode new.
	usedUnits := PyCollectionNode new.
	uintf := PyUnitInterfaceNode new
		declarations: decls;
		usedUnits: usedUnits;
		yourself.
	sfac := PySexprNodeFactory new.
	
	self assert: uintf asSexpr = ((sfac buildUnitInterface: {} using: {}) asSexpr).! !

!TestPyUnitInterfaceNode methodsFor: 'test-accessing' stamp: 'fbs 5/19/2009 16:03'!
testChildren
	| decls intf fac uses |
	fac := PyObjectNodeFactory new.
	decls := PyCollectionNode new.
	uses := PyCollectionNode new.
	intf := fac buildUnitInterface: decls using: uses.
	
	self assert: intf children size = 2.
	self assert: intf children first = uses.
	self assert: intf children second = decls.! !

TestCase subclass: #TestPyUnitNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST-Tests'!

!TestPyUnitNode methodsFor: 'private' stamp: 'fbs 5/19/2009 13:08'!
buildUnit
	^ PyUnitNode new
		head: (TestPyUnitHeadNode new buildUnitHead);
		interface: 'b';
		implementation: 'c';
		code: 'd'.! !

!TestPyUnitNode methodsFor: 'test-converting' stamp: 'fbs 5/19/2009 10:11'!
testAsSexpr
	| sfac u |
	u := self buildUnit.
	sfac := PySexprNodeFactory new.
	
	self assert: u asSexpr = (sfac buildUnit: (u head asSexpr) withInterface: (u interface asSexpr) withImplementation: (u implementation asSexpr) withBlock: (u code asSexpr)).! !

!TestPyUnitNode methodsFor: 'test-accessing' stamp: 'fbs 5/19/2009 09:28'!
testChildren
	| u |
	u := self buildUnit.
	
	self assert: u children size = 4.
	self assert: u children first == u head.
	self assert: u children second == u interface.
	self assert: u children third = u implementation.
	self assert: u children fourth = u code.! !

TestCase subclass: #TestPyVariableNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST-Tests'!

TestPyVariableNode subclass: #TestPyClassVarNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST-Tests'!

!TestPyClassVarNode methodsFor: 'private' stamp: 'fbs 5/25/2009 19:10'!
expectedForm: aVarNode
	^(PySexprNodeFactory new
		changeToClassVar: (super expectedForm: aVarNode)).! !

!TestPyClassVarNode methodsFor: 'private' stamp: 'fbs 5/25/2009 19:09'!
newNode
	^ PyClassVarNode new.! !

!TestPyClassVarNode methodsFor: 'test-instance creation' stamp: 'fbs 5/25/2009 19:27'!
testFrom
	| classVar sfac var |
	var := super newNode
		name: (PyIdentNode named: 'Foo');
		type: (PyIdentNode named: 'TFoo');
		hintingDirective: (PyDirectiveNode platform);
		yourself.
	classVar := PyClassVarNode from: var.
	sfac := PySexprNodeFactory new.
	
	self assert: classVar name = var name.
	self assert: classVar type = var type.
	
	self assert: classVar asSexpr = (sfac changeToClassVar: var asOrderedCollection) asSexpr.! !

TestPyVariableNode subclass: #TestPyInstVarNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST-Tests'!

!TestPyInstVarNode methodsFor: 'private' stamp: 'fbs 5/25/2009 19:12'!
expectedForm: aVarNode
	^(PySexprNodeFactory new
		changeToInstVar: (super expectedForm: aVarNode)).! !

!TestPyInstVarNode methodsFor: 'private' stamp: 'fbs 5/25/2009 19:12'!
newNode
	^ PyInstVarNode new.! !

!TestPyVariableNode methodsFor: 'private' stamp: 'fbs 5/25/2009 19:18'!
expectedForm: aVarNode
	^ (PySexprNodeFactory new
		buildVarDeclaration: {aVarNode name asOrderedCollection.}
		ofType: aVarNode type asOrderedCollection
		havingValue: nil
		hinting: (aVarNode hintingDirective ifNotNil: [(PyCollectionNode with: aVarNode hintingDirective) asOrderedCollection])) first.! !

!TestPyVariableNode methodsFor: 'private' stamp: 'fbs 5/25/2009 19:05'!
newNode
	^ PyVariableNode new.! !

!TestPyVariableNode methodsFor: 'test-converting' stamp: 'fbs 5/25/2009 19:13'!
testAsSexpr
	| name type v |
	name := PyIdentNode named: 'I'.
	type := PyIdentNode named: 'Integer'.
	v := self newNode
		name: name;
		type: type;
		yourself.

	self assert: v asSexpr = (self expectedForm: v) asSexpr.
	
	v hintingDirective: PyDirectiveNode platform.
	
	self assert: v asSexpr = (self expectedForm: v) asSexpr.! !

!TestPyVariableNode methodsFor: 'test-accessing' stamp: 'fbs 5/22/2009 21:27'!
testChildren
	| name type v value |
	name := PyIdentNode named: 'I'.
	type := PyIdentNode named: 'Integer'.
	value := PyIntegerNode value: '123'.
	v := PyVariableNode new
		name: name;
		type: type;
		value: value;
		yourself.
		
	self assert: v children size = 4.
	self assert: v children first = name.
	self assert: v children second = type.
	self assert: v children third = value.
	self assert: v children fourth isEmpty.
	
	v hintingDirective: PyDirectiveNode platform.

	self assert: v children fourth size = 1.
	self assert: v children fourth first platform.! !

!TestPyVariableNode methodsFor: 'test-copying' stamp: 'fbs 5/25/2009 19:35'!
testFrom
	| v newVar |
	v := self newNode.
	newVar := v copy.
	
	self assert: newVar name = v name.
	self assert: newVar type = v type.
	self assert: newVar hintingDirective = v hintingDirective.! !

!TestPyVariableNode methodsFor: 'test-instance creation' stamp: 'fbs 5/25/2009 19:18'!
testNameType
	| name type var |
	name := PyIdentNode named: 'Foo'.
	type := PyIdentNode named: 'TFoo'.
	
	var := self newNode class name: name type: type.
	
	self assert: var name = name.
	self assert: var type = type.! !

TestCase subclass: #TestPythiaExtensions
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-Tests'!

!TestPythiaExtensions methodsFor: 'test-converting' stamp: 'fbs 5/21/2009 10:02'!
testAsSexpr
	self assert: 1 asSexpr = '1'.
	self assert: '1' asSexpr = '"1"'.
	self assert: #i asSexpr = '''i'.
	self assert: Array new asSexpr = '()'.
	self assert: #(1 2) asSexpr = '(1 2)'.
	self assert: #(1 'two') asSexpr = '(1 "two")'.
	self assert: #(1 #(2)) asSexpr = '(1 (2))'.! !

!Symbol methodsFor: '*Pythia-Parser-converting' stamp: 'fbs 5/21/2009 10:00'!
asSexpr
	^ '''', self asString.! !

!Object methodsFor: '*Pythia-Parser-converting' stamp: 'fbs 12/9/2008 21:45'!
asSexpr
	^ self asString.! !

Object subclass: #PyCodeWalker
	instanceVariableNames: 'hasTerminated'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-Analysis'!

!PyCodeWalker commentStamp: 'fbs 5/19/2009 14:05' prior: 0!
I provide a framework for iterating over an AST in a pre-order fashion.

Subclasses may abort a codewalk by "self terminate: true".!

!PyCodeWalker methodsFor: 'testing' stamp: 'fbs 5/19/2009 14:04'!
hasTerminated
	"hasTerminated is true iff the code walk has been aborted."
	^ hasTerminated! !

!PyCodeWalker methodsFor: 'private' stamp: 'fbs 5/19/2009 15:16'!
ifNotTerminated: aBlock
	self hasTerminated ifFalse: aBlock.! !

!PyCodeWalker methodsFor: 'initialize-release' stamp: 'fbs 5/19/2009 15:18'!
initialize
	super initialize.
	self reset.! !

!PyCodeWalker methodsFor: 'initialization' stamp: 'fbs 5/19/2009 15:18'!
reset
	hasTerminated := false.! !

!PyCodeWalker methodsFor: 'private' stamp: 'fbs 5/19/2009 14:04'!
terminate: aBoolean
	"Set to true to cut short the codewalk because, for instance, you have all the information you need."
	hasTerminated := aBoolean.! !

!PyCodeWalker methodsFor: 'enumerating' stamp: 'fbs 5/19/2009 15:38'!
visitAssign: aNode
	self visitNode: aNode.! !

!PyCodeWalker methodsFor: 'enumerating' stamp: 'fbs 5/19/2009 15:38'!
visitCase: aNode
	self visitNode: aNode.! !

!PyCodeWalker methodsFor: 'enumerating' stamp: 'fbs 5/19/2009 15:38'!
visitConstant: aNode
	self visitNode: aNode.! !

!PyCodeWalker methodsFor: 'enumerating' stamp: 'fbs 5/19/2009 15:38'!
visitFor: aNode
	self visitNode: aNode.! !

!PyCodeWalker methodsFor: 'enumerating' stamp: 'fbs 5/19/2009 15:39'!
visitIf: aNode
	self visitNode: aNode.! !

!PyCodeWalker methodsFor: 'enumerating' stamp: 'fbs 5/19/2009 15:37'!
visitNode: aNode! !

!PyCodeWalker methodsFor: 'enumerating' stamp: 'fbs 5/19/2009 15:39'!
visitRepeatUntil: aNode
	self visitNode: aNode.! !

!PyCodeWalker methodsFor: 'enumerating' stamp: 'fbs 5/19/2009 15:15'!
walk: anAST
	self ifNotTerminated: [anAST accept: self].
	anAST children do:
		[:each |
		self ifNotTerminated: [self walk: each]].! !

!PyCodeWalker methodsFor: 'enumerating' stamp: 'fbs 5/19/2009 15:06'!
walkAll: aCollection
	aCollection do: [:each | self walk: each].! !

PyCodeWalker subclass: #PyObjectNamer
	instanceVariableNames: 'output'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-Analysis'!

!PyObjectNamer commentStamp: 'fbs 5/19/2009 13:00' prior: 0!
I provide a way of testing the PyCodeWalker algorithms by simply outputting nodes' class types to a string.!

PyObjectNamer subclass: #PyNullWalker
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-Analysis'!

!PyNullWalker commentStamp: 'fbs 5/19/2009 14:06' prior: 0!
I do not walk an AST at all.!

!PyNullWalker methodsFor: 'enumerating' stamp: 'fbs 5/19/2009 15:36'!
visitNode: aNode
	self terminate: true.! !

!PyObjectNamer methodsFor: 'accessing' stamp: 'fbs 5/19/2009 13:04'!
output
	^ output.! !

!PyObjectNamer methodsFor: 'enumerating' stamp: 'fbs 5/19/2009 15:36'!
visitNode: aNode
	| className |
	className := aNode class asString.
	
	output := output
		ifNil: [className]
		ifNotNil: [output, ' ', className].! !

PyCodeWalker subclass: #PyUnitDependencyFinder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-Analysis'!

!PyUnitDependencyFinder commentStamp: 'fbs 5/19/2009 12:19' prior: 0!
I walk a unit, library, package, program or whatever - or a set of same - and I create a string containing a graphviz description of the dependencies between those various files.!

Object subclass: #PyNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

!PyNode commentStamp: 'fbs 5/18/2009 20:21' prior: 0!
I am the root object of all nodes in a Pythia AST. I provide some basic functionality.!

PyNode subclass: #PyAddressOfNode
	instanceVariableNames: 'reference'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

!PyAddressOfNode class methodsFor: 'instance creation' stamp: 'fbs 5/21/2009 10:58'!
referencing: aVariableNode
	^ self new reference: aVariableNode; yourself.! !

!PyAddressOfNode methodsFor: 'converting' stamp: 'fbs 5/21/2009 10:40'!
asOrderedCollection
	| sfac |
	sfac := PySexprNodeFactory new.
	
	^ sfac buildAddressOf: (self reference asOrderedCollection).! !

!PyAddressOfNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 14:58'!
children
	^ {self reference.}! !

!PyAddressOfNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 10:18'!
reference
	^ reference.! !

!PyAddressOfNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 10:18'!
reference: aFactorNode
	reference := aFactorNode.! !

PyAddressOfNode subclass: #PyAddressOfProcNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

PyNode subclass: #PyAssignmentNode
	instanceVariableNames: 'lvalue rvalue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

!PyAssignmentNode class methodsFor: 'as yet unclassified' stamp: 'fbs 5/25/2009 19:46'!
set: lvalue to: rvalue
	^ self new
		lvalue: lvalue;
		rvalue: rvalue;
		yourself.! !

!PyAssignmentNode methodsFor: 'converting' stamp: 'fbs 5/25/2009 19:53'!
asOrderedCollection
	| sfac |
	sfac := PySexprNodeFactory new.
	^ sfac buildAssign: self rvalue to: self lvalue.! !

!PyAssignmentNode methodsFor: 'accessing' stamp: 'fbs 5/18/2009 20:55'!
children
	^ {self lvalue. self rvalue.}! !

!PyAssignmentNode methodsFor: 'accessing' stamp: 'fbs 5/18/2009 20:50'!
lvalue
	^ lvalue.! !

!PyAssignmentNode methodsFor: 'accessing' stamp: 'fbs 5/18/2009 20:50'!
lvalue: aVariable
	lvalue := aVariable.! !

!PyAssignmentNode methodsFor: 'accessing' stamp: 'fbs 5/18/2009 20:50'!
rvalue
	^ rvalue.! !

!PyAssignmentNode methodsFor: 'accessing' stamp: 'fbs 5/18/2009 20:51'!
rvalue: anExpression
	rvalue := anExpression.! !

PyNode subclass: #PyBinaryOperatorNode
	instanceVariableNames: 'leftExpression rightExpression operator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

!PyBinaryOperatorNode commentStamp: 'fbs 5/19/2009 13:41' prior: 0!
I represent the application of an operator to two operands. For example, "1 + 2", "a shl b", "foo < bar".!

!PyBinaryOperatorNode methodsFor: 'converting' stamp: 'fbs 5/21/2009 15:05'!
asOrderedCollection
	^ OrderedCollection
		with: self operator asOrderedCollection
		with: self leftExpression asOrderedCollection
		with: self rightExpression asOrderedCollection.! !

!PyBinaryOperatorNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 13:30'!
children
	^ {self leftExpression. self operator. self rightExpression.}! !

!PyBinaryOperatorNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 13:07'!
leftExpression
	^ leftExpression.! !

!PyBinaryOperatorNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 13:07'!
leftExpression: anExpression
	leftExpression := anExpression.! !

!PyBinaryOperatorNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 13:28'!
operator
	^ operator.! !

!PyBinaryOperatorNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 13:28'!
operator: aSymbol
	operator := aSymbol.! !

!PyBinaryOperatorNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 13:07'!
rightExpression
	^ rightExpression.! !

!PyBinaryOperatorNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 13:07'!
rightExpression: anExpression
	rightExpression := anExpression.! !

PyNode subclass: #PyCollectionNode
	instanceVariableNames: 'items'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

!PyCollectionNode commentStamp: 'fbs 5/19/2009 15:44' prior: 0!
I contain a collection of Nodes.!

PyCollectionNode subclass: #PyBlockNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

!PyBlockNode commentStamp: 'fbs 5/21/2009 13:31' prior: 0!
I represent a compound statement.!

PyBlockNode subclass: #PyAsmBlockNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

!PyAsmBlockNode commentStamp: 'fbs 5/21/2009 13:31' prior: 0!
I represent an "asm block". I thus contain only assembly statements.!

!PyAsmBlockNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 13:13'!
name
	^ #asm.! !

!PyBlockNode class methodsFor: 'instance creation' stamp: 'fbs 5/21/2009 13:19'!
with: aStatementList
	^ self new items: aStatementList; yourself.! !

!PyBlockNode methodsFor: 'as yet unclassified' stamp: 'fbs 5/21/2009 15:01'!
asOrderedCollection
	^ (OrderedCollection with: self name)
		addAll: self statements;
		yourself.! !

!PyBlockNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 13:18'!
name
	^ #block.! !

!PyBlockNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 13:20'!
statements
	^ self items.! !

!PyCollectionNode class methodsFor: 'instance creation' stamp: 'fbs 5/22/2009 15:12'!
with: object1
	^ self new
		add: object1;
		yourself.! !

!PyCollectionNode class methodsFor: 'instance creation' stamp: 'fbs 5/21/2009 13:45'!
with: object1 with: object2
	^ self new
		add: object1;
		add: object2;
		yourself.! !

!PyCollectionNode methodsFor: 'converting' stamp: 'fbs 5/22/2009 13:41'!
asOrderedCollection
	^ self items collect: [:each | each asOrderedCollection].! !

!PyCollectionNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 15:44'!
children
	^ self items.! !

!PyCollectionNode methodsFor: 'error handling' stamp: 'fbs 5/25/2009 19:00'!
doesNotUnderstand: aMessage
	^ (self items class canUnderstand: aMessage selector)
		ifTrue: [self items perform: aMessage selector withArguments: aMessage arguments]
		ifFalse: [super doesNotUnderstand: aMessage].! !

!PyCollectionNode methodsFor: 'initialize-release' stamp: 'fbs 5/19/2009 15:44'!
initialize
	super initialize.
	items := OrderedCollection new.! !

!PyCollectionNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 15:45'!
items
	^ items.! !

!PyCollectionNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 11:19'!
items: aCollection
	items := aCollection.! !

!PyCollectionNode methodsFor: 'accessing' stamp: 'fbs 5/22/2009 15:14'!
size
	^ self items size.! !

PyCollectionNode subclass: #PyCompoundStatementNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

!PyCompoundStatementNode class methodsFor: 'as yet unclassified' stamp: 'fbs 5/25/2009 19:50'!
newFrom: aCollectionNode
	^ self new
		addAll: aCollectionNode;
		yourself.! !

!PyCompoundStatementNode methodsFor: 'converting' stamp: 'fbs 5/25/2009 19:55'!
asOrderedCollection
	| sfac |
	sfac := PySexprNodeFactory new.
	^ sfac buildCompoundStatement: self items.! !

PyCollectionNode subclass: #PyNamedCollectionNode
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

!PyNamedCollectionNode commentStamp: 'fbs 5/21/2009 11:38' prior: 0!
I hold a collection of PyNodes, and have a name.!

!PyNamedCollectionNode class methodsFor: 'instance creation' stamp: 'fbs 5/21/2009 11:20'!
named: aSymbol withElements: aCollectionOfExpressions
	^ self new
		name: aSymbol;
		items: aCollectionOfExpressions asOrderedCollection; yourself.! !

!PyNamedCollectionNode methodsFor: 'converting' stamp: 'fbs 5/25/2009 19:55'!
asOrderedCollection
	| sfac |
	sfac := PySexprNodeFactory new.
	^ sfac namedList: self name withElements: self items.! !

!PyNamedCollectionNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 15:28'!
children
	^ (OrderedCollection with: self name)
		addAll: (self items asOrderedCollection);
		yourself.! !

!PyNamedCollectionNode methodsFor: 'copying' stamp: 'fbs 5/25/2009 13:27'!
copy
	^ self class new
		name: self name;
		addAll: self items;
		yourself.! !

!PyNamedCollectionNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 11:10'!
name
	^ name.! !

!PyNamedCollectionNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 11:10'!
name: aSymbol
	name := aSymbol.! !

PyCollectionNode subclass: #PyVariableClauseNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

PyNode subclass: #PyCommentNode
	instanceVariableNames: 'comment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

!PyCommentNode commentStamp: '<historical>' prior: 0!
I am a comment!

!PyCommentNode class methodsFor: 'instance creation' stamp: 'fbs 5/25/2009 15:20'!
saying: aString
	^ self new
		comment: aString;
		yourself.! !

!PyCommentNode methodsFor: 'converting' stamp: 'fbs 5/25/2009 15:18'!
asOrderedCollection
	| sfac |
	sfac := PySexprNodeFactory new.
	^ sfac buildComment: self comment.! !

!PyCommentNode methodsFor: 'accessing' stamp: 'fbs 5/25/2009 15:18'!
comment
	^ comment.! !

!PyCommentNode methodsFor: 'accessing' stamp: 'fbs 5/25/2009 15:18'!
comment: aString
	comment := aString.! !

PyNode subclass: #PyConstantNode
	instanceVariableNames: 'operator value literalValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

!PyConstantNode commentStamp: 'fbs 5/19/2009 13:58' prior: 0!
I represent a Delphi constant - an integer, a string, a Boolean, whatever.!

!PyConstantNode class methodsFor: 'instance creation' stamp: 'fbs 5/21/2009 09:31'!
value: aString
	^ self new literalValue: aString; yourself.! !

!PyConstantNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 09:25'!
literalValue
	^ literalValue! !

!PyConstantNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 09:25'!
literalValue: aString
	literalValue := aString.! !

!PyConstantNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 09:26'!
value
	"literalValue shows us what was present in the source file. Value returns the 'meaning' of that string. Thus, PyIntegerNode might return self literalValue asInteger, etc."
	self subclassResponsibility.! !

PyConstantNode subclass: #PyIntegerNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

!PyIntegerNode methodsFor: 'converting' stamp: 'fbs 5/25/2009 13:21'!
asOrderedCollection
	| sfac |
	sfac := PySexprNodeFactory new.
	^ sfac buildIntNum: self literalValue.! !

PyNode subclass: #PyDirectiveNode
	instanceVariableNames: 'name value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

!PyDirectiveNode class methodsFor: 'instance creation' stamp: 'fbs 5/22/2009 20:07'!
deprecated
	^ self named: self deprecatedName.! !

!PyDirectiveNode class methodsFor: 'constants' stamp: 'fbs 5/22/2009 20:08'!
deprecatedName
	^ 'deprecated'! !

!PyDirectiveNode class methodsFor: 'instance creation' stamp: 'fbs 5/22/2009 20:08'!
experimental
	^ self named: self experimentalName.! !

!PyDirectiveNode class methodsFor: 'constants' stamp: 'fbs 5/22/2009 20:08'!
experimentalName
	^ 'experimental'! !

!PyDirectiveNode class methodsFor: 'instance creation' stamp: 'fbs 5/22/2009 20:08'!
library
	^ self named: self libraryName.! !

!PyDirectiveNode class methodsFor: 'constants' stamp: 'fbs 5/22/2009 20:08'!
libraryName
	^ 'library'! !

!PyDirectiveNode class methodsFor: 'instance creation' stamp: 'fbs 5/22/2009 19:59'!
named: aString
	^ self new
		name: aString;
		yourself.! !

!PyDirectiveNode class methodsFor: 'instance creation' stamp: 'fbs 5/22/2009 20:08'!
platform
	^ self named: self platformName.! !

!PyDirectiveNode class methodsFor: 'constants' stamp: 'fbs 5/22/2009 20:08'!
platformName
	^ 'platform'! !

!PyDirectiveNode methodsFor: 'converting' stamp: 'fbs 5/22/2009 21:33'!
asOrderedCollection
	| sfac |
	sfac := PySexprNodeFactory new.
	
	^ self hasValue
		ifFalse: [sfac buildDirective: self name]
		ifTrue: [sfac buildDirective: self name withValue: self value literalValue]! !

!PyDirectiveNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 15:12'!
children
	^ self hasValue
		ifTrue: [{self value.}]
		ifFalse: [super children].! !

!PyDirectiveNode methodsFor: 'testing' stamp: 'fbs 5/22/2009 20:06'!
deprecated
	^ self name = self class deprecatedName.! !

!PyDirectiveNode methodsFor: 'testing' stamp: 'fbs 5/22/2009 20:06'!
experimental
	^ self name = self class experimentalName.! !

!PyDirectiveNode methodsFor: 'testing' stamp: 'fbs 5/21/2009 15:13'!
hasValue
	^ self value notNil.! !

!PyDirectiveNode methodsFor: 'testing' stamp: 'fbs 5/22/2009 20:06'!
library
	^ self name = self class libraryName.! !

!PyDirectiveNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 09:30'!
name
	^ name.! !

!PyDirectiveNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 09:30'!
name: aString
	name := aString.! !

!PyDirectiveNode methodsFor: 'testing' stamp: 'fbs 5/22/2009 20:07'!
platform
	^ self name = self class platformName.! !

!PyDirectiveNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 09:38'!
value
	^ value.! !

!PyDirectiveNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 09:39'!
value: aConstantExpression
	value := aConstantExpression.! !

PyNode subclass: #PyDistinctTypeNode
	instanceVariableNames: 'referringType'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

!PyDistinctTypeNode commentStamp: 'fbs 5/21/2009 12:09' prior: 0!
A "distinct type", in Pythia terms, is a type like this: "type TFoo = type String".!

!PyDistinctTypeNode class methodsFor: 'as yet unclassified' stamp: 'fbs 5/21/2009 12:10'!
distinctFrom: anIdentNode
	^ self new referringType: anIdentNode; yourself.! !

!PyDistinctTypeNode methodsFor: 'converting' stamp: 'fbs 5/21/2009 12:41'!
asOrderedCollection
	| sfac |
	sfac := PySexprNodeFactory new.
	
	^ sfac buildType: (self referringType asOrderedCollection) isDistinct: true.! !

!PyDistinctTypeNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 12:39'!
children
	^ {self referringType.}! !

!PyDistinctTypeNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 12:09'!
referringType
	^ referringType.! !

!PyDistinctTypeNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 12:09'!
referringType: anIdentNode
	referringType := anIdentNode.! !

PyNode subclass: #PyIdentNode
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

!PyIdentNode commentStamp: 'fbs 5/21/2009 10:43' prior: 0!
I represent an identifier. Note I don't represent a qualified identifier, so the AST of "Foo.Bar.Baz" has three PyIdentNodes.!

!PyIdentNode class methodsFor: 'instance creation' stamp: 'fbs 5/21/2009 10:47'!
named: aString
	^ self new name: aString; yourself.! !

!PyIdentNode methodsFor: 'converting' stamp: 'fbs 5/21/2009 10:45'!
asOrderedCollection
	| sfac |
	sfac := PySexprNodeFactory new.
	^ sfac buildIdent: self name.! !

!PyIdentNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 10:43'!
name
	^ name.! !

!PyIdentNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 10:44'!
name: aString
	name := aString.! !

!PyNode methodsFor: 'visiting' stamp: 'fbs 5/19/2009 15:35'!
accept: aCodeWalker
	aCodeWalker visitNode: self.! !

!PyNode methodsFor: 'converting' stamp: 'fbs 5/21/2009 10:36'!
asOrderedCollection
	"Return an OrderedCollection just like what PySexprNodeFactory would return."
	self subclassResponsibility.! !

!PyNode methodsFor: 'converting' stamp: 'fbs 5/21/2009 10:40'!
asSexpr
	^ self asOrderedCollection asSexpr.! !

!PyNode methodsFor: 'accessing' stamp: 'fbs 5/18/2009 20:22'!
children
	"Return a list of the child nodes I point to or, if a leaf node, return an empty list."
	^ #().! !

PyNode subclass: #PyOperatorNode
	instanceVariableNames: 'operator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

!PyOperatorNode commentStamp: 'fbs 5/19/2009 13:41' prior: 0!
I represent an operator: -, +, shl, as, etc.!

!PyOperatorNode methodsFor: 'converting' stamp: 'fbs 5/19/2009 13:46'!
asSexpr
	^ self operator asSexpr.! !

!PyOperatorNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 13:41'!
operator
	^ operator.! !

!PyOperatorNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 13:41'!
operator: aSymbol
	operator := aSymbol.! !

PyNode subclass: #PyProgramHeadNode
	instanceVariableNames: 'name specialisers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

!PyProgramHeadNode commentStamp: 'fbs 5/19/2009 09:10' prior: 0!
I represent the optional "head" of a program. Specialisers contains the parameters of the head, as in "program Foo(Spec1, Spec2);". In practice, noone uses these parameters.!

!PyProgramHeadNode methodsFor: 'converting' stamp: 'fbs 5/19/2009 10:13'!
asSexpr
	| sfac |
	sfac := PySexprNodeFactory new.
	^ sfac buildProgramHead: (sfac buildIdent: self name) with: self specialisers.! !

!PyProgramHeadNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 09:17'!
children
	^ {self name. self specialisers.}! !

!PyProgramHeadNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 09:11'!
name
	^ name.! !

!PyProgramHeadNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 09:11'!
name: anIdentifier
	name := anIdentifier.! !

!PyProgramHeadNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 09:11'!
specialisers
	^ specialisers.! !

!PyProgramHeadNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 09:11'!
specialisers: aCollection
	specialisers := aCollection.! !

PyNode subclass: #PyProgramNode
	instanceVariableNames: 'head name usedUnits'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

!PyProgramNode commentStamp: 'fbs 5/18/2009 21:46' prior: 0!
I represent an entire program (a .dpr file).!

!PyProgramNode methodsFor: 'converting' stamp: 'fbs 5/19/2009 10:12'!
asSexpr
	| sfac |
	sfac := PySexprNodeFactory new.
	^ sfac buildProgram: (self head asSexpr) using: self usedUnits block: (self code asSexpr).! !

!PyProgramNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 09:02'!
children
	^ {self head. self usedUnits. self code.}! !

!PyProgramNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 08:37'!
code
	^ code.! !

!PyProgramNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 08:37'!
code: aPythiaBlock
	code := aPythiaBlock.! !

!PyProgramNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 09:01'!
head
	^head.! !

!PyProgramNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 09:01'!
head: aProgramHead
	head := aProgramHead.! !

!PyProgramNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 08:37'!
usedUnits
	^usedUnits.! !

!PyProgramNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 08:37'!
usedUnits: aUsesClause
	usedUnits := aUsesClause.! !

PyNode subclass: #PyPropertyNode
	instanceVariableNames: 'arrayDefinition index specifiers type name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

!PyPropertyNode commentStamp: 'fbs 5/25/2009 12:51' prior: 0!
I represent a property.!

PyPropertyNode subclass: #PyClassPropertyNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

!PyClassPropertyNode commentStamp: 'fbs 5/25/2009 12:52' prior: 0!
I represent a _class_ property, that is, a property shared by all instances of some class.!

!PyClassPropertyNode class methodsFor: 'as yet unclassified' stamp: 'fbs 5/25/2009 14:28'!
from: aPropertyNode
	^ self new
		name: aPropertyNode name;
		type: aPropertyNode type;
		specifiers: aPropertyNode specifiers;
		arrayDefinition: aPropertyNode arrayDefinition;
		index: aPropertyNode index;
		yourself.! !

!PyClassPropertyNode methodsFor: 'converting' stamp: 'fbs 5/25/2009 14:53'!
asOrderedCollection
	| sfac |
	sfac := PySexprNodeFactory new.
	
	^ sfac buildClassProperty: super asOrderedCollection.! !

!PyPropertyNode methodsFor: 'accessing' stamp: 'fbs 5/25/2009 12:57'!
arrayDefinition
	^ arrayDefinition.! !

!PyPropertyNode methodsFor: 'accessing' stamp: 'fbs 5/25/2009 12:57'!
arrayDefinition: anArrayDefn
	arrayDefinition := anArrayDefn.! !

!PyPropertyNode methodsFor: 'converting' stamp: 'fbs 5/25/2009 14:52'!
asOrderedCollection
	| sfac |
	sfac := PySexprNodeFactory new.
	^ sfac
		buildProperty: self name asOrderedCollection
		withArray:  (self arrayDefinition
			ifNil: [OrderedCollection new]
			ifNotNil: [self arrayDefinition asOrderedCollection])
		withType: self type asOrderedCollection
		withIndex: (self index
			ifNil: [nil]
			ifNotNil: [self index asOrderedCollection])
		withSpecifiers: self specifiers items asOrderedCollection.! !

!PyPropertyNode methodsFor: 'accessing' stamp: 'fbs 5/25/2009 13:00'!
children
	| result |
	result := OrderedCollection
		with: self name
		with: self type
		with: self specifiers.
		
	self index ifNotNil: [result add: self index].
	self arrayDefinition ifNotNil: [result add: self arrayDefinition].
	
	^ result.! !

!PyPropertyNode methodsFor: 'accessing' stamp: 'fbs 5/25/2009 12:58'!
index
	^ index.! !

!PyPropertyNode methodsFor: 'accessing' stamp: 'fbs 5/25/2009 12:58'!
index: anExpression
	index := anExpression.! !

!PyPropertyNode methodsFor: 'accessing' stamp: 'fbs 5/25/2009 12:59'!
name
	^ name.! !

!PyPropertyNode methodsFor: 'accessing' stamp: 'fbs 5/25/2009 12:59'!
name: anIdentNode.
	name := anIdentNode.! !

!PyPropertyNode methodsFor: 'accessing' stamp: 'fbs 5/25/2009 12:58'!
specifiers
	^ specifiers.! !

!PyPropertyNode methodsFor: 'accessing' stamp: 'fbs 5/25/2009 12:58'!
specifiers: aCollectionNode
	specifiers := aCollectionNode.! !

!PyPropertyNode methodsFor: 'accessing' stamp: 'fbs 5/25/2009 12:59'!
type
	^ type.! !

!PyPropertyNode methodsFor: 'accessing' stamp: 'fbs 5/25/2009 12:59'!
type: aTypeNode
	type := aTypeNode.! !

PyNode subclass: #PyStatementNode
	instanceVariableNames: 'label'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

PyStatementNode subclass: #PyCaseStatementNode
	instanceVariableNames: 'condition defaultSwitch switches'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

!PyCaseStatementNode commentStamp: 'fbs 5/21/2009 14:11' prior: 0!
I represent a case statement:

case <condition>
  <switches>
else
  <defaultSwitch>
end.!

!PyCaseStatementNode methodsFor: 'converting' stamp: 'fbs 5/21/2009 14:46'!
asOrderedCollection
	| sfac |
	sfac := PySexprNodeFactory new.
	
	^ sfac
		buildCaseStatement: (self condition asOrderedCollection)
		withSwitches: (self switches asOrderedCollection)
		orElse: (self defaultSwitch asOrderedCollection).! !

!PyCaseStatementNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 14:32'!
children
	^ {self condition. self switches. self defaultSwitch.}! !

!PyCaseStatementNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 14:12'!
condition
	^ condition.! !

!PyCaseStatementNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 14:12'!
condition: anOrdinalExpression
	condition := anOrdinalExpression.! !

!PyCaseStatementNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 14:21'!
defaultSwitch
	^ defaultSwitch.! !

!PyCaseStatementNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 14:21'!
defaultSwitch: aBlock
	defaultSwitch := aBlock! !

!PyCaseStatementNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 14:21'!
switches
	^ switches.! !

!PyCaseStatementNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 14:21'!
switches: aCollection
	switches := aCollection.! !

PyStatementNode subclass: #PyCaseSwitchNode
	instanceVariableNames: 'values statement'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

!PyCaseSwitchNode methodsFor: 'converting' stamp: 'fbs 5/21/2009 14:46'!
asOrderedCollection
	| sfac |
	sfac := PySexprNodeFactory new.
	
	^ sfac
		buildCaseSwitchOn: (self values asOrderedCollection)
		do: (self statement asOrderedCollection).! !

!PyCaseSwitchNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 14:43'!
children
	^ {self values. self statement.}! !

!PyCaseSwitchNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 14:37'!
statement
	^ statement.! !

!PyCaseSwitchNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 14:37'!
statement: aStatementNode
	statement := aStatementNode.! !

!PyCaseSwitchNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 14:36'!
values
	^ values.! !

!PyCaseSwitchNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 14:37'!
values: aSet
	values := aSet.! !

PyStatementNode subclass: #PyIfStatementNode
	instanceVariableNames: 'ifBlock elseBlock condition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

!PyIfStatementNode commentStamp: 'fbs 5/19/2009 12:13' prior: 0!
My subclasses represent the various kinds of statements in the Delphi language.!

!PyIfStatementNode methodsFor: 'converting' stamp: 'fbs 5/18/2009 20:44'!
asSexpr
	| fac |
	fac := PySexprNodeFactory new.
	^ fac buildIf: self condition then: self ifBlock else: self elseBlock.! !

!PyIfStatementNode methodsFor: 'accessing' stamp: 'fbs 5/18/2009 20:37'!
children
	^ {self condition. self ifBlock. self elseBlock.}! !

!PyIfStatementNode methodsFor: 'accessing' stamp: 'fbs 5/18/2009 20:33'!
condition
	^ condition.! !

!PyIfStatementNode methodsFor: 'accessing' stamp: 'fbs 5/18/2009 20:34'!
condition: aBoolean
	condition := aBoolean.! !

!PyIfStatementNode methodsFor: 'accessing' stamp: 'fbs 5/18/2009 20:33'!
elseBlock
	^ elseBlock.! !

!PyIfStatementNode methodsFor: 'accessing' stamp: 'fbs 5/18/2009 20:33'!
elseBlock: aStatementList
	elseBlock := aStatementList! !

!PyIfStatementNode methodsFor: 'accessing' stamp: 'fbs 5/18/2009 20:33'!
ifBlock
	^ ifBlock.! !

!PyIfStatementNode methodsFor: 'accessing' stamp: 'fbs 5/18/2009 20:33'!
ifBlock: aStatementList
	ifBlock := aStatementList! !

!PyStatementNode methodsFor: 'accessing' stamp: 'fbs 5/18/2009 20:24'!
label
	^ label.! !

!PyStatementNode methodsFor: 'accessing' stamp: 'fbs 5/18/2009 20:25'!
label: anIdentifier
	label := anIdentifier.! !

PyNode subclass: #PyTypeNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

!PyTypeNode commentStamp: 'fbs 5/21/2009 12:05' prior: 0!
I represent a Delphi type.!

PyTypeNode subclass: #PyArrayTypeNode
	instanceVariableNames: 'elementType indices'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

!PyArrayTypeNode class methodsFor: 'instance creation' stamp: 'fbs 5/21/2009 11:58'!
containing: aTypeNode withIndices: aCollection
	^ self new
		elementType: aTypeNode;
		indices: aCollection;
		yourself.! !

!PyArrayTypeNode methodsFor: 'converting' stamp: 'fbs 5/21/2009 12:35'!
asOrderedCollection
	| sfac |
	sfac := PySexprNodeFactory new.
	^ sfac buildArrayType: (self elementType asOrderedCollection) withIndices: (self indices asOrderedCollection).! !

!PyArrayTypeNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 14:59'!
children
	^ {self elementType. self indices.}! !

!PyArrayTypeNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 11:39'!
elementType
	^ elementType.! !

!PyArrayTypeNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 11:40'!
elementType: aTypeNode
	elementType := aTypeNode.! !

!PyArrayTypeNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 11:42'!
indices
	^ indices.! !

!PyArrayTypeNode methodsFor: 'accessing' stamp: 'fbs 5/21/2009 11:42'!
indices: aCollection
	indices := aCollection.! !

PyTypeNode subclass: #PyClassNode
	instanceVariableNames: 'superclasses clauses option'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

!PyClassNode commentStamp: 'fbs 5/22/2009 14:13' prior: 0!
I represent a class declaration.

The option instvar tells us if the class is sealed or abstract. Normal class declarations will leave option as nil.!

!PyClassNode methodsFor: 'accessing' stamp: 'fbs 5/25/2009 19:43'!
abstract
	^ self option == #abstract.! !

!PyClassNode methodsFor: 'converting' stamp: 'fbs 5/22/2009 15:47'!
asOrderedCollection
	| result |
	result := OrderedCollection
		with: #class
		with: (self superclasses asOrderedCollection)
		with: (self clauses asOrderedCollection).
		
	self option notNil ifTrue: [result add: self option].
	
	^ result.! !

!PyClassNode methodsFor: 'accessing' stamp: 'fbs 5/25/2009 19:43'!
beAbstract
	self option: #abstract.! !

!PyClassNode methodsFor: 'accessing' stamp: 'fbs 5/25/2009 19:43'!
beSealed
	self option: #sealed.! !

!PyClassNode methodsFor: 'accessing' stamp: 'fbs 5/22/2009 15:38'!
children
	^ {self superclasses. self clauses.}.! !

!PyClassNode methodsFor: 'accessing' stamp: 'fbs 5/22/2009 14:11'!
clauses
	^ clauses.! !

!PyClassNode methodsFor: 'accessing' stamp: 'fbs 5/22/2009 14:11'!
clauses: aCollectionNode
	clauses := aCollectionNode.! !

!PyClassNode methodsFor: 'private' stamp: 'fbs 5/22/2009 13:33'!
defaultSuperclass
	^ PyIdentNode named: 'TObject'.! !

!PyClassNode methodsFor: 'initialize-release' stamp: 'fbs 5/22/2009 15:45'!
initialize
	super initialize.
	clauses := PyCollectionNode new.
	superclasses := PyCollectionNode new.! !

!PyClassNode methodsFor: 'accessing' stamp: 'fbs 5/22/2009 14:12'!
option
	^ option.! !

!PyClassNode methodsFor: 'accessing' stamp: 'fbs 5/22/2009 14:12'!
option: aSymbol
	option := aSymbol! !

!PyClassNode methodsFor: 'accessing' stamp: 'fbs 5/25/2009 19:44'!
sealed
	^ self option == #sealed.! !

!PyClassNode methodsFor: 'accessing' stamp: 'fbs 5/22/2009 13:32'!
superclasses
	^ superclasses.! !

!PyClassNode methodsFor: 'accessing' stamp: 'fbs 5/22/2009 14:11'!
superclasses: aCollectionNode
	superclasses := aCollectionNode.! !

PyTypeNode subclass: #PyClassTypeNode
	instanceVariableNames: 'classType'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

!PyClassTypeNode class methodsFor: 'instance creation' stamp: 'fbs 5/25/2009 14:34'!
of: aClassNode
	^ self new
		classType: aClassNode;
		yourself.! !

!PyClassTypeNode methodsFor: 'converting' stamp: 'fbs 5/25/2009 14:45'!
asOrderedCollection
	| sfac |
	sfac := PySexprNodeFactory new.
	
	^ sfac buildClassTypeRef: self classType asOrderedCollection.! !

!PyClassTypeNode methodsFor: 'accessing' stamp: 'fbs 5/25/2009 14:35'!
children
	^ OrderedCollection with: self classType.! !

!PyClassTypeNode methodsFor: 'accessing' stamp: 'fbs 5/25/2009 14:35'!
classType
	^ classType.! !

!PyClassTypeNode methodsFor: 'accessing' stamp: 'fbs 5/25/2009 14:35'!
classType: aClassNode
	classType := aClassNode.! !

PyNode subclass: #PyUnitHeadNode
	instanceVariableNames: 'hintingDirectives name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

!PyUnitHeadNode commentStamp: 'fbs 5/19/2009 09:34' prior: 0!
I represent the "head" of a unit - its name, and any directives that might apply to the entire unit.!

!PyUnitHeadNode methodsFor: 'converting' stamp: 'fbs 5/19/2009 09:58'!
asSexpr
	| sfac |
	sfac := PySexprNodeFactory new.
	^ sfac buildUnitHead: self name withDirectives: self hintingDirectives.! !

!PyUnitHeadNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 09:41'!
children
	^ {self name. self hintingDirectives.}! !

!PyUnitHeadNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 09:33'!
hintingDirectives
	^ hintingDirectives.! !

!PyUnitHeadNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 09:34'!
hintingDirectives: aCollection
	hintingDirectives := aCollection.! !

!PyUnitHeadNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 09:34'!
name
	^ name.! !

!PyUnitHeadNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 09:34'!
name: anIdentifier
	name := anIdentifier.! !

PyNode subclass: #PyUnitInterfaceNode
	instanceVariableNames: 'declarations usedUnits'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

!PyUnitInterfaceNode methodsFor: 'converting' stamp: 'fbs 5/21/2009 10:08'!
asSexpr
	| sfac |
	sfac := PySexprNodeFactory new.
	
	^ (sfac buildUnitInterface: (self declarations asOrderedCollection) using: (self usedUnits asOrderedCollection)) asSexpr.! !

!PyUnitInterfaceNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 16:03'!
children
	^ {self usedUnits. self declarations.}! !

!PyUnitInterfaceNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 15:51'!
declarations
	^ declarations.! !

!PyUnitInterfaceNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 15:50'!
declarations: aCollection
	declarations := aCollection.! !

!PyUnitInterfaceNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 15:50'!
usedUnits
	^ usedUnits.! !

!PyUnitInterfaceNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 15:50'!
usedUnits: aCollection
	usedUnits := aCollection.! !

PyNode subclass: #PyUnitNode
	instanceVariableNames: 'code head interface implementation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

!PyUnitNode commentStamp: 'fbs 5/19/2009 09:24' prior: 0!
I represent an entire unit - a .pas file.

I have a "code" instvar rather than things like "initialization" or "finalization" because a unit can end in a compound statement rather than initialization/finalization. I defer this disambiguation to PyUnitBlockNode.!

!PyUnitNode methodsFor: 'converting' stamp: 'fbs 5/19/2009 10:09'!
asSexpr
	| sfac |
	sfac := PySexprNodeFactory new.
	^ sfac buildUnit: (self head asSexpr) withInterface: (self interface asSexpr) withImplementation: (self implementation asSexpr) withBlock: (self code asSexpr).! !

!PyUnitNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 09:28'!
children
	^ {self head. self interface. self implementation. self code.}.! !

!PyUnitNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 09:23'!
code
	^ code.! !

!PyUnitNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 09:23'!
code: aUnitBlock
	code := aUnitBlock.! !

!PyUnitNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 09:26'!
head
	^ head.! !

!PyUnitNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 09:26'!
head: aUnitHead
	head := aUnitHead.! !

!PyUnitNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 09:22'!
implementation
	^ implementation.! !

!PyUnitNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 09:22'!
implementation: anImplementation
	implementation := anImplementation.! !

!PyUnitNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 09:22'!
interface
	^ interface.! !

!PyUnitNode methodsFor: 'accessing' stamp: 'fbs 5/19/2009 09:23'!
interface: anInterface
	interface := anInterface.! !

PyNode subclass: #PyVariableNode
	instanceVariableNames: 'name type hintingDirective value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

!PyVariableNode commentStamp: 'fbs 5/22/2009 20:15' prior: 0!
I represent a normal variable, as might appear in a var clause.!

PyVariableNode subclass: #PyClassHelperNode
	instanceVariableNames: 'helpedClass parent declarations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

!PyClassHelperNode class methodsFor: 'instance creation' stamp: 'fbs 5/23/2009 20:56'!
helping: anIdentNode parent: aParentIdent declarations: aCollection
	^ self new
		helpedClass: anIdentNode;
		parent: aParentIdent;
		declarations: aCollection;
		yourself.! !

!PyClassHelperNode methodsFor: 'accessing' stamp: 'fbs 5/23/2009 21:01'!
asOrderedCollection
	| sfac |
	sfac := PySexprNodeFactory new.
	
	^ sfac
		buildClassHelperFor: self helpedClass asOrderedCollection
		withParent: self parent asOrderedCollection
		having: self declarations asOrderedCollection.! !

!PyClassHelperNode methodsFor: 'accessing' stamp: 'fbs 5/23/2009 21:00'!
children
	^ {self helpedClass. self parent. self declarations.}.! !

!PyClassHelperNode methodsFor: 'accessing' stamp: 'fbs 5/23/2009 20:59'!
declarations
	^ declarations.! !

!PyClassHelperNode methodsFor: 'accessing' stamp: 'fbs 5/23/2009 20:59'!
declarations: aCollectionNode
	declarations := aCollectionNode.! !

!PyClassHelperNode methodsFor: 'accessing' stamp: 'fbs 5/23/2009 20:55'!
helpedClass
	^ helpedClass.! !

!PyClassHelperNode methodsFor: 'accessing' stamp: 'fbs 5/23/2009 20:55'!
helpedClass: anIdentNode
	helpedClass := anIdentNode.! !

!PyClassHelperNode methodsFor: 'initialize-release' stamp: 'fbs 5/23/2009 21:00'!
initialize
	super initialize.
	declarations := PyCollectionNode new.! !

!PyClassHelperNode methodsFor: 'accessing' stamp: 'fbs 5/23/2009 20:59'!
parent
	^ parent.! !

!PyClassHelperNode methodsFor: 'accessing' stamp: 'fbs 5/23/2009 20:59'!
parent: anIdentNode
	parent := anIdentNode.! !

PyVariableNode subclass: #PyClassVarNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

!PyClassVarNode commentStamp: 'fbs 5/25/2009 15:45' prior: 0!
I represent a classvar - a variable shared across all instances of a Delphi class.!

!PyClassVarNode class methodsFor: 'instance creation' stamp: 'fbs 5/25/2009 19:26'!
from: aVarNode
	^ self new
		name: aVarNode name;
		type: aVarNode type;
		hintingDirective: aVarNode hintingDirective;
		yourself.! !

!PyClassVarNode methodsFor: 'converting' stamp: 'fbs 5/25/2009 19:29'!
asOrderedCollection
	| sfac |
	sfac := PySexprNodeFactory new.
	
	^ sfac changeToClassVar: super asOrderedCollection.! !

PyVariableNode subclass: #PyInstVarNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-AST'!

!PyInstVarNode commentStamp: 'fbs 5/22/2009 20:12' prior: 0!
I represent an instance variable of a Delphi object. Other languages might call this a "member field".!

!PyInstVarNode methodsFor: 'converting' stamp: 'fbs 5/25/2009 19:29'!
asOrderedCollection
	| sfac |
	sfac := PySexprNodeFactory new.
	
	^ sfac changeToInstVar: super asOrderedCollection.! !

!PyVariableNode class methodsFor: 'instance creation' stamp: 'fbs 5/25/2009 19:34'!
from: aVarNode
	^ aVarNode copy.! !

!PyVariableNode class methodsFor: 'instance creation' stamp: 'fbs 5/25/2009 16:01'!
name: anIdentNode type: anotherIdentNode
	^ self new
		name: anIdentNode;
		type: anotherIdentNode;
		yourself.! !

!PyVariableNode methodsFor: 'converting' stamp: 'fbs 5/22/2009 21:28'!
asOrderedCollection
	| directives sfac |
	directives := PyCollectionNode new.
	self hintingDirective ifNotNil: [directives add: self hintingDirective].
	sfac := PySexprNodeFactory new.
	
	^ (sfac
		buildVarDeclaration: {self name asOrderedCollection.}
		ofType: self type asOrderedCollection
		havingValue: (self value ifNil: [nil] ifNotNil: [self value asOrderedCollection])
		hinting: directives asOrderedCollection) first.! !

!PyVariableNode methodsFor: 'accessing' stamp: 'fbs 5/22/2009 21:23'!
children
	| directives |
	directives := OrderedCollection new.
	self hintingDirective notNil ifTrue: [directives add: self hintingDirective].
	^ {self name. self type. self value. directives.}.! !

!PyVariableNode methodsFor: 'as yet unclassified' stamp: 'fbs 5/25/2009 19:35'!
copy
	^ self class new
		name: self name;
		type: self type;
		hintingDirective: self hintingDirective;
		yourself.! !

!PyVariableNode methodsFor: 'accessing' stamp: 'fbs 5/22/2009 20:58'!
hintingDirective
	^ hintingDirective.! !

!PyVariableNode methodsFor: 'accessing' stamp: 'fbs 5/22/2009 20:58'!
hintingDirective: aDirective
	hintingDirective := aDirective.! !

!PyVariableNode methodsFor: 'accessing' stamp: 'fbs 5/22/2009 20:57'!
name
	^ name.! !

!PyVariableNode methodsFor: 'accessing' stamp: 'fbs 5/22/2009 20:57'!
name: aString
	name := aString.! !

!PyVariableNode methodsFor: 'accessing' stamp: 'fbs 5/22/2009 20:57'!
type
	^ type.! !

!PyVariableNode methodsFor: 'accessing' stamp: 'fbs 5/22/2009 20:57'!
type: aTypeNode
	type := aTypeNode.! !

!PyVariableNode methodsFor: 'accessing' stamp: 'fbs 5/22/2009 21:23'!
value
	^ value.! !

!PyVariableNode methodsFor: 'accessing' stamp: 'fbs 5/22/2009 21:24'!
value: aConstantValueOrAbsoluteDirective
	value := aConstantValueOrAbsoluteDirective.! !

Object subclass: #PyNodeFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-Parser'!

!PyNodeFactory commentStamp: 'fbs 11/24/2008 21:48' prior: 0!
A PyNodeFactory is a Factory of nodes, used by a PyParser.

Its subclasses thus define the creation of different kinds of ASTs.!

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildAbsoluteDirective: identOrExpression! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildAddressOf: aFactor! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildAddressOfProc: aFactor! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildArrayIndex: anExpressionList! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildArrayType: elementType withIndices: aCollectionOfArrayIndices! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildAsmBlock: aStatementList! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildAssign: rValue to: lValue! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildBlock: aStatementList! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildBlock: body withDeclarations: aCollection! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildCaseStatement: ordinalExpression withSwitches: aListOfCaseItems orElse: aBlock! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildCaseSwitchOn: aSet do: aStatement! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 5/22/2009 14:53'!
buildClassDecl: superclassNames withClauses: aCollectionOfClauses withOption: aSymbol! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildClassField: aCollectionOfIdents ofType: anIdent hinting: aDirective! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildClassHelperFor: anIdent withParent: aClassParent having: declarations! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildClassProperty: aPropertyDesc! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildClassTypeRef: aType! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildClassVarClause: aVarSection! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildComment: aString! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildCompoundStatement: aCollection! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildConst: anIdent ofType: aType withValue: aConstantExpression! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildConst: anIdent ofType: aType withValue: aConstantExpression hinting: directives! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildConstRecord: anIdent value: aConstExpression! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildConstSection: constDeclarations! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildConstSection: constDeclarations named: aSymbol! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildContainsClause: namespaceFileNames! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 22:00'!
buildControlString: anIntNumOrHexNum
	^ OrderedCollection new.! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildDecimalFraction: integralPart with: fractionalPart! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildDereference! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildDesignator: head with: tail! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildDesignator: ident withParts: designatorParts! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildDirective: aString! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildDirective: aString withValue: anExpression! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildDirectives: aCollectionOfDirectives! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildDot: ident! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildElseBlock: aStatementList! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildEnum: aListOfValuedIdents! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildEnumValue: anIdent withOrdinality: anExpression! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildExportItem: name withParams: params withDirectives: aCollectionOfDirectives! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildExportsClause: aCollectionOfExportItems! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 5/11/2009 10:24'!
buildExpression: anExpression nextOperator: anOperator nextOperand: aSimpleExpression! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildExternalDirective: anIdent andSpecifiers: aCollectionOfDirectives! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildExternalDirective: anIdent withValue: anExpression andSpecifiers: aCollectionOfDirectives! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildExternalSpecifier: aNameOrIndex withValue: anExpression! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildFileType: aType! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildFor: anIdent over: aRange do: aStatement! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildForRangeFrom: aStartExpression to: anEndExpression! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildForwardClassDecl! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildFunctionCallArgument: argument! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildFunctionCallArguments: arguments! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildFunctionResult: aType! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildGenericConstruct: aListOfTypes! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildGoto: aLabel! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildGuid: aString! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildHandler: anIdent for: aType doing: aStatement! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildHexNum: aString! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 5/22/2009 14:16'!
buildIdent: aString
	^ PyIdentNode named: aString.! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildIf: aBoolean then: aBlock! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildIf: aBooleanExpression then: aStatementList else: anotherStatementList! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildInherited! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildInherited: aDesignator! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildInstVarClause: aVarSection! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildIntNum: aString! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildInterfaceExtending: superclasses! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildInterfaceExtending: superclasses withGuid: aGuid withDecls: ifaceDecls! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildLabel: aLabel! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildLabelDecl: aCollectionOfLabels! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildLeftAssociativeList: head tail: tail! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildLibrary: aLibraryHead withUses: aUsesClause with: declarations! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildLibraryHead: anIdent withDirectives: aCollectionOfDirectives! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildLiteralCharacter: intOrHexNum! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildMethod: aMethodHeading withDirectives: directives! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildMethodDecl: aMethodHeading withBinding: aDirective! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildMethodDelegationOn: anOverridingMethodIdent over: aMethodIdent in: aClassIdent! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildMethodHeading: aSymbol named: anIdent withParams: aParameterSection! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildMethodImpl: aDeclaration withBody: aBlockOfCode! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildNamespaceFileName: anIdent! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildNamespaceFileName: anIdent in: aString! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildNegated: aFactor! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildNegative: aFactor! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildObjectDecl: superclasses withClauses: aCollectionOfDeclarationCollections! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildPackage: aPackageHead requiring: aRequiresClause containing: aContainsClause! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildPackageHead: anIdent! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildParam: identList ofType: anIdent kind: paramType defaultValue: value! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildParams: aCollectionOfParams! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildParams: aCollectionOfParams withResult: aResultType! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildParmType: aSymbol! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildPointerTo: aFactor! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildPointerType! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildPointerType: anIdent! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildProcDecl: heading directives: directives! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildProcHeading: funcOrProc named: anIdent withParams: params! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildProcImpl: aDeclaration withBody: aBlockOfCode! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildProcedureType: aHeading isMethod: aBoolean withDirectives: aCollection! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildProcedureTypeHeading: aSimpleHeading isReference: aBoolean! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildProcedureTypeHeadingSimple: funcOrProc withParams: params! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildProgram: aProgramHead using: aUsesClause block: aBlock! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildProgramHead: name with: spec! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildProperty: anIdent withArray: arrayDefn withType: type withIndex: index withSpecifiers: propertySpecifiers! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildPropertySpecifiers: propertySpecifiers! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildQualifiedIdent: aCollectionOfIdents for: anIdent! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildQualifiedIdent: aRootIdent qualifying: anIdent! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 21:58'!
buildQuotedString: collectionOfChar
	^ OrderedCollection new.! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildRaise: ident! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildRaise: ident at: address! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildRangeFrom: aStartExpression to: anEndExpression! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildReal: aFractionalNum raisedTo: anotherFractionalNum! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildReal: significand raisedTo: exponent withSign: aCharOrOther! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildRecord: recordFields withVariantSection: recordVariantSection! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildRecordField: aCollectionOfIdents ofType: anIdent hinting: aCollectionOfDirectives! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildRecordVariant: recordFields withIndices: expressions! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildRecordVariantSection: anIdent ofType: aType containingUnions: unionDecls! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildRepeat: aStatementList until: aBooleanExpression! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildRequiresClause: namespaceFileNames! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildSet: aSetDescription! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildSetType: aType! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildStatement: aStatement! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildStatement: aStatement labelled: aLabel! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildStringType: anInteger! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildStructType: aStructTypePart isPacked: aBoolean! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildTry: aListOfStatements except: handlers! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildTry: aListOfStatements finally: cleanup! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildType: anIdent isDistinct: aBoolean! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildType: anIdent isDistinct: aBoolean withGenerics: typeArguments! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildType: anIdent  withTypeArguments: typeArguments definedBy: aType withHints: hintDirectives! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildTypeClause: aListOfTypeDeclarations! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildUnit: aUnitHead withInterface: aUnitInterface withImplementation: aUnitImplementation withBlock: block! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildUnitFinalization: someStatements! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildUnitHead: anIdent withDirectives: aCollectionOfDirectives! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildUnitImplementation: aCollectionOfDeclarations using: usesClause! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildUnitInitialization: someStatements withFinalization: aUnitFinalization! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildUnitInterface: aCollectionOfDeclarations using: usesClause! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildUsesClause: aCollection! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildVarDeclaration: aCollectionOfIdents ofType: anIdent havingValue: value hinting: aCollectionOfDirectives! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildVarSection: varType for: aCollectionOfVarDecls! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildVarlikeNode: aSymbol fromNames: aCollectionOfIdents ofType: anIdent hinting: aCollectionOfDirectives! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildVisibility: aSymbol! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildVisibilityClause: aVisibility withDeclarations: decls! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildWhile: aBooleanExpression do: aStatement! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
buildWith: aCollectionOfVars doing: aStatementList! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
changeMarkerTag: anArbitraryNode to: aSymbol! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
changeToClassProperty: aPropertyDecl! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
changeToClassVar: aVarDecl! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
changeToInstVar: aVarDecl! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
constSectionType: aSymbol! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
decimalRepresentationOf: intOrHexNum! !

!PyNodeFactory methodsFor: 'defaults' stamp: 'fbs 1/11/2009 20:54'!
defaultInterfaceSuperclass! !

!PyNodeFactory methodsFor: 'defaults' stamp: 'fbs 5/22/2009 15:12'!
defaultSuperclass! !

!PyNodeFactory methodsFor: 'defaults' stamp: 'fbs 1/11/2009 20:54'!
defaultVisibility! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
functionDeclType: funcOrProc! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
functionTypeDeclType: funcOrProc isMethod: aBoolean! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
isList: aNode! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
methodDeclType: aSymbol! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
namedList: aSymbol withElements: aCollection! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
structType: aBoolean! !

!PyNodeFactory methodsFor: 'building' stamp: 'fbs 1/11/2009 20:54'!
varSectionType: aSymbol! !

PyNodeFactory subclass: #PyObjectNodeFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-Parser'!

!PyObjectNodeFactory commentStamp: 'fbs 5/18/2009 20:28' prior: 0!
I build an AST out of proper objects (subclasses of PyNode).!

!PyObjectNodeFactory methodsFor: 'building' stamp: 'fbs 5/20/2009 12:05'!
buildAbsoluteDirective: identOrExpression
	^ PyDirectiveNode new;
		isAbsolute;
		value: identOrExpression.! !

!PyObjectNodeFactory methodsFor: 'building' stamp: 'fbs 5/21/2009 10:57'!
buildAddressOf: aFactor
	^ PyAddressOfNode referencing: aFactor.! !

!PyObjectNodeFactory methodsFor: 'building' stamp: 'fbs 5/21/2009 11:08'!
buildAddressOfProc: aFactor
	^ PyAddressOfProcNode referencing: aFactor.! !

!PyObjectNodeFactory methodsFor: 'building' stamp: 'fbs 5/21/2009 11:09'!
buildArrayIndex: anExpressionList
	^ PyNamedCollectionNode named: #arrayIndex withElements: anExpressionList.! !

!PyObjectNodeFactory methodsFor: 'building' stamp: 'fbs 5/21/2009 12:03'!
buildArrayType: elementType withIndices: aCollectionOfArrayIndices
	^ PyArrayTypeNode containing: elementType withIndices: aCollectionOfArrayIndices.! !

!PyObjectNodeFactory methodsFor: 'building' stamp: 'fbs 5/21/2009 13:34'!
buildAsmBlock: aStatementList
	^ PyAsmBlockNode with: aStatementList.! !

!PyObjectNodeFactory methodsFor: 'building' stamp: 'fbs 5/19/2009 10:43'!
buildAssign: rValue to: lValue
	^ PyAssignmentNode new
		rvalue: rValue;
		lvalue: lValue;
		yourself.! !

!PyObjectNodeFactory methodsFor: 'building' stamp: 'fbs 5/21/2009 13:39'!
buildBlock: aStatementList
	^ PyBlockNode with: aStatementList.! !

!PyObjectNodeFactory methodsFor: 'building' stamp: 'fbs 5/21/2009 13:44'!
buildBlock: body withDeclarations: aCollection
	^ PyCollectionNode
		with: aCollection
		with: body.! !

!PyObjectNodeFactory methodsFor: 'building' stamp: 'fbs 5/21/2009 14:35'!
buildCaseStatement: ordinalExpression withSwitches: aListOfCaseItems orElse: aBlock
	^ PyCaseStatementNode new
		condition: ordinalExpression;
		switches: aListOfCaseItems;
		defaultSwitch: aBlock.! !

!PyObjectNodeFactory methodsFor: 'building' stamp: 'fbs 5/21/2009 15:32'!
buildCaseSwitchOn: aSet do: aStatement
	^PyCaseSwitchNode new
		values: aSet;
		statement: aStatement;
		yourself.! !

!PyObjectNodeFactory methodsFor: 'building' stamp: 'fbs 5/22/2009 14:53'!
buildClassDecl: superclassNames withClauses: aCollectionOfClauses withOption: aSymbol
	^ PyClassNode new
		superclasses: (superclassNames ifNil: [self defaultSuperclass]);
		clauses: aCollectionOfClauses;
		option: aSymbol;
		yourself.! !

!PyObjectNodeFactory methodsFor: 'building' stamp: 'fbs 5/22/2009 20:56'!
buildClassField: aCollectionOfIdents ofType: anIdent hinting: aDirective
	^ aCollectionOfIdents collect:
		[:each |
		PyInstVarNode new
			name: each name;
			type: anIdent;
			hintingDirective: aDirective;
			yourself.]! !

!PyObjectNodeFactory methodsFor: 'building' stamp: 'fbs 5/25/2009 12:26'!
buildClassHelperFor: anIdent withParent: aClassParent having: declarations
	^ PyClassHelperNode 
		helping: anIdent
		parent: aClassParent
		declarations: declarations.! !

!PyObjectNodeFactory methodsFor: 'building' stamp: 'fbs 5/25/2009 14:24'!
buildClassProperty: aPropertyDesc
	^ PyClassPropertyNode from: aPropertyDesc.! !

!PyObjectNodeFactory methodsFor: 'building' stamp: 'fbs 5/25/2009 14:36'!
buildClassTypeRef: aType
	^ PyClassTypeNode of: aType.! !

!PyObjectNodeFactory methodsFor: 'building' stamp: 'fbs 5/25/2009 19:04'!
buildClassVarClause: aVarSection
	^ aVarSection collect:
		[:each | PyClassVarNode from: each]! !

!PyObjectNodeFactory methodsFor: 'building' stamp: 'fbs 5/25/2009 15:22'!
buildComment: aString
	^ PyCommentNode saying: aString.! !

!PyObjectNodeFactory methodsFor: 'building' stamp: 'fbs 5/25/2009 19:47'!
buildCompoundStatement: aCollection
	^ PyCompoundStatementNode newFrom: aCollection.! !

!PyObjectNodeFactory methodsFor: 'building' stamp: 'fbs 5/19/2009 10:44'!
buildIf: aBoolean then: aBlock
	^ PyIfStatementNode new
		condition: aBoolean;
		ifBlock: aBlock.! !

!PyObjectNodeFactory methodsFor: 'building' stamp: 'fbs 5/19/2009 10:45'!
buildIf: aBooleanExpression then: aStatementList else: anotherStatementList
	^ PyIfStatementNode new
		condition: aBooleanExpression;
		ifBlock: aStatementList;
		elseBlock: anotherStatementList.! !

!PyObjectNodeFactory methodsFor: 'building' stamp: 'fbs 5/25/2009 19:33'!
buildInstVarClause: aVarSection
	^ aVarSection collect:
		[:each | PyInstVarNode from: each]! !

!PyObjectNodeFactory methodsFor: 'building' stamp: 'fbs 5/25/2009 19:37'!
buildIntNum: aString
	^ PyIntegerNode value: aString.! !

!PyObjectNodeFactory methodsFor: 'building' stamp: 'fbs 5/18/2009 20:29'!
buildStatement: aStatement
	^ aStatement! !

!PyObjectNodeFactory methodsFor: 'building' stamp: 'fbs 5/18/2009 20:29'!
buildStatement: aStatement labelled: aLabel
	^ (aStatement copy) label: aLabel.! !

!PyObjectNodeFactory methodsFor: 'building' stamp: 'fbs 5/19/2009 15:14'!
buildUnit: aUnitHead withInterface: aUnitInterface withImplementation: aUnitImplementation withBlock: block
	^ PyUnitNode new
		head: aUnitHead;
		interface: aUnitInterface;
		implementation: aUnitImplementation;
		code: block.! !

!PyObjectNodeFactory methodsFor: 'building' stamp: 'fbs 5/19/2009 15:49'!
buildUnitInterface: aCollectionOfDeclarations using: usesClause
	^ PyUnitInterfaceNode new
		declarations: aCollectionOfDeclarations;
		usedUnits: usesClause;
		yourself.! !

!PyObjectNodeFactory methodsFor: 'defaults' stamp: 'fbs 5/22/2009 15:12'!
defaultSuperclass
	^ PyCollectionNode with: (self buildIdent: 'TObject').! !

PyNodeFactory subclass: #PySexprNodeFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-Parser'!

!PySexprNodeFactory commentStamp: 'fbs 1/11/2009 21:34' prior: 0!
The AST built using this Factory is a big s-expression. Lead elements of a list define what kind of node the list represents, or a list is a list of nodes.!

!PySexprNodeFactory methodsFor: 'private' stamp: 'fbs 1/5/2009 21:27'!
addVisibility: aVisibility to: aNode
	^ (aNode copy)
		add: aVisibility copy;
		yourself.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/18/2008 15:28'!
buildAbsoluteDirective: identOrExpression
	^ OrderedCollection with: #absolute with: identOrExpression.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 11/30/2008 20:53'!
buildAddressOf: aFactor
	^ OrderedCollection
		with: #addressOf
		with: aFactor.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/12/2008 21:29'!
buildAddressOfProc: aFactor
	^ OrderedCollection
		with: #addressOfProc
		with: aFactor.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/3/2009 13:53'!
buildArrayIndex: anExpressionList
	^ self namedList: #arrayIndex withElements: anExpressionList.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/3/2009 13:53'!
buildArrayType: elementType withIndices: aCollectionOfArrayIndices
	^ OrderedCollection
		with: #arrayType
		with: elementType
		with: aCollectionOfArrayIndices! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/3/2009 13:53'!
buildAsmBlock: aStatementList
	^ self namedList: #asm withElements: aStatementList.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/3/2009 13:53'!
buildAssign: rValue to: lValue
	^ OrderedCollection
		with: #assign
		with: lValue
		with: rValue.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/3/2009 13:52'!
buildBlock: aStatementList
	^ self namedList: #block withElements: aStatementList.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/30/2008 20:26'!
buildBlock: body withDeclarations: aCollection
	^ OrderedCollection
		with: aCollection
		with: (body ifNil: [self buildBlock: #()]).! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/30/2008 13:36'!
buildCaseStatement: ordinalExpression withSwitches: aListOfCaseItems orElse: aBlock
	^ OrderedCollection
		with: #case
		with: ordinalExpression
		with: aListOfCaseItems
		with: aBlock.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/3/2009 13:54'!
buildCaseSwitchOn: aSet do: aStatement
	^ OrderedCollection
		with: #switch
		with: aSet
		with: aStatement.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/2/2009 22:01'!
buildClassDecl: superclassNames withClauses: aCollectionOfClauses withOption: aSymbol
	"The classBody production returns a collection of member declarations. Thus aCollectionOfClauses is a collection of collections of member declarations."
	| result |
	result := OrderedCollection
		with: #class
		with: (superclassNames ifNil: [self defaultSuperclass])
		with: OrderedCollection new.
		
	aCollectionOfClauses do: [:clause | result last addAll: clause].
	
	aSymbol ifNotNil: [result add: aSymbol].

	^ result.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/2/2009 22:37'!
buildClassField: aCollectionOfIdents ofType: anIdent hinting: aDirective
	^ self
		buildVarlikeNode: #instvar
		fromNames: aCollectionOfIdents
		ofType: anIdent
		hinting: (aDirective ifNotNil: [OrderedCollection with: aDirective]).! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/1/2009 23:45'!
buildClassHelperFor: anIdent withParent: aClassParent having: declarations
	^ OrderedCollection
		with: #classHelper
		with: anIdent
		with: (aClassParent ifNil: [OrderedCollection new])
		with: declarations.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/3/2009 21:13'!
buildClassProperty: aPropertyDesc
	^ self changeToClassProperty: aPropertyDesc.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/31/2008 14:04'!
buildClassTypeRef: aType
	^ OrderedCollection with: #classType with: aType.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 5/25/2009 15:41'!
buildClassVarClause: aVarSection
	"Like #buildInstVarClause, this returns an _unnamed_ collection of classvar declarations."
	^ (OrderedCollection new)
		addAll: (aVarSection allButFirst collect:
			[:each | self changeToClassVar: each]);
		yourself.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/6/2009 21:14'!
buildComment: aString
	^ OrderedCollection
		with: #comment
		with: aString.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/8/2008 22:32'!
buildCompoundStatement: aCollection
	^ (OrderedCollection with: #block)
		addAll: aCollection;
		yourself.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/28/2008 23:17'!
buildConst: anIdent ofType: aType withValue: aConstantExpression
	^ OrderedCollection
		with: #const
		with: anIdent
		with: (aType ifNil: [OrderedCollection new])
		with: aConstantExpression.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/4/2009 20:35'!
buildConst: anIdent ofType: aType withValue: aConstantExpression hinting: directives
	^ OrderedCollection
		with: #const
		with: anIdent
		with: (aType ifNil: [OrderedCollection new])
		with: aConstantExpression
		with: (self buildDirectives: (directives ifNil: [#()])).! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/4/2009 14:31'!
buildConstRecord: anIdent value: aConstExpression
	^ OrderedCollection
		with: anIdent
		with: aConstExpression.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/3/2009 13:52'!
buildConstSection: constDeclarations
	^ self namedList: #consts withElements: constDeclarations.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/8/2009 13:47'!
buildConstSection: constDeclarations named: aSymbol
	^ self namedList: (self constSectionType: aSymbol) withElements: constDeclarations.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/4/2009 23:26'!
buildContainsClause: namespaceFileNames
	^ self namedList: #contains withElements: namespaceFileNames.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/12/2008 23:50'!
buildControlString: anIntNumOrHexNum
	^ (Character codePoint: ((anIntNumOrHexNum first == #intNum)
		ifTrue: [anIntNumOrHexNum second asInteger]
		ifFalse: [Integer readFrom: anIntNumOrHexNum second readStream base: 16])) asString! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 11/28/2008 21:39'!
buildDecimalFraction: integralPart with: fractionalPart
	^ OrderedCollection
		with: #real
		with: integralPart second, '.', fractionalPart second! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/3/2008 22:48'!
buildDereference
	^ #dereference.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/7/2009 12:21'!
buildDesignator: head with: tail
	"head is an ident, and tail a collection of idents."
	| o |
	o := OrderedCollection with: #designator.
	head ifNotNil: [o add: head].
	o addAll: tail.
	
	^ o.
		! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/7/2009 22:50'!
buildDesignator: ident withParts: designatorParts
	"ident is an ident or designator; designatorParts is a collection of things like dereferences, array accesses, and the like."
	| result |
	designatorParts ifNil: [^ident].

	result := ident.
	designatorParts do:
		[:each |
		(each = self buildDereference)
			ifTrue: [result := OrderedCollection with: each with: result].

		(each isCollection and: [each first = #arrayIndex])
			ifTrue:
				[result := (OrderedCollection with: #arrayIndex with: result)
					addAll: each allButFirst;
					yourself].
		(each isCollection and: [each first = #args])
			ifTrue:
				[result := (OrderedCollection with: #funcall with: result with: each)]].
	^ result.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/19/2008 23:16'!
buildDirective: aString
	^ OrderedCollection with: #directive with: aString.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/19/2008 23:17'!
buildDirective: aString withValue: anExpression
	^ OrderedCollection with: #directive with: aString with: anExpression.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/3/2009 13:49'!
buildDirectives: aCollectionOfDirectives
	^ self namedList: #directives withElements: aCollectionOfDirectives.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 11/26/2008 14:30'!
buildDot: ident
	^ OrderedCollection
		with: #dot
		with: ident.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/18/2008 12:02'!
buildElseBlock: aStatementList
	^ (OrderedCollection with: #elseBlock)
		addAll: aStatementList;
		yourself.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/3/2009 13:52'!
buildEnum: aListOfValuedIdents
	| result |
	result := OrderedCollection with: #enum.
	
	result addAll:
		(aListOfValuedIdents withIndexCollect:
			[:each :i |
			"Enumerated type values have default ordinality equal to their (zero-based) position in the declaration."
			OrderedCollection with: each first with: (each second ifNil: [self buildIntNum: (i - 1) asString])]).
	
	^ result.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/29/2008 23:26'!
buildEnumValue: anIdent withOrdinality: anExpression
	^ OrderedCollection with: anIdent with: anExpression.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/5/2009 10:17'!
buildExportItem: name withParams: params withDirectives: aCollectionOfDirectives
	^ OrderedCollection
		with: #export
		with: name
		with: (self buildParams: (params ifNil: [#()]))
		with: (self buildDirectives: aCollectionOfDirectives).! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/5/2009 08:55'!
buildExportsClause: aCollectionOfExportItems
	^ self namedList: #exports withElements: aCollectionOfExportItems.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 5/11/2009 10:23'!
buildExpression: anExpression nextOperator: anOperator nextOperand: aSimpleExpression
	^ OrderedCollection
		with: anOperator
		with: anExpression
		with: aSimpleExpression! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/22/2008 11:28'!
buildExternalDirective: anIdent andSpecifiers: aCollectionOfDirectives
	^ (self buildDirective: anIdent)
		addAll: aCollectionOfDirectives;
		yourself.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/22/2008 11:27'!
buildExternalDirective: anIdent withValue: anExpression andSpecifiers: aCollectionOfDirectives
	^ (self buildDirective: anIdent withValue: anExpression)
		addAll: aCollectionOfDirectives;
		yourself.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/22/2008 11:38'!
buildExternalSpecifier: aNameOrIndex withValue: anExpression
	^ OrderedCollection
		with: aNameOrIndex asLowercase asSymbol
		with: anExpression.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/31/2008 14:04'!
buildFileType: aType
	^ OrderedCollection
		with: #fileType
		with: (aType ifNil: [OrderedCollection new])! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/9/2008 23:05'!
buildFor: anIdent over: aRange do: aStatement
	^ OrderedCollection with: #for with: anIdent with: aRange with: aStatement! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/9/2008 23:13'!
buildForRangeFrom: aStartExpression to: anEndExpression
	^ OrderedCollection with: #range with: aStartExpression with: anEndExpression.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/2/2009 13:56'!
buildForwardClassDecl
	^ OrderedCollection with: #forwardClass! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/7/2009 12:32'!
buildFunctionCallArgument: argument
	^ argument.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/7/2009 12:56'!
buildFunctionCallArguments: arguments
	^ self namedList: #args withElements: arguments.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/23/2008 22:56'!
buildFunctionResult: aType
	^ OrderedCollection with: #result with: aType.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/29/2008 22:00'!
buildGenericConstruct: aListOfTypes
	^ (OrderedCollection with: #typeArguments)
		addAll: aListOfTypes;
		yourself.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/9/2008 22:44'!
buildGoto: aLabel
	^ OrderedCollection with: #goto with: aLabel.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 11/26/2008 14:46'!
buildGuid: aString
	^ OrderedCollection
		with: #guid
		with: aString withoutQuoting.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/7/2009 20:04'!
buildHandler: anIdent for: aType doing: aStatement
	^ OrderedCollection
		with: #exceptionHandler
		with: (anIdent ifNil: [OrderedCollection new])
		with: aType
		with: (aStatement ifNil: [OrderedCollection new]).! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 11/28/2008 22:01'!
buildHexNum: aString
	^ OrderedCollection
		with: #hexNum
		with: aString! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 5/21/2009 10:41'!
buildIdent: aString
	^ OrderedCollection
		with: #ident
		with: aString.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/9/2008 08:53'!
buildIf: aBoolean then: aBlock
	^ self buildIf: aBoolean then: aBlock else: OrderedCollection new.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/9/2008 08:55'!
buildIf: aBooleanExpression then: aStatementList else: anotherStatementList
	^ OrderedCollection new
		add: #if;
		add: aBooleanExpression;
		add: aStatementList;
		add: anotherStatementList;
		yourself.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/8/2009 16:20'!
buildInherited
	^ OrderedCollection 
		with: #inherited.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/3/2008 23:04'!
buildInherited: aDesignator
	^ OrderedCollection 
		with: #inherited
		with: aDesignator.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/2/2009 13:19'!
buildInstVarClause: aVarSection
	"A var clause inside a class is exactly the same as any field declaration. Usually, field declarations must occur before any other kind of declaration; the var clause simply allows one to declare other things (methods, properties) before the field declarations."
	^ (OrderedCollection new)
		addAll: (aVarSection allButFirst collect:
			[:each | self changeToInstVar: each]);
		yourself.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 11/28/2008 21:29'!
buildIntNum: aString
	^ OrderedCollection
		with: #intNum
		with: aString.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/5/2009 21:43'!
buildInterfaceExtending: superclasses
	^ self buildInterfaceExtending: superclasses withGuid: nil withDecls: nil! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/5/2009 21:43'!
buildInterfaceExtending: superclasses withGuid: aGuid withDecls: ifaceDecls
	^ OrderedCollection
		with: #interface
		with: (superclasses ifNil: [self defaultInterfaceSuperclass])
		with: (aGuid ifNil: [OrderedCollection new])
		with: (ifaceDecls ifNil: [OrderedCollection new]).! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/7/2008 15:18'!
buildLabel: aLabel
	^ OrderedCollection with: #label with: aLabel.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/20/2008 22:30'!
buildLabelDecl: aCollectionOfLabels
	^ (OrderedCollection with: #labels)
		addAll: aCollectionOfLabels;
		yourself.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/9/2008 20:26'!
buildLeftAssociativeList: head tail: tail
	"Given a left-most operand and a sequence of operators and operands, return a tree of operations treating the operand as right-associative. For example, 1 + 2 + 3 + 4 = 1 + (2 + (3 + 4))."
	"tail contains a collection of pairs of objects (operator operand) where operand is of the same type as head."
	^ tail
		ifEmpty: [head]
		ifNotEmpty:
			[tail
				inject: head asOrderedCollection
				into: [:result :next |
					result := {next first. result. next second.} asOrderedCollection]]! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/30/2008 20:06'!
buildLibrary: aLibraryHead withUses: aUsesClause with: declarations
	^ (aLibraryHead copy)
		add: (aUsesClause ifNil: [self buildUsesClause: #()]);
		addAll: declarations;
		yourself.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/30/2008 14:29'!
buildLibraryHead: anIdent withDirectives: aCollectionOfDirectives
	^ OrderedCollection
		with: #library
		with: anIdent
		with: (self buildDirectives: aCollectionOfDirectives).! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 11/28/2008 22:03'!
buildLiteralCharacter: intOrHexNum
	^ OrderedCollection
		with: #char
		with: (self decimalRepresentationOf: intOrHexNum)! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/7/2009 09:49'!
buildMethod: aMethodHeading withDirectives: directives
	^ aMethodHeading copy
		add: (self buildDirectives: directives);
		yourself.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/4/2009 22:03'!
buildMethodDecl: aMethodHeading withBinding: aDirective
	^ (aMethodHeading copy)
		add: (self buildDirectives: (aDirective ifNil: [#()] ifNotNil: [{aDirective.}]));
		yourself.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/9/2009 12:52'!
buildMethodDelegationOn: anOverridingMethodIdent over: aMethodIdent in: aClassIdent
	^ OrderedCollection
		with: #methodDelegation
		with: anOverridingMethodIdent
		with: aMethodIdent
		with: aClassIdent.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/4/2009 22:43'!
buildMethodHeading: aSymbol named: anIdent withParams: aParameterSection
	^ OrderedCollection
		with: (self methodDeclType: aSymbol)
		with: anIdent
		with: aParameterSection.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/4/2009 22:42'!
buildMethodImpl: aDeclaration withBody: aBlockOfCode
	^ (OrderedCollection
		with: #methodImpl
		with: aDeclaration)
		addAll: aBlockOfCode;
		yourself.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/14/2008 21:56'!
buildNamespaceFileName: anIdent
	^ anIdent.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/14/2008 22:07'!
buildNamespaceFileName: anIdent in: aString
	^ OrderedCollection
		with: #unitFile
		with: anIdent
		with: (OrderedCollection with: #fileName with: aString).! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 11/30/2008 23:55'!
buildNegated: aFactor
	^ OrderedCollection
		with: #not
		with: aFactor.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/4/2008 22:55'!
buildNegative: aFactor
	^ OrderedCollection
		with: #-
		with: aFactor.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/5/2009 16:31'!
buildObjectDecl: superclasses withClauses: aCollectionOfDeclarationCollections
	^ (OrderedCollection with: #objectType)
		addAll: (self buildClassDecl: superclasses withClauses: aCollectionOfDeclarationCollections withOption: nil) allButFirst;
		yourself.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/4/2009 23:37'!
buildPackage: aPackageHead requiring: aRequiresClause containing: aContainsClause
	^ (aPackageHead copy)
		add: (aRequiresClause ifNil: [self buildRequiresClause: #()]);
		add: (aContainsClause ifNil: [self buildContainsClause: #()]);
		yourself.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/4/2009 23:00'!
buildPackageHead: anIdent
	^ OrderedCollection
		with: #package
		with: anIdent.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/9/2009 09:58'!
buildParam: identList ofType: anIdent kind: paramType defaultValue: value
	"This returns a LIST of parameters."
	^ identList collect:
		[:each| |param|
		param :=
			OrderedCollection
			with: #param
			with: each
			with: (anIdent ifNil: [OrderedCollection new]).
					
		paramType ifNotNil: [param add: paramType].
		value ifNotNil: [param add: value].
		param]
! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/5/2009 10:06'!
buildParams: aCollectionOfParams
	^ self buildParams: aCollectionOfParams withResult: nil.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/23/2008 22:54'!
buildParams: aCollectionOfParams withResult: aResultType
	| result |
	result := OrderedCollection with: #params.
	
	aCollectionOfParams ifNotNil:
		[result 
			addAll:
				(aCollectionOfParams
					inject: OrderedCollection new
					into:
						[:acc :each | acc addAll: each; yourself])].
		
	aResultType ifNotNil: [result add: aResultType].
	
	^ result.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/19/2008 21:15'!
buildParmType: aSymbol
	^ OrderedCollection with: #paramType with: aSymbol.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 11/30/2008 23:49'!
buildPointerTo: aFactor
	^ OrderedCollection
		with: #pointerTo
		with: aFactor.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/29/2008 20:59'!
buildPointerType
	^ self buildPointerType: #pointer.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/29/2008 20:59'!
buildPointerType: anIdent
	^ OrderedCollection with: #pointerType with: anIdent.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/20/2008 22:45'!
buildProcDecl: heading directives: directives
	^ heading copy
		add: (self buildDirectives: directives);
		yourself.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/20/2008 20:41'!
buildProcHeading: funcOrProc named: anIdent withParams: params
	^ (OrderedCollection
		with: (self functionDeclType: funcOrProc)
		with: anIdent)
		add: (params ifNil: [self buildParams: #()]);
		yourself.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/30/2008 20:09'!
buildProcImpl: aDeclaration withBody: aBlockOfCode
	^ (OrderedCollection
		with: #procedureImpl
		with: aDeclaration)
		addAll: aBlockOfCode;
		yourself.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/29/2008 16:35'!
buildProcedureType: aHeading isMethod: aBoolean withDirectives: aCollection
	| result |
	result := OrderedCollection with: (self functionTypeDeclType: aHeading first first isMethod: aBoolean).
	
	result addAll: aHeading first allButFirst.
	result add: (self buildDirectives: aCollection).
	
	result addAll: aHeading allButFirst.
	
	^ result.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/29/2008 16:16'!
buildProcedureTypeHeading: aSimpleHeading isReference: aBoolean
	| result |
	result := OrderedCollection with: aSimpleHeading.
	
	aBoolean ifTrue: [result add: #reference].
	
	^ result.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/29/2008 16:16'!
buildProcedureTypeHeadingSimple: funcOrProc withParams: params
	^ OrderedCollection with: funcOrProc with: params.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/30/2008 20:07'!
buildProgram: aProgramHead using: aUsesClause block: aBlock
	| result |
	result := (aProgramHead ifNil: [OrderedCollection with: #program]) copy asOrderedCollection.
	aUsesClause ifNotEmpty: [result add: aUsesClause].
	result addAll: aBlock.
	^ result.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/14/2008 20:38'!
buildProgramHead: name with: spec
	| result |
	result := OrderedCollection with: #program with: name.
	spec ifNotEmpty: [result add: spec].
	^ result.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/9/2009 06:22'!
buildProperty: anIdent withArray: arrayDefn withType: type withIndex: index withSpecifiers: propertySpecifiers
	| directives |
	directives := propertySpecifiers copy.
	index ifNotNil: [directives addFirst: index].
	
	^ OrderedCollection
		with: #property
		with: anIdent
		with: (arrayDefn ifNil: [OrderedCollection new])
		with: (type ifNil: [OrderedCollection new])
		with: (self buildPropertySpecifiers: directives).! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/3/2009 13:55'!
buildPropertySpecifiers: propertySpecifiers
	^ self namedList: #propertySpecifiers withElements: propertySpecifiers.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/4/2009 23:06'!
buildQualifiedIdent: aCollectionOfIdents for: anIdent
	"This builds a tree representation treating the dot 'operator' as a right-associative operator. Perhaps one actually wants a _left_-associative operator. We'll wait until we have consumers of this AST before we decide."
	^ (aCollectionOfIdents reverse
		inject: anIdent
		into: [:result :each | result := {#dot. each. result.}]) asOrderedCollection! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/7/2009 23:05'!
buildQualifiedIdent: aRootIdent qualifying: anIdent
	^ OrderedCollection new
		add: #dot;
		add: aRootIdent;
		add: anIdent;
		yourself.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/13/2008 21:39'!
buildQuotedString: collectionOfChar
	^ String withAll: collectionOfChar! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/13/2008 22:03'!
buildRaise: ident
	^ self buildRaise: ident at: nil.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/13/2008 22:08'!
buildRaise: ident at: address
	| result |
	result := OrderedCollection with: #raise.
	
	ident ifNotNil: [result add: ident].
	address ifNotNil: [result add: address].
	
	^ result.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/10/2008 21:39'!
buildRangeFrom: aStartExpression to: anEndExpression
	^ OrderedCollection with: #range with: aStartExpression with: anEndExpression.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 11/28/2008 21:44'!
buildReal: aFractionalNum raisedTo: anotherFractionalNum
	^ OrderedCollection
		with: #real
		with: aFractionalNum second, 'e', anotherFractionalNum second.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 11/28/2008 21:48'!
buildReal: significand raisedTo: exponent withSign: aCharOrOther
	^ OrderedCollection
		with: #real
		with: (significand second, 'e', ((aCharOrOther isCharacter) ifTrue: [aCharOrOther asString] ifFalse: ['']), exponent second)! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/3/2009 22:32'!
buildRecord: recordFields withVariantSection: recordVariantSection
	"The recordField production returns a list of record fields. The other legal productions just return one declaration each. Thus recordFields is a list of elements that are either nodes, or lists of nodes."
	| result |
	result := OrderedCollection with: #record with: OrderedCollection new.

	recordFields do:
		[:each |
		(self isList: each)
			ifTrue: [result last addAll: each]
			ifFalse: [result last add: each]].
		
	^ result.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/2/2009 22:33'!
buildRecordField: aCollectionOfIdents ofType: anIdent hinting: aCollectionOfDirectives
	^ self
		buildVarlikeNode: #recordField
		fromNames: aCollectionOfIdents
		ofType: anIdent
		hinting: aCollectionOfDirectives.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/4/2009 13:58'!
buildRecordVariant: recordFields withIndices: expressions
	| result |
	result := OrderedCollection
		with: #recordVariant
		with: (self buildSet: expressions)
		with: OrderedCollection new.
		
	recordFields do: [:each | result last addAll: each].
	
	^ result.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/4/2009 14:07'!
buildRecordVariantSection: anIdent ofType: aType containingUnions: unionDecls
	^ OrderedCollection
		with: #recordVariants
		with: (anIdent ifNil: [OrderedCollection new])
		with: aType
		with: unionDecls.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/9/2008 21:58'!
buildRepeat: aStatementList until: aBooleanExpression
	^ OrderedCollection with: #repeat with: aStatementList with: aBooleanExpression.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/4/2009 23:27'!
buildRequiresClause: namespaceFileNames
	^ self namedList: #requires withElements: namespaceFileNames.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/10/2008 21:23'!
buildSet: aSetDescription
	^ (OrderedCollection with: #set)
		addAll: aSetDescription;
		yourself.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/29/2008 21:38'!
buildSetType: aType
	^ OrderedCollection with: #setType with: aType.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/7/2008 15:21'!
buildStatement: aStatement
	^ aStatement! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/7/2008 20:37'!
buildStatement: aStatement labelled: aLabel
	^ (aStatement copy) add: aLabel; yourself.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/29/2008 21:04'!
buildStringType: anInteger
	^ OrderedCollection with: #shortStringType with: anInteger.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/29/2008 21:34'!
buildStructType: aStructTypePart isPacked: aBoolean
	^ aBoolean
		ifTrue: [OrderedCollection with: #packed with: aStructTypePart]
		ifFalse: [aStructTypePart].! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/18/2008 12:00'!
buildTry: aListOfStatements except: handlers
	^ OrderedCollection with: #tryExcept with: aListOfStatements with: handlers.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/15/2008 23:26'!
buildTry: aListOfStatements finally: cleanup
	^ OrderedCollection with: #tryFinally with: aListOfStatements with: cleanup.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/29/2008 21:45'!
buildType: anIdent isDistinct: aBoolean
	^ aBoolean
		ifTrue: [OrderedCollection with: #typeOf with: anIdent]
		ifFalse: [anIdent]! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/31/2008 13:24'!
buildType: anIdent isDistinct: aBoolean withGenerics: typeArguments
	^ self
		buildType: (typeArguments
			ifNil: [anIdent]
			ifNotNil: [OrderedCollection with: #genericType with: anIdent with: typeArguments])
		isDistinct: aBoolean.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 5/21/2009 13:02'!
buildType: anIdent withTypeArguments: typeArguments definedBy: aType withHints: hintDirectives
	^ OrderedCollection
		with: #type
		with: anIdent
		with: (typeArguments ifNil: [self buildGenericConstruct: #()])
		with: aType
		with: (self buildDirectives: hintDirectives).! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/3/2009 13:50'!
buildTypeClause: aListOfTypeDeclarations
	^ self namedList: #types withElements: aListOfTypeDeclarations.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/27/2008 20:03'!
buildUnit: aUnitHead withInterface: aUnitInterface withImplementation: aUnitImplementation withBlock: block
	^ (aUnitHead copy)
		add: aUnitInterface;
		add: aUnitImplementation;
		add: block;
		yourself.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/28/2008 22:02'!
buildUnitFinalization: someStatements
	^ OrderedCollection with: #unitFinalization with: someStatements.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/30/2008 14:29'!
buildUnitHead: anIdent withDirectives: aCollectionOfDirectives
	^ OrderedCollection
		with: #unit
		with: anIdent
		with: (self buildDirectives: aCollectionOfDirectives).! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/27/2008 16:58'!
buildUnitImplementation: aCollectionOfDeclarations using: usesClause
	^ OrderedCollection
		with: #unitImplementation
		with: (usesClause ifNil: [self buildUsesClause: #()])
		with: aCollectionOfDeclarations.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/28/2008 21:44'!
buildUnitInitialization: someStatements withFinalization: aUnitFinalization
	| result |
	result := OrderedCollection with: #unitInitialization with: someStatements.
	
	aUnitFinalization ifNotNil: [result add: aUnitFinalization].
	^ result.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/27/2008 16:58'!
buildUnitInterface: aCollectionOfDeclarations using: usesClause
	^ OrderedCollection
		with: #unitInterface
		with: (usesClause ifNil: [self buildUsesClause: #()])
		with: aCollectionOfDeclarations.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/3/2009 13:50'!
buildUsesClause: aCollection
	^ self namedList: #uses withElements: aCollection.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/30/2008 22:56'!
buildVarDeclaration: aCollectionOfIdents ofType: anIdent havingValue: value hinting: aCollectionOfDirectives
	^ aCollectionOfIdents collect:
		[:each |
		OrderedCollection
			with: #var
			with: each
			with: anIdent
			with: (value ifNil: [OrderedCollection new])
			with: (self buildDirectives: (aCollectionOfDirectives ifNil: [#()]))].! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/31/2008 10:16'!
buildVarSection: varType for: aCollectionOfVarDecls
	| result |
	result := OrderedCollection with: (self varSectionType: varType).
	
	aCollectionOfVarDecls do: [:each | result addAll: each].
	
	^ result.! !

!PySexprNodeFactory methodsFor: 'private' stamp: 'fbs 1/2/2009 22:33'!
buildVarlikeNode: aSymbol fromNames: aCollectionOfIdents ofType: anIdent hinting: aCollectionOfDirectives
	^ aCollectionOfIdents collect:
		[:each |
		OrderedCollection
			with: aSymbol
			with: each
			with: anIdent
			with: (self buildDirectives: (aCollectionOfDirectives ifNil: [#()]))].! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/2/2009 00:20'!
buildVisibility: aSymbol
	^ OrderedCollection with: #visibility with: aSymbol.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/7/2009 12:46'!
buildVisibilityClause: aVisibility withDeclarations: decls
	"Some declaration productions produce collections of declarations (like classField), while most produce just one declaration. Thus each element of decls could be one or many actual declarations."
	| result vis |
	vis := aVisibility ifNil: [self defaultVisibility].
	result := OrderedCollection new.
	decls do:
		[:each |
		(self isList: each)
			ifFalse: [result add: (self addVisibility: vis to: each)]
			ifTrue: [result addAll:
						(each collect:
							[:e | self addVisibility: vis to: e])]].
	^ result.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/9/2008 22:14'!
buildWhile: aBooleanExpression do: aStatement
	^ OrderedCollection with: #while with: aBooleanExpression with: aStatement.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 12/9/2008 20:35'!
buildWith: aCollectionOfVars doing: aStatementList
	^ (OrderedCollection with: #with)
		add: aCollectionOfVars;
		add: aStatementList;
		yourself.! !

!PySexprNodeFactory methodsFor: 'node converting' stamp: 'fbs 1/3/2009 21:11'!
changeMarkerTag: anArbitraryNode to: aSymbol
	^ (OrderedCollection with: aSymbol)
		addAll: anArbitraryNode allButFirst;
		yourself.! !

!PySexprNodeFactory methodsFor: 'node converting' stamp: 'fbs 1/3/2009 21:15'!
changeToClassProperty: aPropertyDecl
	^ self changeMarkerTag: aPropertyDecl to: #classProperty.! !

!PySexprNodeFactory methodsFor: 'node converting' stamp: 'fbs 1/3/2009 21:12'!
changeToClassVar: aVarDecl
	^ self changeMarkerTag: aVarDecl to: #classvar.! !

!PySexprNodeFactory methodsFor: 'node converting' stamp: 'fbs 1/3/2009 21:12'!
changeToInstVar: aVarDecl
	^ self changeMarkerTag: aVarDecl to: #instvar.! !

!PySexprNodeFactory methodsFor: 'symbols' stamp: 'fbs 1/8/2009 13:47'!
constSectionType: aSymbol
	^ (aSymbol = #const)
		ifTrue: [#consts]
		ifFalse: [#resourceStrings].! !

!PySexprNodeFactory methodsFor: 'private' stamp: 'fbs 11/28/2008 22:05'!
decimalRepresentationOf: intOrHexNum
	^ (intOrHexNum first = #intNum)
		ifTrue: [intOrHexNum second]
		ifFalse: [(Integer readFrom: intOrHexNum second asUppercase readStream base: 16) asString]! !

!PySexprNodeFactory methodsFor: 'defaults' stamp: 'fbs 1/5/2009 15:40'!
defaultInterfaceSuperclass
	^ OrderedCollection with: (self buildIdent: 'IInterface').! !

!PySexprNodeFactory methodsFor: 'defaults' stamp: 'fbs 5/22/2009 15:12'!
defaultSuperclass
	^ OrderedCollection with: (self buildIdent: 'TObject').! !

!PySexprNodeFactory methodsFor: 'defaults' stamp: 'fbs 1/2/2009 00:19'!
defaultVisibility
	^ self buildVisibility: #published.! !

!PySexprNodeFactory methodsFor: 'symbols' stamp: 'fbs 12/19/2008 23:07'!
functionDeclType: funcOrProc
	^ (funcOrProc = #function)
		ifTrue: [#functionDecl]
		ifFalse: [#procedureDecl].! !

!PySexprNodeFactory methodsFor: 'symbols' stamp: 'fbs 12/29/2008 16:19'!
functionTypeDeclType: funcOrProc isMethod: aBoolean
	^ (funcOrProc = #function)
		ifTrue: [aBoolean
			ifTrue: [#functionMethodType]
			ifFalse: [#functionType]]
		ifFalse: [aBoolean
			ifTrue: [#procedureMethodType]
			ifFalse: [#procedureType]]! !

!PySexprNodeFactory methodsFor: 'testing' stamp: 'fbs 1/11/2009 21:37'!
isList: aNode
	"Return true if aNode represents a list of nodes in the AST. Thus #foo, 'foo', #(params) and ('foo') are not lists, while #() and #((param (ident 'Foo')) (param (ident 'Bar'))) are."
	^ aNode isCollection
		and: [aNode isEmpty or: [aNode first isCollection and: [aNode first isString not]]]! !

!PySexprNodeFactory methodsFor: 'symbols' stamp: 'fbs 1/4/2009 21:39'!
methodDeclType: aSymbol
	"See the methodKey, procKey productions for the source of valid values for aSymbol."

	(aSymbol = #function) ifTrue: [^ #functionDecl].
	(aSymbol = #procedure) ifTrue: [^ #procedureDecl].
	(aSymbol = #functionMethod) ifTrue: [^ #functionMethodDecl].
	(aSymbol = #procedureMethod) ifTrue: [^ #procedureMethodDecl].
	(aSymbol = #classFunctionMethod) ifTrue: [^ #classFunctionMethodDecl].
	(aSymbol = #classProcedureMethod) ifTrue: [^ #classProcedureMethodDecl].
	(aSymbol = #constructor) ifTrue: [^ #constructorDecl].
	(aSymbol = #destructor) ifTrue: [^ #destructorDecl].
	(aSymbol = #classConstructor) ifTrue: [^ #classConstructorDecl].
	
	^ #operatorDecl.! !

!PySexprNodeFactory methodsFor: 'building' stamp: 'fbs 1/3/2009 13:49'!
namedList: aSymbol withElements: aCollection
	^ (OrderedCollection with: aSymbol)
		addAll: aCollection;
		yourself.! !

!PySexprNodeFactory methodsFor: 'symbols' stamp: 'fbs 12/29/2008 21:31'!
structType: aBoolean
	^ aBoolean
		ifTrue: [#packedRecord]
		ifFalse: [#record].! !

!PySexprNodeFactory methodsFor: 'symbols' stamp: 'fbs 12/30/2008 22:58'!
varSectionType: aSymbol
	^ (aSymbol = #var)
		ifTrue: [#vars]
		ifFalse: [#threadVars].! !

Object subclass: #Pythia
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pythia-Parser'!

!Pythia commentStamp: 'fbs 1/9/2009 17:56' prior: 0!
Copyright (c) 2009 Frank Shearar

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.!
PyParser initialize!
